/*******************************************************************************
 **
 ** $Id: download.C 177 2009-09-19 08:53:42Z AndrewLahiff $
 **
 ******************************************************************************
 **
 ** Copyright (C) 2007-2008 Andrew Lahiff.
 **
 ** This file is part of PEACEmoments CAA.
 **
 ** PEACEmoments CAA is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** PEACEmoments CAA is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with PEACEmoments CAA.  If not, see <http://www.gnu.org/licenses/>.
 **
 *******************************************************************************/

#include "peacemoments_caa.h"
#include <QDebug>

//-----------------------------------------------------------------------------
//
// prepare request
//
//-----------------------------------------------------------------------------

int downloadDataPrep(QString scnStr,
                     QString filePeace1,
                     QString filePeace2,
                     QString filePotential,
                     QString fileAuxGSE,
                     QString fileMagField,
                     QString timeRange,
                     QString userName, QString passWord,
                     QString *request) 
{
	
	int returnValue = 0;
	
	QString datasetFilePeace1, datasetFilePeace2, datasetFilePotential,
	        datasetFileAuxGSE, datasetFileMagField;

// determine the spacecraft number

   int scn = 0;
   if (scnStr == "C1") scn = 1;
   else if (scnStr == "C2") scn = 2;
   else if (scnStr == "C3") scn = 3;
   else if (scnStr == "C4") scn = 4;

   if (scn >= 1 && scn <= 4)
   {  
   }
   else return(-1);

// determine the PEACE sensor 1 dataset ID

   if (filePeace1 == "3DRL")
   {
      datasetFilePeace1 = QString("C%1_CP_PEA_3DRL_PSD").arg(scn);
   }
   else if (filePeace1 == "3DRH")
   {
      datasetFilePeace1 = QString("C%1_CP_PEA_3DRH_PSD").arg(scn);
   }
   else if (filePeace1 == "3DXL")
   {
      datasetFilePeace1 = QString("C%1_CP_PEA_3DXL_PSD").arg(scn);
   }
   else if (filePeace1 == "3DXH")
   {
      datasetFilePeace1 = QString("C%1_CP_PEA_3XH_PSD").arg(scn);
   }
   else if (filePeace1 == "3DXPL")
   {
      datasetFilePeace1 = QString("C%1_CP_PEA_3DXPL_PSD").arg(scn);	
   }
   else if (filePeace1 == "3DXPH")
   {
      datasetFilePeace1 = QString("C%1_CP_PEA_3DXPH_PSD").arg(scn); 	
   }
   else
   {
      datasetFilePeace1 = QString("NULL");
   }

// determine the PEACE sensor 2 dataset ID

   if (filePeace2 == "3DRL")
   {
      datasetFilePeace2 = QString("C%1_CP_PEA_3DRL_PSD").arg(scn);
   }
   else if (filePeace2 == "3DRH")
   {
      datasetFilePeace2 = QString("C%1_CP_PEA_3DRH_PSD").arg(scn);
   }
   else if (filePeace2 == "3DXL")
   {
      datasetFilePeace2 = QString("C%1_CP_PEA_3DXL_PSD").arg(scn);
   }
   else if (filePeace2 == "3DXH")
   {
      datasetFilePeace2 = QString("C%1_CP_PEA_3XH_PSD").arg(scn);
   }
   else if (filePeace2 == "3DXPL")
   {
      datasetFilePeace2 = QString("C%1_CP_PEA_3DXPL_PSD").arg(scn);	
   }
   else if (filePeace2 == "3DXPH")
   {
      datasetFilePeace2 = QString("C%1_CP_PEA_3DXPH_PSD").arg(scn); 	
   }   
   else
   {
      datasetFilePeace2 = QString("NULL");
   }

// determine the spacecraft potential dataset ID

   if (filePotential == "EFW_L3_P")
   {
      datasetFilePotential = QString("C%1_CP_EFW_L3_P").arg(scn);
   }
   else if (filePotential == "EFW_L2_P")
   {
      datasetFilePotential = QString("C%1_CP_EFW_L2_P").arg(scn);
   }
   else
   {
      datasetFilePotential = QString("NULL");
   }

// determine the AUX SP dataset ID

   if (fileAuxGSE == "CL_SP_AUX")
   {
      datasetFileAuxGSE = QString("CL_SP_AUX");
   }
   else
   {
      datasetFileAuxGSE = QString("NULL");
   }

// determine the FGM magnetic field dataset ID

   if (fileMagField == "FGM_SPIN")
   {
      datasetFileMagField = QString("C%1_CP_FGM_SPIN").arg(scn);
   }
   else
   {
      datasetFileMagField = QString("NULL");
   }

// prepare the request

   *request = "/caa_query/?uname=" + userName + "&pwd=" + passWord + "&dataset_id=";
   
   QString sp = "";
   if (datasetFilePeace1 != "NULL")
   {
      *request = *request + sp + datasetFilePeace1;
      returnValue++;
      sp = ",";
   }
   if (datasetFilePeace2 != "NULL")
   {
      *request = *request + sp + datasetFilePeace2;
      returnValue++;
      sp = ",";
   }   
   if (datasetFilePotential != "NULL")
   {
      *request = *request + sp + datasetFilePotential;
      sp = ",";
   }   
   if (datasetFileAuxGSE != "NULL")
   {
      *request = *request + sp + datasetFileAuxGSE;
      sp = ",";
   }  
   if (datasetFileMagField != "NULL")
   {
      *request = *request + sp + datasetFileMagField;
      sp = ",";
   }
 
   *request = *request + "&nonotify=1&time_range=" + timeRange;  

   if (returnValue > 0) return(_SUCCESS);
   else return(_NO_VALID_PEACE_FILES_SPECIFIED);

}

