/*******************************************************************************
 **
 ** $Id: getlocations.C 149 2008-11-28 20:51:29Z AndrewLahiff $
 **
 ******************************************************************************
 **
 ** Copyright (C) 2007-2008 Andrew Lahiff.
 **
 ** This file is part of PEACEmoments CAA.
 **
 ** PEACEmoments CAA is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** PEACEmoments CAA is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with PEACEmoments CAA.  If not, see <http://www.gnu.org/licenses/>.
 **
 *******************************************************************************/

#include "peacemoments_caa.h"

//-----------------------------------------------------------------------------
//
// EFW spacecraft potential
//
//-----------------------------------------------------------------------------

void getPotentialLocations(ReadFile *inCef, int iWhat[])
{
   for (int i=0; i<inCef->getNumVariables(); i++)
   {
      if ((inCef->getVariableAttr(i, "NAME").contains("time_tags__") ||
          inCef->getVariableAttr(i, "NAME").contains("Epoch__")) && iWhat[_time_tags] == -1)
      {
         iWhat[_time_tags] = i;
      }
      else if ((inCef->getVariableAttr(i, "NAME").contains("Spacecraft_potential__") ||
                inCef->getVariableAttr(i, "NAME").contains("U_probe_sc__")) && iWhat[_Spacecraft_potential] == -1)
      {
         iWhat[_Spacecraft_potential] = i;
      }
   }
}

//-----------------------------------------------------------------------------
//
// FGM magnetic field
//
//-----------------------------------------------------------------------------

void getMagFieldLocations(ReadFile *inCef, int iWhat[])
{
   for (int i=0; i<inCef->getNumVariables(); i++)
   {
      if ((inCef->getVariableAttr(i, "NAME").contains("Epoch__") ||
           inCef->getVariableAttr(i, "NAME").contains("time_tags__")) && iWhat[_time_tags] == -1)
      {
         iWhat[_time_tags] = i;
      }
      else if ((inCef->getVariableAttr(i, "NAME").contains("B_xyz_gse__") ||
               inCef->getVariableAttr(i, "NAME").contains("B_vec_xyz_gse__")) && iWhat[_MagneticField] == -1)
      {
         iWhat[_MagneticField] = i;
      }
   }
}

//-----------------------------------------------------------------------------
//
// AUX GSE spin-axis angles
//
//-----------------------------------------------------------------------------

void getAuxGSELocations(ReadFile *inCef, int iWhat[])
{
   for (int i=0; i<inCef->getNumVariables(); i++)
   {
      if ((inCef->getVariableAttr(i, "NAME").contains("Epoch__") ||
           inCef->getVariableAttr(i, "NAME").contains("time_tags__")) && iWhat[_time_tags] == -1) // handle both CSDS SP & CAA files
      {
         iWhat[_time_tags] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Half_interval__", Qt::CaseInsensitive) && iWhat[_time_tags_HalfInterval] == -1)
      {
	 iWhat[_time_tags_HalfInterval] = i;     
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("sc_at1_lat__CL_SP_AUX") && iWhat[1] == -1)
      {
         iWhat[1] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("sc_at1_long__CL_SP_AUX") && iWhat[2] == -1)
      {
         iWhat[2] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("sc_at2_lat__CL_SP_AUX") && iWhat[3] == -1)
      {
         iWhat[3] = i;
      }  
      else if (inCef->getVariableAttr(i, "NAME").contains("sc_at2_long__CL_SP_AUX") && iWhat[4] == -1)
      {
         iWhat[4] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("sc_at3_lat__CL_SP_AUX") && iWhat[5] == -1)
      {
         iWhat[5] = i;
      }  
      else if (inCef->getVariableAttr(i, "NAME").contains("sc_at3_long__CL_SP_AUX") && iWhat[6] == -1)
      {
         iWhat[6] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("sc_at4_lat__CL_SP_AUX") && iWhat[7] == -1)
      {
         iWhat[7] = i;
      }            
      else if (inCef->getVariableAttr(i, "NAME").contains("sc_at4_long__CL_SP_AUX") && iWhat[8] == -1)
      {
         iWhat[8] = i;
      }
   }
}

//-----------------------------------------------------------------------------
//
// PEACE
//
//-----------------------------------------------------------------------------

void getPeaceLocations(ReadFile *inCef, int iWhat[])
{ 
   for (int i=0; i<inCef->getNumVariables(); i++)
   {
      if (inCef->getVariableAttr(i, "NAME").contains("time_tags__"))
      {
         iWhat[_time_tags] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("time_tags_DeltaLower__") && iWhat[_time_tags_DeltaLower] < 0)
      {
         iWhat[_time_tags_DeltaLower] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("time_tags_DeltaUpper__") && iWhat[_time_tags_DeltaUpper] < 0)
      {
         iWhat[_time_tags_DeltaUpper] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Angle_SR2phi__") && iWhat[_Angle_SR2phi] < 0)
      {
         iWhat[_Angle_SR2phi] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Mode_Sensor__"))
      {
         iWhat[_Mode_Sensor] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Mode_SweepMode__"))
      {
         iWhat[_Mode_SweepMode] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Mode_RealSize__") && iWhat[_Mode_RealSize] < 0)
      {
         iWhat[_Mode_RealSize] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Mode_RealBinNum__") && iWhat[_Mode_RealBinNum] < 0)
      {
         iWhat[_Mode_RealBinNum] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Mode_EnergyLevelRange__") && iWhat[_Mode_EnergyLevelRange] < 0)
      {
         iWhat[_Mode_EnergyLevelRange] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Mode_DataOrigin__"))
      {
         iWhat[_Mode_DataOrigin] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Mode_SunpulseRephaseOffset"))
      {
         iWhat[_Mode_SPOS] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Energy__"))
      {
         iWhat[_Energy] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Energy_DeltaLower__"))
      {
         iWhat[_Energy_DeltaLower] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Energy_DeltaUpper__"))
      {
         iWhat[_Energy_DeltaUpper] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").startsWith("Data__"))
      {
         iWhat[_Data] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").startsWith("BackgroundLevel__"))
      {
         iWhat[_BackgroundLevel] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").startsWith("Status_Quality__") && iWhat[_Status_Quality] < 0)
      {
         iWhat[_Status_Quality] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").startsWith("Status_CountStats__") && iWhat[_Status_CountsStats] < 0)
      {
         iWhat[_Status_CountsStats] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").startsWith("Status_InterferenceFromEFW__") && iWhat[_Status_EFW] < 0)
      {
         iWhat[_Status_EFW] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").startsWith("Status_InterferenceFromWHI__") && iWhat[_Status_WHI] < 0)
      {
         iWhat[_Status_WHI] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").startsWith("Status_DatastreamErrors__") && iWhat[_Status_DataStream] < 0)
      {
         iWhat[_Status_DataStream] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").startsWith("Status_Eclipse__") && iWhat[_Status_Eclipse] < 0)
      {
         iWhat[_Status_Eclipse] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Sweep_Azimuth__") && iWhat[_Azimuth] == -1)
      {
         iWhat[_Azimuth] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Sweep_Azimuth_DeltaLower__") && iWhat[_Azimuth_DeltaLower] == -1)
      {
         iWhat[_Azimuth_DeltaLower] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Sweep_Azimuth_DeltaUpper__") && iWhat[_Azimuth_DeltaUpper] == -1)
      {
         iWhat[_Azimuth_DeltaUpper] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Data_Overlap_LEEA") && iWhat[_Data_Overlap_LEEA] == -1)
      {
         iWhat[_Data_Overlap_LEEA] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Data_Overlap_HEEA") && iWhat[_Data_Overlap_HEEA] == -1)
      {
         iWhat[_Data_Overlap_HEEA] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Mode_SweepMode_HEEA") && iWhat[_Mode_SweepMode_HEEA] == -1)
      {
         iWhat[_Mode_SweepMode_HEEA] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Mode_SweepMode_LEEA") && iWhat[_Mode_SweepMode_LEEA] == -1)
      {
         iWhat[_Mode_SweepMode_LEEA] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Mode_RealSize_Overlap_LEEA") && iWhat[_Mode_RealSize_Overlap_LEEA] == -1)
      {
         iWhat[_Mode_RealSize_Overlap_LEEA] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Mode_RealSize_Overlap_HEEA") && iWhat[_Mode_RealSize_Overlap_HEEA] == -1)
      {
         iWhat[_Mode_RealSize_Overlap_HEEA] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Mode_RealBinNum_Overlap_LEEA") && iWhat[_Mode_RealBinNum_Overlap_LEEA] == -1)
      {
         iWhat[_Mode_RealBinNum_Overlap_LEEA] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Mode_RealBinNum_Overlap_HEEA") && iWhat[_Mode_RealBinNum_Overlap_HEEA] == -1)
      {
         iWhat[_Mode_RealBinNum_Overlap_HEEA] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Energy_Overlap_LEEA__"))
      {
         iWhat[_Energy_Overlap_LEEA] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Energy_Overlap_LEEA_DeltaLower__"))
      {
         iWhat[_Energy_DeltaLower_Overlap_LEEA] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Energy_Overlap_LEEA_DeltaUpper__"))
      {  
         iWhat[_Energy_DeltaUpper_Overlap_LEEA] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Energy_Overlap_HEEA__"))
      {
         iWhat[_Energy_Overlap_HEEA] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Energy_Overlap_HEEA_DeltaLower__"))
      {
         iWhat[_Energy_DeltaLower_Overlap_HEEA] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Energy_Overlap_HEEA_DeltaUpper__"))
      {  
         iWhat[_Energy_DeltaUpper_Overlap_HEEA] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Sweep_Azimuth_Overlap_LEEA__") &&
               iWhat[_Azimuth_Overlap_LEEA] == -1)
      {
         iWhat[_Azimuth_Overlap_LEEA] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Sweep_Azimuth_Overlap_LEEA_DeltaLower__") &&
               iWhat[_Azimuth_Overlap_LEEA_DeltaLower] == -1)
      {
         iWhat[_Azimuth_Overlap_LEEA_DeltaLower] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Sweep_Azimuth_Overlap_LEEA_DeltaUpper__") &&
               iWhat[_Azimuth_Overlap_LEEA_DeltaUpper] == -1)
      {
         iWhat[_Azimuth_Overlap_LEEA_DeltaUpper] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Sweep_Azimuth_Overlap_HEEA__") &&
               iWhat[_Azimuth_Overlap_HEEA] == -1)
      {
         iWhat[_Azimuth_Overlap_HEEA] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Sweep_Azimuth_Overlap_HEEA_DeltaLower__") &&
               iWhat[_Azimuth_Overlap_HEEA_DeltaLower] == -1)
      {
         iWhat[_Azimuth_Overlap_HEEA_DeltaLower] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Sweep_Azimuth_Overlap_HEEA_DeltaUpper__") &&
               iWhat[_Azimuth_Overlap_HEEA_DeltaUpper] == -1)
      {
         iWhat[_Azimuth_Overlap_HEEA_DeltaUpper] = i;
      }
      else if (inCef->getVariableAttr(i, "NAME").contains("Mode_EnergyLevelRange_LEEA__") && iWhat[_Mode_EnergyLevelRange_LEEA] < 0)
      {
         iWhat[_Mode_EnergyLevelRange_LEEA] = i;
      }  
      else if (inCef->getVariableAttr(i, "NAME").contains("Mode_EnergyLevelRange_HEEA__") && iWhat[_Mode_EnergyLevelRange_HEEA] < 0)
      {
         iWhat[_Mode_EnergyLevelRange_HEEA] = i;
      }  
   }
}
