/*******************************************************************************
 **
 ** $Id: handler.C 166 2008-11-30 09:42:37Z AndrewLahiff $
 **
 ******************************************************************************
 **
 ** Copyright (C) 2007-2008 Andrew Lahiff.
 **
 ** This file is part of PEACEmoments CAA.
 **
 ** PEACEmoments CAA is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** PEACEmoments CAA is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with PEACEmoments CAA.  If not, see <http://www.gnu.org/licenses/>.
 **
 *******************************************************************************/

#include "peacemoments_caa.h"

PeacemomentsParser::PeacemomentsParser()
{
   potOffset1 = QString("%1").arg(INT_FILL_VALUE_PM);
   potOffset2 = QString("%1").arg(INT_FILL_VALUE_PM);

   int_en_low = QString("%1").arg(INT_FILL_VALUE_PM);
   int_en_high = QString("%1").arg(INT_FILL_VALUE_PM);
   int_en_state = INT_FILL_VALUE_PM;
   int_en_type = INT_FILL_VALUE_PM;

   int_pl_low = QString("%1").arg(INT_FILL_VALUE_PM);
   int_pl_high = QString("%1").arg(INT_FILL_VALUE_PM);
   int_pl_state = INT_FILL_VALUE_PM;
}

bool PeacemomentsParser::startDocument()
{
   return true;
}

bool PeacemomentsParser::endElement(QString &, const QString &, const QString &name)
{
   return true;
}

bool PeacemomentsParser::startElement(const QString &, const QString &,
                                      const QString &name,
				      const QXmlAttributes &attrs)
{
   if (name == "files")
   {
      for (int i=0; i<attrs.count(); i++)
      {
         if (attrs.localName(i) == "peace1")
	 {
	    filePeace1 = attrs.value(i);
         }
	 else if (attrs.localName(i) == "peace2")
	 {
	    filePeace2 = attrs.value(i);
	 }
	 else if (attrs.localName(i) == "potential")
	 {
	    filePotential = attrs.value(i);
         }
	 else if (attrs.localName(i) == "auxGSE")
	 {
	    fileAuxGSE = attrs.value(i);
	 }
	 else if (attrs.localName(i) == "fgmB")
	 {
	    fileFGM = attrs.value(i);
         }
      }
   }
   else if (name == "setup")
   {
      for (int i=0; i<attrs.count(); i++)
      {
         if (attrs.localName(i) == "sensor")
	 {
	    setupSensor = attrs.value(i);
	 }
	 else if (attrs.localName(i) == "coord")
	 {
	    setupCoord = attrs.value(i);
         }      
      }      
   }
   else if (name == "pot_corr")
   {
      for (int i=0; i<attrs.count(); i++)
      {
         if (attrs.localName(i) == "offset1")
	 {
	    potOffset1 = attrs.value(i);
	 }
	 else if (attrs.localName(i) == "offset2")
	 {
	    potOffset2 = attrs.value(i);
         }      
	 else if (attrs.localName(i) == "units")
	 {
	    potOffset2units = attrs.value(i).toInt();
	 }
	 else if (attrs.localName(i) == "onoff")
	 {
	    onoff = attrs.value(i).toInt();
	 }
      }      
   }
   else if (name == "output")
   {
      for (int i=0; i<attrs.count(); i++)
      {
         if (attrs.localName(i) == "out_dir")
	 {
	    outputDir = attrs.value(i);
	 }
	 else if (attrs.localName(i) == "mom_filename")
	 {
	    outputFile = attrs.value(i);
         }
      }   
   }
   else if (name == "includes")
   {
      for (int i=0; i<attrs.count(); i++)
      {
         if (attrs.localName(i) == "n")
	 {
	    outInclN = attrs.value(i).toInt();
	 }
	 else if (attrs.localName(i) == "v")
	 {
	    outInclV = attrs.value(i).toInt();	 
         }
	 else if (attrs.localName(i) == "t")
	 {
	    outInclT = attrs.value(i).toInt();	 
         }
	 else if (attrs.localName(i) == "q")
	 {
	    outInclQ = attrs.value(i).toInt();	 
         }
	 else if (attrs.localName(i) == "h")
	 {
	    outInclH = attrs.value(i).toInt();	 
         }
	 else if (attrs.localName(i) == "p")
	 {
	    outInclP = attrs.value(i).toInt();	 
         }
	 else if (attrs.localName(i) == "vpara")
	 {
	    outInclVpara = attrs.value(i).toInt();	 
         }
	 else if (attrs.localName(i) == "vperp")
	 {
	    outInclVperp = attrs.value(i).toInt();	 
         }
	 else if (attrs.localName(i) == "qpara")
	 {
	    outInclQpara = attrs.value(i).toInt();	 
         }
	 else if (attrs.localName(i) == "qperp")
	 {
	    outInclQperp = attrs.value(i).toInt();	 
         }
	 else if (attrs.localName(i) == "hpara")
	 {
	    outInclHpara = attrs.value(i).toInt();	 
         }
	 else if (attrs.localName(i) == "hperp")
	 {
	    outInclHperp = attrs.value(i).toInt();	 
         }
	 else if (attrs.localName(i) == "tpara")
	 {
	    outInclTpara = attrs.value(i).toInt();	 
         }
	 else if (attrs.localName(i) == "tperp")
	 {
	    outInclTperp = attrs.value(i).toInt();	 
         }
         else if (attrs.localName(i) == "j")
         {
            outInclJ = attrs.value(i).toInt();
         }
         else if (attrs.localName(i) == "jpara")
         {
            outInclJpara = attrs.value(i).toInt();
         }
         else if (attrs.localName(i) == "jperp")
         {
            outInclJperp = attrs.value(i).toInt();
         }
         else if (attrs.localName(i) == "ce")
         {
            outInclCE = attrs.value(i).toInt();
         }
         else if (attrs.localName(i) == "b")
         {
            outInclB = attrs.value(i).toInt();
         }
         else if (attrs.localName(i) == "scp")
         {
            outInclScP = attrs.value(i).toInt();
         }
      }      
   }   
   else if (name == "filters")
   {
      for (int i=0; i<attrs.count(); i++)
      {
         if (attrs.localName(i) == "quality")
	 {
	    filterQua = attrs.value(i).toInt();
	 }
	 else if (attrs.localName(i) == "WHI")
	 {
	    filterWHI = attrs.value(i).toInt();
	 }
	 else if (attrs.localName(i) == "EFW")
	 {
	    filterEFW = attrs.value(i).toInt();
	 }
	 else if (attrs.localName(i) == "stats")
	 {
	    filterSta = attrs.value(i).toInt();
	 }
	 else if (attrs.localName(i) == "datastream")
	 {
	    filterDS = attrs.value(i).toInt();
	 }
      }   
   }   
   else if (name == "res_int_en")
   {
      for (int i=0; i<attrs.count(); i++) 
      {
         if (attrs.localName(i) == "low")
         {
            int_en_low = attrs.value(i);
         }
         else if (attrs.localName(i) == "high")
         {
            int_en_high = attrs.value(i);
         }
         else if (attrs.localName(i) == "type")
         {
            int_en_type = attrs.value(i).toInt();
         }
         else if (attrs.localName(i) == "state")
         {
            int_en_state = attrs.value(i).toInt();
         }
	 else if (attrs.localName(i) == "scp_type")
	 {
	    int_en_scp_type = attrs.value(i).toInt();
	 }
      }
   }
   else if (name == "res_int_pl")
   {
      for (int i=0; i<attrs.count(); i++)
      {
         if (attrs.localName(i) == "low")
         {  
            int_pl_low = attrs.value(i);
         }
         else if (attrs.localName(i) == "high")
         {  
            int_pl_high = attrs.value(i);
         }
         else if (attrs.localName(i) == "state")
         {  
            int_pl_state = attrs.value(i).toInt();
         }
      }
   }
   else if (name == "data")
   {
      for (int i=0; i<attrs.count(); i++)
      {
         if (attrs.localName(i) == "scn")
         {
            dataD_SCN = attrs.value(i).toInt();
         }
         else if (attrs.localName(i) == "peace1")
         {
            dataD_PEACE1 = attrs.value(i).toInt();
         }
         else if (attrs.localName(i) == "peace2")
         {
            dataD_PEACE2 = attrs.value(i).toInt();
         }
         else if (attrs.localName(i) == "potential")
         {
            dataD_POT = attrs.value(i).toInt();
         }
         else if (attrs.localName(i) == "auxGSE")
         {
            dataD_AUX = attrs.value(i).toInt();
         }
         else if (attrs.localName(i) == "fgmB")
         {
            dataD_FGM = attrs.value(i).toInt();
         }
      }
   }
   else if (name == "begin_time")
   {
      for (int i=0; i<attrs.count(); i++)
      {
         if (attrs.localName(i) == "year")
         {  
            yearBegin = attrs.value(i).toInt();
         }
         else if (attrs.localName(i) == "doy")
         {
            doyBegin = attrs.value(i).toInt();
         }
         else if (attrs.localName(i) == "day")
         {
            dayBegin = attrs.value(i).toInt();
         }
         else if (attrs.localName(i) == "month")
         {
            monthBegin = attrs.value(i).toInt();
         }
         else if (attrs.localName(i) == "time")
         {
            timeBegin = attrs.value(i);
         }
      }
   }
   else if (name == "end_time")
   {
      for (int i=0; i<attrs.count(); i++)
      {
         if (attrs.localName(i) == "year")
         {  
            yearEnd = attrs.value(i).toInt();
         }
         else if (attrs.localName(i) == "doy")
         {
            doyEnd = attrs.value(i).toInt();
         }
         else if (attrs.localName(i) == "day")
         {
            dayEnd = attrs.value(i).toInt();
         }
         else if (attrs.localName(i) == "month")
         {
            monthEnd = attrs.value(i).toInt();
         }
         else if (attrs.localName(i) == "time")
         {
            timeEnd = attrs.value(i);
         }
      }
   }
   return true;
}
				      

