/*******************************************************************************
 **
 ** $Id: int_trig.C 149 2008-11-28 20:51:29Z AndrewLahiff $
 **
 ******************************************************************************
 **
 ** Copyright (C) 2007-2008 Andrew Lahiff.
 **
 ** This file is part of PEACEmoments CAA.
 **
 ** PEACEmoments CAA is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** PEACEmoments CAA is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with PEACEmoments CAA.  If not, see <http://www.gnu.org/licenses/>.
 **
 *******************************************************************************/

#include "peacemoments_caa.h"

//-----------------------------------------------------------------------------
//
// angle integrals
//
//-----------------------------------------------------------------------------

double T_PM(double theta2, double theta1, int n)
{
   double result;

   theta2 = theta2*4.0*atan(1.0)/180.0;
   theta1 = theta1*4.0*atan(1.0)/180.0;
   
   switch(n)
   {
      case 1:
      result = theta2 - theta1;   
      break;
      
      case 2:
      result = cos(theta1) - cos(theta2);
      break;
      
      case 3:
      result = sin(theta2) - sin(theta1);
      break;
            
      case 4:
      result = (theta2 - theta1 + (sin(2.0*theta1) - sin(2.0*theta2))/2.0)/2.0;
      break;
            
      case 5:
      result = (theta2 - theta1 - (sin(2.0*theta1) - sin(2.0*theta2))/2.0)/2.0;
      break;
            
      case 6:
      result = (cos(2.0*theta1) - cos(2.0*theta2))/4.0;
      break;
            
      case 7:
      result = cos(theta1) - cos(theta2) + (pow(cos(theta2),3.0) - pow(cos(theta1),3.0))/3.0;
      break;
            
      case 8:
      result = sin(theta2) - sin(theta1) - (pow(sin(theta2),3.0) - pow(sin(theta1),3.0))/3.0;
      break;
            
      case 9:
      result = (pow(sin(theta2),3.0) - pow(sin(theta1),3.0))/3.0;
      break;
            
      case 10:
      result = (pow(cos(theta1),3.0) - pow(cos(theta2),3.0))/3.0;
      break;
      
      default:
      result = 0;
      cerr << endl << "ERROR: Incorrect variable 'n' given to function 'T'." << endl << endl;
            
   };
   return(result);
}

//-----------------------------------------------------------------------------
//
// velocity integrals
//
//-----------------------------------------------------------------------------

double VN_PM(double v2, double v1, int n)
{
   double temp1;
   temp1 = pow(v2,(n+1)) - pow(v1,(n+1));
   temp1 = temp1/(n+1);     
   return(temp1);
}
