/*******************************************************************************
 **
 ** $Id: integrationwindow.C 149 2008-11-28 20:51:29Z AndrewLahiff $
 **
 ******************************************************************************
 **
 ** Copyright (C) 2007-2008 Andrew Lahiff.
 **
 ** This file is part of PEACEmoments CAA.
 **
 ** PEACEmoments CAA is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** PEACEmoments CAA is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with PEACEmoments CAA.  If not, see <http://www.gnu.org/licenses/>.
 **
 *******************************************************************************/

#include "peacemoments_caa.h"
#include "peacemoments_gui.h"
#include "window.h"

#define INTEGRATION_WIDTH 60

//-----------------------------------------------------------------------------
//
// constructor
//
//-----------------------------------------------------------------------------

IntegrationWindow::IntegrationWindow(QWidget *parent)
   : QWidget(parent)
{

//   setPalette(QPalette(QColor("#c0c0c0")));
//   setAutoFillBackground(true);

   energyIntGroup = new QGroupBox(tr("Restricted energy integration"));
   energyIntGroup->setCheckable(true);
   energyIntGroup->setChecked(false);
   energyIntGroup->setToolTip(tr("Restricted energy range on/off"));

   QHBoxLayout *energyInsideGroup = new QHBoxLayout;

   energyIntSelect = new QComboBox;
   energyIntSelect->setFixedWidth(90);
   energyIntSelect->addItem("Constant");
//   energyIntSelect->addItem("File");
   energyIntSelect->setToolTip("Select either constant lower and upper limits or an ASCII file for time-varying limits");

   connect(energyIntSelect, SIGNAL(activated(int)),
           this, SLOT(slotEnergyIntSelect(int)));

   energyFileButton = new QPushButton("Select file...");
   energyFileButton->setAutoDefault(false);
   energyFileButton->setEnabled(false);
   energyFileButton->setToolTip("Select an ASCII file containing time-varying low and high energy cutoffs");

//   connect(energyFileButton, SIGNAL(clicked()),
//           this, SLOT(slotEnergyFileSelect()));

   QLabel *energyLowLabel = new QLabel("Lower limit");
   energyLowLabel->setToolTip("Set integration lower limit (eV)");
   QLabel *energyHighLabel = new QLabel("Upper limit");
   energyHighLabel->setToolTip("Set integration upper limit (eV)");

   energyLowEdit = new QLineEdit();
   energyLowEdit->setAlignment(Qt::AlignRight);
   energyLowEdit->setFixedWidth(INTEGRATION_WIDTH);
   energyLowEdit->setToolTip("Set integration lower limit (eV)");

   energyHighEdit = new QLineEdit();
   energyHighEdit->setAlignment(Qt::AlignRight);
   energyHighEdit->setFixedWidth(INTEGRATION_WIDTH);
   energyHighEdit->setToolTip("Set integration upper limit (eV)");
   
   energyIntSpType = new QCheckBox("Spacecraft potential corrected energies");
   energyIntSpType->setToolTip("Click this if the energies are from a spacecraft potential corrected spectrogram");
   
   energyIntType = new QCheckBox("Use exact energy limits");
   energyIntType->setToolTip("Use interpolation so that the energy integration limits are exactly as specified by the user"); 

   energyInsideGroup->addWidget(energyIntSelect);
   energyInsideGroup->addWidget(energyFileButton);
   energyInsideGroup->addStretch(1);
   energyInsideGroup->addWidget(energyLowLabel);
   energyInsideGroup->addWidget(energyLowEdit);
   energyInsideGroup->addStretch(1);
   energyInsideGroup->addWidget(energyHighLabel);
   energyInsideGroup->addWidget(energyHighEdit);
   
   energyFileEdit = new QLineEdit();
   energyFileEdit->setAlignment(Qt::AlignRight);
   energyFileEdit->setReadOnly(true);
   energyFileEdit->setEnabled(false);

// polar integration

   polarIntGroup = new QGroupBox(tr("Restricted polar integration"));
   polarIntGroup->setCheckable(true);
   polarIntGroup->setChecked(false);
   polarIntGroup->setToolTip(tr("Restricted polar range on/off"));

   QLabel *polarLowLabel = new QLabel("Lower limit");
   polarLowLabel->setToolTip("Set integration lower limit (bin number)");
   QLabel *polarHighLabel = new QLabel("Upper limit");
   polarHighLabel->setToolTip("Set integration upper limit (bin number)");

   polarLowEdit = new QLineEdit();
   polarLowEdit->setAlignment(Qt::AlignRight);
   polarLowEdit->setFixedWidth(INTEGRATION_WIDTH);
   polarLowEdit->setToolTip("Set integration lower limit (bin number)");

   polarHighEdit = new QLineEdit();
   polarHighEdit->setAlignment(Qt::AlignRight);
   polarHighEdit->setFixedWidth(INTEGRATION_WIDTH);
   polarHighEdit->setToolTip("Set integration upper limit (bin number)");

   QHBoxLayout *polarInsideGroup = new QHBoxLayout;

   polarInsideGroup->addStretch(1);
   polarInsideGroup->addWidget(polarLowLabel, 0, Qt::AlignRight);
   polarInsideGroup->addWidget(polarLowEdit, 0, Qt::AlignRight);
   polarInsideGroup->addWidget(polarHighLabel, 0, Qt::AlignRight);
   polarInsideGroup->addWidget(polarHighEdit, 0, Qt::AlignRight);

   polarIntGroup->setLayout(polarInsideGroup);

// close button
   
   closeIntegrationButton = new QPushButton("&Close");
   
   connect(closeIntegrationButton, SIGNAL(clicked()),
           this, SLOT(closeIntegrationWindow()));
   
   QHBoxLayout *bottomLayout = new QHBoxLayout;
   bottomLayout->addStretch(1);
   bottomLayout->addWidget(closeIntegrationButton);

// main vertical layout

   QVBoxLayout *layoutIntegration = new QVBoxLayout;
   layoutIntegration->addWidget(energyIntGroup);
   layoutIntegration->addWidget(polarIntGroup);
   layoutIntegration->addLayout(bottomLayout);

   QVBoxLayout *energyVerticalBox = new QVBoxLayout;
   energyVerticalBox->addLayout(energyInsideGroup);
   energyVerticalBox->addWidget(energyIntSpType);
   energyVerticalBox->addWidget(energyIntType);
   energyVerticalBox->addWidget(energyFileEdit);

   energyIntGroup->setLayout(energyVerticalBox);

   setLayout(layoutIntegration);


   setWindowTitle("Integration settings");
}

//------------------------------------------------------------------------------
//
// close integration window
//
//------------------------------------------------------------------------------
   
void IntegrationWindow::closeIntegrationWindow()
{     
   hide();
}  

//------------------------------------------------------------------------------
//
// energy integration select
//
//------------------------------------------------------------------------------

void IntegrationWindow::slotEnergyIntSelect(int iEnergyIntSelect)
{
   if (iEnergyIntSelect == 0)
   {
      energyFileButton->setEnabled(false);
      energyLowEdit->setEnabled(true);
      energyHighEdit->setEnabled(true);
      energyFileEdit->setEnabled(false);
   }
   else
   {
      energyFileButton->setEnabled(true);
      energyLowEdit->setEnabled(false);
      energyHighEdit->setEnabled(false);
      energyFileEdit->setEnabled(true);
   }
}

