/*******************************************************************************
 **
 ** $Id: main.C 176 2009-04-17 23:39:24Z AndrewLahiff $
 **
 ******************************************************************************
 **
 ** Copyright (C) 2007-2008 Andrew Lahiff.
 **
 ** This file is part of PEACEmoments CAA.
 **
 ** PEACEmoments CAA is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** PEACEmoments CAA is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with PEACEmoments CAA.  If not, see <http://www.gnu.org/licenses/>.
 **
 *******************************************************************************/

#define PEACEMOMENTS_MAIN
#include "peacemoments_caa.h"
#include "peacemoments_gui.h"

int main(int argc, char *argv[])
{

// firstly, prepare some general settings for use with QSettings

   QCoreApplication::setOrganizationName("mssl-adl");
   QCoreApplication::setOrganizationDomain("mssl.ucl.ac.uk");
   QCoreApplication::setApplicationName("peacemoments_caa");

// launch GUI or run command-line version

   if (argc == 1 || (argc == 2 && strstr(argv[1], ".xml")) ||
       (argc == 2 && strstr(argv[1], "-psn")))
   {

      QApplication app(argc, argv);
      QApplication::setStyle(new QPlastiqueStyle);
      QApplication::setPalette(QApplication::style()->standardPalette());
      QFont defaultFont = QApplication::font();
      defaultFontSize = defaultFont.pointSize();
      
      int uFS = defaultFontSize - 1;
      
      QString style1 = "";
      style1 = style1 + QString("QPushButton {font-size: %1pt; background-color: #b0b0b0; color: #000000; border-width: 1px; border-color: #909090; border-style: outset; border-radius: 2px; padding: 2px}").arg(uFS);
      style1 = style1 + QString("QPushButton:pressed {background-color: #a0a0a0}");
      style1 = style1 + QString("QPushButton:hover {background-color: #d0d0d0}");
      style1 = style1 + QString("QPushButton:flat {background-color: #a0a0a0}");
      style1 = style1 + QString("QCheckBox {font-size: %1pt;}").arg(uFS);
      style1 = style1 + QString("QGroupBox {font-size: %1pt; background-color: #c0c0c0;}").arg(uFS);
      style1 = style1 + QString("QLineEdit {font-size: %1pt;}").arg(uFS);
      style1 = style1 + QString("QLabel {font-size: %1pt}").arg(uFS);
      style1 = style1 + QString("QComboBox {font-size: %1pt; padding: 2px; border-radius: 2px; background-color: #f0f0f0; selection-background-color: #3333ff}").arg(uFS);
      style1 = style1 + QString("QTabBar::tab {color: #000000; border-width: 1px; padding: 3px;}");
      style1 = style1 + QString("QTabBar::tab:!selected {background-color: #b0b0b0; color: #555555}");
      style1 = style1 + QString("QTabBar::tab:selected {background-color: #d0d0d0; color: #000000}");      
      style1 = style1 + QString("QSpinBox {background-color: #f0f0f0;}");
      style1 = style1 + QString("QSpinBox::up-button {border-width: 1px;}");
      style1 = style1 + QString("QProgressBar {background-color: #e0e0e0; border: 2px; border-color: #d0d0d0}");
      style1 = style1 + QString("QProgressBar::chunk {background-color: #3333ff; width: 10px;}");

      char *fileToLoad = "";
      if (argc == 2 && strstr(argv[1], ".xml")) fileToLoad = argv[1]; // load GUI settings from .xml file
      
      Window window(fileToLoad);
      //window.setStyleSheet(style1);
      window.show();
      window.setMaximumSize(window.minimumWidth(), window.minimumHeight());
      return app.exec();
   }
   else
   {

//-----------------------------------------------------------------------------
//
// command-line calculation of moments
//
//-----------------------------------------------------------------------------
      
      bool runFromXML = false;
      if (argc == 3 && strcmp(argv[1], "-run") == 0 && strstr(argv[2], ".xml")) runFromXML = true;

      int iSensorCombination = -1, iCoordinateSystem = 0, iPotentialOffset2units = 1;
      double potentialOffset1 = 0, potentialOffset2 = 0;

      char *filePeace[2], *fileScPot, *fileGSE, *fileB, *fileOut;
      int iNumPeace = 0;
      bool usePeace1 = false, usePeace2 = false;
      bool useScPot = false, useGSE = false, useB = false;
      bool userSpecifiedOutFile = false;
      bool twoSensorData = false;
      double modeBackgroundLevel = 0.0;

      // remove annoying compiler warnings

      fileGSE = "";
      fileB = "";
      fileScPot = "";
      fileOut = "";

      // search for required arguments

      for (int i = 1; i < argc; i++)
      {
         if (quickCheckFileType(argv[i]) == _LEEA || quickCheckFileType(argv[i]) == _HEEA ||
             quickCheckFileType(argv[i]) == _LEEA_HEEA)
         {
	    if (iNumPeace < 2) filePeace[iNumPeace] = argv[i];
	    iNumPeace++;
            if (quickCheckFileType(argv[i]) == _LEEA_HEEA) twoSensorData = true;
	 }
	 if (strcmp(argv[i], "-s") == 0) // set sensor combination
	 {
            if ((i+1) < argc)
	    {
               iSensorCombination = getSensorCombination(argv[i+1]);
	       if (iSensorCombination == _FAILED)
	       {
                  cerr << "ERROR: invalid sensor combination" << endl;
		  exit(1);
	       }
	    }
	 }
	 if (strcmp(argv[i], "-c") == 0) // set coordinate system
	 {
            if ((i+1) < argc)
	    {
               iCoordinateSystem = getCoordinateSystem(argv[i+1]);
	       if (iCoordinateSystem == _FAILED)
	       {
	          cerr << "ERROR: invalid coordinate system" << endl;
		  exit(1);
	       }
	    }
	 }
	 if (strcmp(argv[i], "-off1") == 0) // set S/C potential offset 1
	 { 
            if ((i+1) < argc)
	    {
               potentialOffset1 = atof(argv[i+1]);
	    }
	 }
	 if (strcmp(argv[i], "-off2") == 0) // set S/C potential offset 2
	 {
            if ((i+1) < argc)
	    {
               potentialOffset2 = atof(argv[i+1]);
	    }
	    if ((i+2) < argc)
	    {
	       if (strcmp(argv[i+2], "bins") == 0) iPotentialOffset2units = _BINS;
	       else if (strcmp(argv[i+2], "eV") == 0) iPotentialOffset2units = _EV;	       
	    }
	 }
         if (strcmp(argv[i], "-o") == 0) // specify output file
         {
            if ((i+1) < argc)
            {
               fileOut = argv[i+1];
               userSpecifiedOutFile = true;
            }
         }
         if (strcmp(argv[i], "-integration-testing") == 0) // use background level data rather than data
         {
            std::cout << "- Integration testing mode" << std::endl;
            if ((i+1) < argc)
            {
               modeBackgroundLevel = atof(argv[i+1]);
            }
            else modeBackgroundLevel = -1.0;
         }
         if (quickCheckFileType(argv[i]) == _POTENTIAL_SPIN ||
             quickCheckFileType(argv[i]) == _POTENTIAL_HIGH)
	 {
            useScPot = true;
	    fileScPot = argv[i];
	 }
         if (quickCheckFileType(argv[i]) == _BFIELD_SPIN)
	 {
            useB = true;
	    fileB = argv[i];
	 }
         if (quickCheckFileType(argv[i]) == _GSE_SPINAXIS_SP)
	 {
            useGSE = true;
	    fileGSE = argv[i];
	 }
      }

// check that we have at least one PEACE file

      if (iNumPeace != 1 && iNumPeace != 2 && !runFromXML)
      {
         cerr << "ERROR: must specify either 1 or 2 PEACE data files" << endl;
	 exit(1);
      }
      else
      {
         usePeace1 = true;
	 if (iNumPeace == 2) usePeace2 = true;
      }

// if user has requested GSE/BFA coordinate system, check that they specified a GSE file

      if ((iCoordinateSystem == _GSE || iCoordinateSystem == _SR2 ||
           iCoordinateSystem == _ISR2) && !useGSE && !runFromXML)
      {
         cerr << "ERROR: must specify a file containing GSE spin-axis angles" << endl;
	 exit(1);
      }

// for PITCH_FULL, default coordinate system is B-field aligned

      if (twoSensorData)
      {
         iCoordinateSystem = _BFA;
         iSensorCombination = _OVERLAP_TWO_SEC;
      }

// decide what moments to write to the output file

      struct sWML myWML;

      if (!runFromXML)
      {
         myWML.n = true;
         myWML.v = true;
         myWML.q = false;
         myWML.h = false;
         myWML.t = true;
         myWML.p = false;
         myWML.vpara = true;
         myWML.vperp = true;
         myWML.qpara = false;
         myWML.qperp = false;
         myWML.hpara = false;
         myWML.hperp = false;
         myWML.tpara = false;
         myWML.tperp = false;
      
         myWML.j = false;
         myWML.jpara = false;
         myWML.jperp = false;

         myWML.ce = false;
      
         myWML.b = false;
         myWML.scp = false;
      }

// decide what moments to plot

      struct sPML myPML;

      if (!runFromXML)
      {
         myPML.n = true;
         myPML.v = true;
         myPML.t = true;

         myPML.q = false;
         myPML.p = false;
         myPML.vpara = false;
         myPML.vperp = false;
         myPML.qpara = false;
         myPML.qperp = false;
         myPML.hpara = false;
         myPML.hperp = false;
         myPML.tpara = false;
         myPML.tperp = false;

         myPML.j = false;
         myPML.j_mag = false;
         myPML.jpara = false;
         myPML.jpara_mag = false;
         myPML.jperp = false;
         myPML.jperp_mag = false;

         myPML.qpara_mag = false;
         myPML.qperp_mag = false;
         myPML.hpara_mag = false;
         myPML.hperp_mag = false;
         myPML.vpara_mag = false;
         myPML.vperp_mag = false;

         myPML.v_mag = false;
         myPML.q_mag = false;
         myPML.h_mag = false;
      }
      
// get settings from .xml file if required

      bool filterQuality = false, filterEFW = false, filterWHI = false, filterStats = false, 
           filterDataStream = false;
	   
      int energyRestrictStatus = 0, energyRestrictScpType = 0, energyRestrictType = 0;
      double energyRestrictLow = 0., energyRestrictHigh = 0.;   

      int polarRestrictStatus = 0;
      double polarRestrictLow = 0., polarRestrictHigh = 0.;
      
      QString userFileOutQ;   

      if (runFromXML)
      {
         QFile fileLoad(argv[2]);
         if (!fileLoad.open(QIODevice::ReadOnly))
	 {
            char message[100];
            statusHandler(-30, message);
	    cerr << "ERROR: " << message << endl;
	    exit(1);	    
	 }
         QXmlInputSource source(&fileLoad);
         QXmlSimpleReader reader;
      
// parse the XML file

         PeacemomentsParser handler;
         reader.setContentHandler(&handler);
         reader.setErrorHandler(&handler);
         reader.parse(source);
      
         fileLoad.close();
	 
// read in the required data

         char fPeace1[400], fPeace2[400], fPotential[400], fAuxGSE[400], fFGM[400];
	 
	 sprintf(fPeace1, handler.filePeace1.toStdString().c_str());
	 sprintf(fPeace2, handler.filePeace2.toStdString().c_str());
	 sprintf(fPotential, handler.filePotential.toStdString().c_str());
	 sprintf(fAuxGSE, handler.fileAuxGSE.toStdString().c_str());
	 sprintf(fFGM, handler.fileFGM.toStdString().c_str());
	 
	 filePeace[0] = fPeace1;
	 filePeace[1] = fPeace2;
	 fileScPot    = fPotential;
	 fileGSE      = fAuxGSE;
	 fileB        = fFGM;
	 
	 if (handler.filePeace1.isEmpty()) usePeace1 = false;
	 else usePeace1 = true;
	 
	 if (handler.filePeace2.isEmpty()) usePeace2 = false;
	 else usePeace2 = true;
	 
	 if (handler.filePotential.isEmpty()) useScPot = false;
	 else useScPot = true;
	 
	 if (handler.fileAuxGSE.isEmpty()) useGSE = false;
	 else useGSE = true; 
	 
	 if (handler.fileFGM.isEmpty()) useB = false;
	 else useB = true;
	 
	 potentialOffset1 = handler.potOffset1.toFloat();
	 potentialOffset2 = handler.potOffset2.toFloat();
	 iPotentialOffset2units = handler.potOffset2units;
	 
	 iSensorCombination = getSensorCombination((char *)handler.setupSensor.toStdString().c_str());
	 iCoordinateSystem = getCoordinateSystem((char *)handler.setupCoord.toStdString().c_str());
	 
	 filterQuality    = handler.filterQua;
	 filterEFW        = handler.filterEFW;
	 filterWHI        = handler.filterWHI;
	 filterStats      = handler.filterSta;
	 filterDataStream = handler.filterDS;
	 
	 energyRestrictStatus = handler.int_en_state; 
	 energyRestrictScpType= handler.int_en_scp_type;
	 energyRestrictType   = handler.int_en_type;
	 energyRestrictLow    = handler.int_en_low.toFloat();
	 energyRestrictHigh   = handler.int_en_high.toFloat();	 

         polarRestrictStatus  = handler.int_pl_state;
         polarRestrictLow     = handler.int_pl_low.toFloat();
         polarRestrictHigh    = handler.int_pl_high.toFloat();
	 
	 myWML.n = handler.outInclN;
	 myWML.v = handler.outInclV;
	 myWML.q = handler.outInclQ;
	 myWML.h = handler.outInclH;
	 myWML.t = handler.outInclT;
	 myWML.p = handler.outInclP;
	 myWML.vpara = handler.outInclVpara;
	 myWML.vperp = handler.outInclVperp;
	 myWML.qpara = handler.outInclQpara;
	 myWML.qperp = handler.outInclQperp;
	 myWML.hpara = handler.outInclHpara;
	 myWML.hperp = handler.outInclHperp;
	 myWML.tpara = handler.outInclTpara;
	 myWML.tperp = handler.outInclTperp;
	 myWML.j = handler.outInclJ;
	 myWML.jpara = handler.outInclJpara;
	 myWML.jperp = handler.outInclJperp;
	 myWML.b = handler.outInclB;
	 myWML.scp = handler.outInclScP; 
	 
         if (!handler.outputDir.isEmpty())
	 {
            userFileOutQ = handler.outputDir + "/" + handler.outputFile;
	 }
	 else if (!handler.outputFile.isEmpty())
	 {
	    userFileOutQ = handler.outputFile;
	 }
	 else userFileOutQ = "moments.cef";	 
	 	 	       
      }     

//-----------------------------------------------------------------------------
//       
// handle special cases
//    
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
//
// calculate moments
//
//-----------------------------------------------------------------------------

      char *userFileOut;
      if (userSpecifiedOutFile)
      {
         userFileOut = fileOut;
      }
      else if (runFromXML) 
      {
         char tmp[400];
	 sprintf(tmp, userFileOutQ.toStdString().c_str());
         userFileOut = tmp;
      }
      else userFileOut = "moments.cef";

// prepare integration testing

      bool testingInt = false;
      if (modeBackgroundLevel == -1) testingInt = true;
            
// run moments code

      int numRecords = 0;
      PeaceMoments peaceMoments;
      int ret = peaceMoments.run_calc(0, usePeace1, usePeace2, useScPot, useB, useGSE,
  		                 filePeace[0], filePeace[1], fileScPot, fileGSE, fileB,
			         iSensorCombination, iCoordinateSystem, myWML, myPML,
			         potentialOffset1, potentialOffset2, iPotentialOffset2units,
                                 energyRestrictStatus, energyRestrictScpType, energyRestrictType,
			         energyRestrictLow, energyRestrictHigh,
                                 polarRestrictStatus, polarRestrictLow, polarRestrictHigh,
                                 filterQuality, filterEFW, filterWHI, filterStats, filterDataStream,
			         userFileOut, &numRecords, false, testingInt, modeBackgroundLevel);

// print error message if necessary

      if (ret != _SUCCESS)
      {
         char message[100];
         statusHandler(ret, message);
	 cerr << "ERROR: " << message << endl;
	 exit(1);
      }
   }
   exit(0);
}

