/*******************************************************************************
 **
 ** $Id: misc.C 149 2008-11-28 20:51:29Z AndrewLahiff $
 **
 ******************************************************************************
 **
 ** Copyright (C) 2007-2008 Andrew Lahiff.
 **
 ** This file is part of PEACEmoments CAA.
 **
 ** PEACEmoments CAA is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** PEACEmoments CAA is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with PEACEmoments CAA.  If not, see <http://www.gnu.org/licenses/>.
 **
 *******************************************************************************/

#include "peacemoments_caa.h"

//-----------------------------------------------------------------------------
//
// determine spacecraft number
//
//-----------------------------------------------------------------------------

int returnSpacecraftNumber(QString dataSetID)
{
   if (dataSetID.contains("C1_CP") || dataSetID.contains("C1_PP"))
   {
      return(1);
   }
   else if (dataSetID.contains("C2_CP") || dataSetID.contains("C2_PP"))
   {
      return(2);
   }
   else if (dataSetID.contains("C3_CP") || dataSetID.contains("C3_PP"))
   {
      return(3);
   }
   else if (dataSetID.contains("C4_CP") || dataSetID.contains("C4_PP"))
   {
      return(4);
   }
   else return(_FAILED);
}

//-----------------------------------------------------------------------------
//
// determine file type
//
//-----------------------------------------------------------------------------

// handle CP3DRL and CP3DRH as well as the more recent 3DRL and 3DRH. For 3DX
// products we only will allow names without "CP", as there were problems with
// the older files.

int checkForFileType(QString dataSetID)
{
   if (dataSetID.contains("PEA"))
   {
      if (dataSetID.contains("_3DRL_") || dataSetID.contains("_3DXL_") ||
          dataSetID.contains("_3DXPL_") || dataSetID.contains("_3DXLARL_") ||
          dataSetID.contains("_3DXPLARL_") || dataSetID.contains("_CP3DRL_"))
      {
         return(_LEEA); 
      }
      else if (dataSetID.contains("_3DRH_") || dataSetID.contains("_3DXH_") ||
               dataSetID.contains("_3DXPH_") || dataSetID.contains("_3DXLARH_") ||
               dataSetID.contains("_3DXPLARH_") || dataSetID.contains("_CP3DRH_"))
      {
	 return(_HEEA);
      }
      else if (dataSetID.contains("_PITCH_FULL_") || 
               dataSetID.contains("_PITCH_3D_"))
      {
         return(_LEEA_HEEA);
      }
      else
      {
         return(-1);
      }
   }
   else if (dataSetID.contains("EFW"))
   {
      if (dataSetID.contains("_EFW_L3_P") || dataSetID.contains("_PP_EFW"))
      {
         return(_POTENTIAL_SPIN);
      }
      else if (dataSetID.contains("_EFW_L2_P"))
      {
         return(_POTENTIAL_HIGH);
      }
      else
      {
         return(-1); 
      }
   }
   else if (dataSetID.contains("FGM"))
   {
      if (dataSetID.contains("_SPIN") || dataSetID.contains("_PP_FGM"))
      {
         return(_BFIELD_SPIN);
      }
      else if (dataSetID.contains("_FULL_"))
      {
         return(_BFIELD_HIGH);
      }
      else
      {
         return(-1);
      }
   }
   else if (dataSetID.contains("CL_SP_AUX"))
   {
      return(_GSE_SPINAXIS_SP);
   }
   else
   {
      return(-1);
   }
}

//-----------------------------------------------------------------------------
//
// insert PEACE data into LEEA storage
//
//-----------------------------------------------------------------------------

int putPeaceDataIntoLEEA(ReadFile *peaceRead, PeaceData *peaceSpin, int *iWhat, bool which)
{                    
   bool r1, r2, r3, r4, r5, r6;
   r1 = peaceRead->getDataValue(iWhat[_Mode_SweepMode], &peaceSpin->sweepModeLEEA);
   r2 = peaceRead->getDataValue(iWhat[_Energy], peaceSpin->energyLEEA);
   r3 = peaceRead->getDataValue(iWhat[_Energy_DeltaLower], peaceSpin->energyLowLEEA);
   r4 = peaceRead->getDataValue(iWhat[_Energy_DeltaUpper], peaceSpin->energyHighLEEA);

   if (!which) r5 = peaceRead->getDataValue(iWhat[_Data], peaceSpin->dataStoreLEEA);
   else r5 = peaceRead->getDataValue(iWhat[_BackgroundLevel], peaceSpin->dataStoreLEEA);

   r6 = peaceRead->getDataValue(iWhat[_Mode_DataOrigin], &peaceSpin->dataOriginLEEA);

   if (r1 && r2 && r3 && r4 && r5 && r6) return(0);
   else return(1);
}

//-----------------------------------------------------------------------------
//
// insert PEACE data into HEEA storage
//
//-----------------------------------------------------------------------------

int putPeaceDataIntoHEEA(ReadFile *peaceRead, PeaceData *peaceSpin, int *iWhat, bool which)
{
   bool r1, r2, r3, r4, r5, r6;
   r1 = peaceRead->getDataValue(iWhat[_Mode_SweepMode], &peaceSpin->sweepModeHEEA);
   r2 = peaceRead->getDataValue(iWhat[_Energy], peaceSpin->energyHEEA);
   r3 = peaceRead->getDataValue(iWhat[_Energy_DeltaLower], peaceSpin->energyLowHEEA);
   r4 = peaceRead->getDataValue(iWhat[_Energy_DeltaUpper], peaceSpin->energyHighHEEA);

   if (!which) r5 = peaceRead->getDataValue(iWhat[_Data], peaceSpin->dataStoreHEEA);
   else r5 = peaceRead->getDataValue(iWhat[_BackgroundLevel], peaceSpin->dataStoreHEEA);

   r6 = peaceRead->getDataValue(iWhat[_Mode_DataOrigin], &peaceSpin->dataOriginHEEA);

   if (r1 && r2 && r3 && r4 && r5 && r6) return(0);
   else return(1);
}

//-----------------------------------------------------------------------------
//
// insert PEACE data into LEEA storage (LEEA overlap)
// 
//-----------------------------------------------------------------------------

int putPeaceDataIntoLEEAo(ReadFile *peaceRead, PeaceData *peaceSpin, int *iWhat)
{                    
   bool r1, r2, r3, r4, r5;
   r1 = peaceRead->getDataValue(iWhat[_Mode_SweepMode_LEEA], &peaceSpin->sweepModeLEEA);
   r2 = peaceRead->getDataValue(iWhat[_Energy_Overlap_LEEA], peaceSpin->energyLEEA);
   r3 = peaceRead->getDataValue(iWhat[_Energy_DeltaLower_Overlap_LEEA], peaceSpin->energyLowLEEA);
   r4 = peaceRead->getDataValue(iWhat[_Energy_DeltaUpper_Overlap_LEEA], peaceSpin->energyHighLEEA);
   r5 = peaceRead->getDataValue(iWhat[_Data_Overlap_LEEA], peaceSpin->dataStoreLEEA);
//   r6 = peaceRead->getDataValue(iWhat[_Mode_DataOrigin], &peaceSpin->dataOriginLEEA);
   
   if (r1 && r2 && r3 && r4 && r5) return(0);
   else return(1); 
}  

//-----------------------------------------------------------------------------
//
// insert PEACE data into HEEA storage (HEEA overlap)
// 
//-----------------------------------------------------------------------------

int putPeaceDataIntoHEEAo(ReadFile *peaceRead, PeaceData *peaceSpin, int *iWhat)
{
   bool r1, r2, r3, r4, r5;
   r1 = peaceRead->getDataValue(iWhat[_Mode_SweepMode_HEEA], &peaceSpin->sweepModeHEEA);
   r2 = peaceRead->getDataValue(iWhat[_Energy_Overlap_HEEA], peaceSpin->energyHEEA);
   r3 = peaceRead->getDataValue(iWhat[_Energy_DeltaLower_Overlap_HEEA], peaceSpin->energyLowHEEA);
   r4 = peaceRead->getDataValue(iWhat[_Energy_DeltaUpper_Overlap_HEEA], peaceSpin->energyHighHEEA);
   r5 = peaceRead->getDataValue(iWhat[_Data_Overlap_HEEA], peaceSpin->dataStoreHEEA);
//   r6 = peaceRead->getDataValue(iWhat[_Mode_DataOrigin], &peaceSpin->dataOriginHEEA);
   
   if (r1 && r2 && r3 && r4 && r5) return(0);
   else return(1);
}

//-----------------------------------------------------------------------------
//
// determine energy bin information
//
//-----------------------------------------------------------------------------

void energyLevels(int sweepMode, int energyLevelHigh,
		  int *energySkip, int *energyNum, int *energyHigh, int distType)
{

// determine the total number of energy levels (assuming no 3DX energy window) &
// the 'distance" between each used energy level

   if (sweepMode == 1)
   {
      *energySkip = 1;
      *energyNum = 60;
   }
   else if (sweepMode == 2)
   {
      *energySkip = 2;
      *energyNum = 15;
   }
   else if (sweepMode == 3)
   {
      *energySkip = 2;
      *energyNum = 30;
   }
   else
   {
      *energySkip = -1;
      *energyNum = -1;
      return;
   }

// handle energy summation

   if (distType == _3DXE || (distType == _3DR && sweepMode != 2))
   {
      *energySkip = *energySkip*2;
   }

// highest energy level

   *energyHigh = energyLevelHigh - 1;
}

//-----------------------------------------------------------------------------
//
// determine sensor combination from user input
//
//-----------------------------------------------------------------------------

int getSensorCombination(char in[])
{
   int iret = _FAILED;
   if (strcmp(in, "LEEA") == 0) iret = _LEEA;
   else if (strcmp(in, "HEEA") == 0) iret = _HEEA;
   else if (strcmp(in, "TL1L2B") == 0) iret = _TL1L2B;
   else if (strcmp(in, "TH1H2B") == 0) iret = _TH1H2B;
   else if (strcmp(in, "TL1H1B") == 0) iret = _TL1H1B;
   else if (strcmp(in, "TL2H2B") == 0) iret = _TL2H2B;
   else if (strcmp(in, "TL1L2") == 0) iret = _TL1L2;
   else if (strcmp(in, "TH1H2") == 0) iret = _TH1H2;
   else if (strcmp(in, "L1L2") == 0) iret = _L1L2;
   else if (strcmp(in, "H1H2") == 0) iret = _H1H2;
   else if (strcmp(in, "L1L2B") == 0) iret = _L1L2B;
   else if (strcmp(in, "H1H2B") == 0) iret = _H1H2B;
   else if (strcmp(in, "Overlap 2-sec") == 0) iret = _OVERLAP_TWO_SEC;
   return(iret);
}

//-----------------------------------------------------------------------------
//
// determine coordinate system from user input
//
//-----------------------------------------------------------------------------

int getCoordinateSystem(char in[])
{
   int iret = _FAILED;
   if (strcmp(in, "SENSOR") == 0 || strcmp(in, "Sensor") == 0) iret = _SENSOR;
   else if (strcmp(in, "SPACECRAFT") == 0 || strcmp(in, "Spacecraft") == 0) iret = _SPACECRAFT;
   else if (strcmp(in, "GSE") == 0) iret = _GSE;
   else if (strcmp(in, "GSM") == 0) iret = _GSM;
   else if (strcmp(in, "ISR2") == 0) iret = _ISR2;
   else if (strcmp(in, "SR2") == 0) iret = _SR2;
   else if (strcmp(in, "BFA") == 0) iret = _BFA;
   else if (strcmp(in, "B-field aligned") == 0) iret = _BFA;
   return(iret);
}

//-----------------------------------------------------------------------------
//
// convert sensor combination int to QString
//
//-----------------------------------------------------------------------------

QString sensorCombinationToStr(int in)
{
   switch(in)
   {
      case _HEEA:
      return("HEEA"); break;
      case _LEEA:
      return("LEEA"); break;
      case _TH1H2B:
      return("TH1H2B"); break;
      case _TL1L2B:
      return("TL1L2B"); break;
      case _TL1L2:
      return("TL1L2"); break;
      case _H1H2B:
      return("H1H2B"); break;
      case _TH1H2:
      return("TH1H2"); break;
      case _L1L2B:
      return("L1L2B"); break;
      case _L1L2:
      return("L1L2"); break;
      case _H1H2:
      return("H1H2"); break;
      case _TL1H1B:
      return("TL1H1B"); break;
      case _TL2H2B:
      return("TL2H2B"); break;
      default:
      return(""); break;
   }
}

//-----------------------------------------------------------------------------
//
// convert coordinate system int to QString
//
//-----------------------------------------------------------------------------

void coordinateSystemToStr(int in, char C_S[], char F[], char C[])
{
   switch(in)
   {
      case _GSE:
      sprintf(C, "GSE");
      sprintf(F, "vector>gse_xyz");
      sprintf(C_S, "GSE>Geocentric Solar Ecliptic");
      break;

      case _GSM:
      sprintf(C, "GSM");
      sprintf(F, "vector>gsm_xyz");
      sprintf(C_S, "GSE>Geocentric Solar Magnetic");      
      break;

      case _SR2:
      sprintf(C, "SR2");
      sprintf(F, "vector>sr2_xyz");
      sprintf(C_S, "SR2>Despun spacecraft (spin-reference) frame");
      break;

      case _ISR2:
      sprintf(C, "ISR2");
      sprintf(F, "vector>isr2_xyz");
      sprintf(C_S, "ISR2>Inverted (about x-axis) SR2");         
      break;

      case _SENSOR:
      sprintf(C, "Instrument");
      sprintf(F, "vector>inst_xyz");
      sprintf(C_S, "INSTRUMENT>Instrument frame");         
      break;

      case _SPACECRAFT:
      sprintf(C, "Spacecraft");
      sprintf(F, "vector>sc_xyz");
      sprintf(C_S, "SC>SpaceCraft"); 
      break;

      case _BFA:
      sprintf(C, "BFA");                         // this is just temporary!
      sprintf(F, "vector>bfa_xyz");              // this is just temporary!
      sprintf(C_S, "BFA>B-field Aligned frame"); // this is just temporary!
      break;

      default:
      break;
   }
}

