/*******************************************************************************
 **
 ** $Id: outputwindow.C 166 2008-11-30 09:42:37Z AndrewLahiff $
 **
 ******************************************************************************
 **
 ** Copyright (C) 2007-2008 Andrew Lahiff.
 **
 ** This file is part of PEACEmoments CAA.
 **
 ** PEACEmoments CAA is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** PEACEmoments CAA is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with PEACEmoments CAA.  If not, see <http://www.gnu.org/licenses/>.
 **
 *******************************************************************************/

#include "peacemoments_caa.h"
#include "peacemoments_gui.h"
#include "window.h"

#define FILE_WIDTH 400
#define FILE_BUTTON_WIDTH 30

//-----------------------------------------------------------------------------
//
// constructor
//
//-----------------------------------------------------------------------------

OutputWindow::OutputWindow(QWidget *parent)
   : QWidget(parent)
{

//   setPalette(QPalette(QColor("#c0c0c0")));
//   setAutoFillBackground(true);

// output directory

   QGroupBox *outDirGroup = new QGroupBox("Output directory");

   outputDirectoryEdit = new QLineEdit();
   outputDirectoryEdit->setFixedWidth(FILE_WIDTH);
   outputDirectoryEdit->setAlignment(Qt::AlignRight);
   outputDirectoryEdit->setToolTip("Set the directory in which output files will be generated");

   QPushButton *selectOutputDirectoryEdit = new QPushButton("...");
   selectOutputDirectoryEdit->setFixedWidth(FILE_BUTTON_WIDTH);
   selectOutputDirectoryEdit->setToolTip("Browse file system");

   QHBoxLayout *gh1 = new QHBoxLayout();
   gh1->addWidget(outputDirectoryEdit);
   gh1->addWidget(selectOutputDirectoryEdit);
   gh1->addStretch(1);

   outDirGroup->setLayout(gh1);

   connect(selectOutputDirectoryEdit, SIGNAL(clicked()),
           this, SLOT(slotSelectOutputDirectory()));

   currentOutputDirectory = new QString("");

// output file

   QGroupBox *outFileGroup = new QGroupBox("Moments output file");

   outputFileEdit = new QLineEdit();
   outputFileEdit->setFixedWidth(FILE_WIDTH);
   outputFileEdit->setAlignment(Qt::AlignRight);
   outputFileEdit->setToolTip("Set the name of the output file containing moments");

   QHBoxLayout *gh2 = new QHBoxLayout();
   gh2->addWidget(outputFileEdit);
   gh2->addStretch(1);

   outFileGroup->setLayout(gh2);

// moments

   inclOutN = new QCheckBox("Density");
   inclOutV = new QCheckBox("Velocity");
   inclOutQ = new QCheckBox("Energy flux");
   inclOutH = new QCheckBox("Heat flux");
   inclOutVpara = new QCheckBox("Velocity (para)");
   inclOutVperp = new QCheckBox("Velocity (perp)");
   inclOutQpara = new QCheckBox("Energy flux (para)");
   inclOutQperp = new QCheckBox("Energy flux (perp)");
   inclOutHpara = new QCheckBox("Heat flux (para)");
   inclOutHperp = new QCheckBox("Heat flux (perp)");
   inclOutP = new QCheckBox("Pressure");
   inclOutT = new QCheckBox("Temperature");
   inclOutTpara = new QCheckBox("Temperature (para)");
   inclOutTperp = new QCheckBox("Temperature (perp)");
   
   inclOutJ = new QCheckBox("Current");
   inclOutJpara = new QCheckBox("Current (para)");
   inclOutJperp = new QCheckBox("Current (perp)");

   inclOutCE = new QCheckBox("Char. energy");

   inclOutB = new QCheckBox("Magnetic field vector");
   inclOutScP = new QCheckBox("Spacecraft potential");

   inclOutVpara->setToolTip("Note that para/perp moments are only available if both GSE and FGM input files are specified");
   inclOutVperp->setToolTip("Note that para/perp moments are only available if both GSE and FGM input files are specified");
   inclOutQpara->setToolTip("Note that para/perp moments are only available if both GSE and FGM input files are specified");
   inclOutQperp->setToolTip("Note that para/perp moments are only available if both GSE and FGM input files are specified");
   inclOutHpara->setToolTip("Note that para/perp moments are only available if both GSE and FGM input files are specified");
   inclOutHperp->setToolTip("Note that para/perp moments are only available if both GSE and FGM input files are specified");
   inclOutTpara->setToolTip("Note that para/perp moments are only available if both GSE and FGM input files are specified"); 
   inclOutTperp->setToolTip("Note that para/perp moments are only available if both GSE and FGM input files are specified");

// default settings

   inclOutN->setChecked(true);
   inclOutV->setChecked(true);
   inclOutT->setChecked(true);

// grid

   QGridLayout *mainLayout = new QGridLayout;
   mainLayout->addWidget(inclOutN, 0, 0);
   mainLayout->addWidget(inclOutV, 0, 1);
   mainLayout->addWidget(inclOutQ, 0, 2);
   mainLayout->addWidget(inclOutH, 0, 3);
   mainLayout->addWidget(inclOutVpara, 1, 0);
   mainLayout->addWidget(inclOutVperp, 1, 1);
   mainLayout->addWidget(inclOutQpara, 1, 2);
   mainLayout->addWidget(inclOutQperp, 1, 3);
   mainLayout->addWidget(inclOutHpara, 2, 0);
   mainLayout->addWidget(inclOutHperp, 2, 1);
   mainLayout->addWidget(inclOutP, 2, 2);
   mainLayout->addWidget(inclOutT, 2, 3);
   mainLayout->addWidget(inclOutTpara, 3, 0);
   mainLayout->addWidget(inclOutTperp, 3, 1);
   mainLayout->addWidget(inclOutJ, 3, 2);
   mainLayout->addWidget(inclOutJpara, 3, 3);
   mainLayout->addWidget(inclOutJperp, 4, 0);
   mainLayout->addWidget(inclOutCE, 4, 1);
   mainLayout->addWidget(inclOutB, 4, 2);
   mainLayout->addWidget(inclOutScP, 4, 3);

   QVBoxLayout *mainLayoutV = new QVBoxLayout;
   mainLayoutV->addLayout(mainLayout);

   selectAllMoments = new QPushButton("Select all");

   QHBoxLayout *mainSelectAll = new QHBoxLayout;
   mainSelectAll->addStretch(1);
   mainSelectAll->addWidget(selectAllMoments);
   mainLayoutV->addLayout(mainSelectAll);

   QGroupBox *momentsGroup = new QGroupBox("Output file content");
   momentsGroup->setLayout(mainLayoutV);
   momentsGroup->setToolTip("Moments to include in the output file");

   connect(selectAllMoments, SIGNAL(clicked()),
           this, SLOT(slotSelectAllMoments()));

// filtering

   filterQUA = new QCheckBox("Remove spins with poor quality data");
   filterWHI = new QCheckBox("Remove spins containing possible WHISPER interference");
   filterEFW = new QCheckBox("Remove spins containing possible EFW interference");
   filterStats = new QCheckBox("Remove spins with possible poor statistics or saturation");
   filterDataStream = new QCheckBox("Remove spins with datastream errors");

   selectAllFilters = new QPushButton("Select all");
   QHBoxLayout *filterSelectAll = new QHBoxLayout;
   filterSelectAll->addStretch(1);
   filterSelectAll->addWidget(selectAllFilters);

   connect(selectAllFilters, SIGNAL(clicked()),
           this, SLOT(slotSelectAllFilters()));

   QVBoxLayout *insideV1 = new QVBoxLayout;
   insideV1->addWidget(filterQUA);
   insideV1->addWidget(filterWHI);
   insideV1->addWidget(filterEFW);
   insideV1->addWidget(filterStats);
   insideV1->addWidget(filterDataStream);
   insideV1->addLayout(filterSelectAll);
   QGroupBox *filterGroup = new QGroupBox("Output filtering");
   filterGroup->setLayout(insideV1);
   filterGroup->setToolTip("Remove data with possible problems");

// close button

   closeOutputButton = new QPushButton("&Close");
   closeOutputButton->setToolTip("Close this window");

   connect(closeOutputButton, SIGNAL(clicked()),
           this, SLOT(closeOutputWindow()));

   saveAsDefaultsButton = new QPushButton("&Save as defaults");
   saveAsDefaultsButton->setToolTip("Save all output file content and filtering settings as the default");

   connect(saveAsDefaultsButton, SIGNAL(clicked()),
           this, SLOT(saveAsDefaultsSlot()));

   QHBoxLayout *bottomLayout = new QHBoxLayout;
   bottomLayout->addWidget(saveAsDefaultsButton);
   bottomLayout->addStretch(1);
   bottomLayout->addWidget(closeOutputButton);

// main vertical layout

   QVBoxLayout *layoutOutput = new QVBoxLayout;
   layoutOutput->addWidget(outDirGroup);
   layoutOutput->addWidget(outFileGroup);
   layoutOutput->addWidget(momentsGroup);
   layoutOutput->addWidget(filterGroup);
   layoutOutput->addLayout(bottomLayout);

   setLayout(layoutOutput);
   setWindowTitle("Output settings");
}

//------------------------------------------------------------------------------
//
// close integration window
//
//------------------------------------------------------------------------------
   
void OutputWindow::closeOutputWindow()
{     
   hide();
}  

//-----------------------------------------------------------------------------
//
// Files slots
// 
//-----------------------------------------------------------------------------

void OutputWindow::slotSelectOutputDirectory()
{
   QString tmp;
   tmp = QFileDialog::getExistingDirectory(this,
                         "Choose a directory", *currentOutputDirectory);

   if (tmp.isNull() || tmp.isEmpty())
   {

   }
   else
   {
      outputDirectoryEdit->setText(tmp);
      *currentOutputDirectory = tmp;
   }

}  

//-----------------------------------------------------------------------------
//
// save default settings
//
//-----------------------------------------------------------------------------

void OutputWindow::saveAsDefaultsSlot()
{
   QSettings settings(QSettings::NativeFormat, QSettings::UserScope, "mssl-adl", "peacemoments_caa");
   settings.setValue("out-incl/n", inclOutN->isChecked());
   settings.setValue("out-incl/v", inclOutV->isChecked());
   settings.setValue("out-incl/vpara", inclOutVpara->isChecked());
   settings.setValue("out-incl/vperp", inclOutVperp->isChecked());
   settings.setValue("out-incl/q", inclOutQ->isChecked());
   settings.setValue("out-incl/qpara", inclOutQpara->isChecked());
   settings.setValue("out-incl/qperp", inclOutQperp->isChecked());
   settings.setValue("out-incl/h", inclOutH->isChecked());
   settings.setValue("out-incl/hpara", inclOutHpara->isChecked());
   settings.setValue("out-incl/hperp", inclOutHperp->isChecked());
   settings.setValue("out-incl/p", inclOutP->isChecked());
   settings.setValue("out-incl/t", inclOutT->isChecked());
   settings.setValue("out-incl/tpara", inclOutTpara->isChecked());
   settings.setValue("out-incl/tperp", inclOutTperp->isChecked());
   settings.setValue("out-incl/j", inclOutJ->isChecked());
   settings.setValue("out-incl/jpara", inclOutJpara->isChecked());
   settings.setValue("out-incl/jperp", inclOutJperp->isChecked());
   settings.setValue("out-incl/b", inclOutB->isChecked());
   settings.setValue("out-incl/scp", inclOutScP->isChecked());
   settings.setValue("out-incl/ce", inclOutCE->isChecked());
   settings.setValue("out-filt/quality", filterQUA->isChecked());
   settings.setValue("out-filt/whi", filterWHI->isChecked());
   settings.setValue("out-filt/efw", filterEFW->isChecked());
   settings.setValue("out-filt/counts", filterStats->isChecked());
   settings.setValue("out-filt/datas", filterDataStream->isChecked());
}

//-----------------------------------------------------------------------------
//
// slot for moments select all
// 
//-----------------------------------------------------------------------------

void OutputWindow::slotSelectAllMoments()
{
   inclOutN->setChecked(true);
   inclOutV->setChecked(true);
   inclOutQ->setChecked(true);
   inclOutH->setChecked(true);
   inclOutVpara->setChecked(true);
   inclOutVperp->setChecked(true);
   inclOutQpara->setChecked(true);
   inclOutQperp->setChecked(true);
   inclOutHpara->setChecked(true);
   inclOutHperp->setChecked(true);
   inclOutP->setChecked(true);
   inclOutT->setChecked(true);
   inclOutTpara->setChecked(true);
   inclOutTperp->setChecked(true);
   inclOutJ->setChecked(true);
   inclOutJpara->setChecked(true);
   inclOutJperp->setChecked(true);
   inclOutB->setChecked(true);
   inclOutScP->setChecked(true);
   inclOutCE->setChecked(true);
}  

//-----------------------------------------------------------------------------
//
// slot for filters select all
// 
//-----------------------------------------------------------------------------

void OutputWindow::slotSelectAllFilters()
{
   filterQUA->setChecked(true);
   filterWHI->setChecked(true);
   filterEFW->setChecked(true);
   filterStats->setChecked(true);
   filterDataStream->setChecked(true);
}  

