/*******************************************************************************
 **
 ** $Id: peacemoments.C 176 2009-04-17 23:39:24Z AndrewLahiff $
 **
 ******************************************************************************
 **
 ** Copyright (C) 2007-2008 Andrew Lahiff.
 **
 ** This file is part of PEACEmoments CAA.
 **
 ** PEACEmoments CAA is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** PEACEmoments CAA is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with PEACEmoments CAA.  If not, see <http://www.gnu.org/licenses/>.
 **
 *******************************************************************************/

#include "peacemoments_caa.h"

//-----------------------------------------------------------------------------
//
// constructor
//
//-----------------------------------------------------------------------------

PeaceMoments::PeaceMoments(QObject *parent)
{

}

//-----------------------------------------------------------------------------
//
// destructor
//
//-----------------------------------------------------------------------------

PeaceMoments::~PeaceMoments()
{

}

//-----------------------------------------------------------------------------
//
// calculate moments
//
//-----------------------------------------------------------------------------

int PeaceMoments::run_calc(int guiOrNot, bool usePeace1, bool usePeace2,
	 	      bool usePotential, bool useMagField, bool useAuxGSE,
		      char filePeace1[], char filePeace2[],
		      char filePotential[], char fileAuxGSE[], char fileMagField[],
		      int sensorCombination, int coordinateSystem,
		      struct sWML WML, struct sPML PML,
		      double potentialOffset1, double potentialOffset2, int potentialOffset2units,
                      int energyRestrictStatus, int energyRestrictScpType, int energyRestrictType,
		      double energyRestrictLow, double energyRestrictHigh,
                      int polarRestrictStatus, double polarRestrictLow, double polarRestrictHigh,
                      bool filterQuality, bool filterEFW, bool filterWHI, bool filterStats, bool filterDataStream,
		      char fileName[], int *numRecordsReturned, bool guiOnOff, bool modeBackgroundLevel,
                      double oneCountValue)
{

//-----------------------------------------------------------------------------
//
// print info for debugging purposes (make sure this is "false" in the 
// subversion repository, otherwise users will download versions of the 
// software that print out debugging info!)
//
//-----------------------------------------------------------------------------

   bool dl = false;

//-----------------------------------------------------------------------------
// 
// integration testing
//
//-----------------------------------------------------------------------------

   bool testingInt = false;
   if (oneCountValue > 0) testingInt = true;

//-----------------------------------------------------------------------------
//
// initial checking
//
//-----------------------------------------------------------------------------

   if (!usePeace1 && !usePeace2) return(_NO_VALID_PEACE_FILES_SPECIFIED);

   if (coordinateSystem == _GSE || coordinateSystem == _SR2 || coordinateSystem == _ISR2)
   {
      if (!useAuxGSE) return(_AUX_GSE_FILE_REQUIRED_FOR_COORD);
   }

//-----------------------------------------------------------------------------
//
// variable declarations
//
//-----------------------------------------------------------------------------

   QString beginDateTimePeace1, endDateTimePeace1;
   QString beginDateTimePeace2, endDateTimePeace2;
   QString beginDateTimePotential, endDateTimePotential, datasetIdPotential;
   QString beginDateTimeMagField, endDateTimeMagField, datasetIdMagField;
   QString beginDateTimeAuxGSE, endDateTimeAuxGSE, datasetIdAuxGSE;

   QString fileTimeSpanBegin, fileTimeSpanEnd;

   QString datasetIdPeace1 = "";
   QString datasetIdPeace2 = "";

   float lowerDeltaAuxGSE, upperDeltaAuxGSE;
   double beginTimeAuxGSE, endTimeAuxGSE;

   float currentMagneticField[3] = {0};
   float currentSpinAxisGseLat = -1, currentSpinAxisGseLong = -1;
   float angleTimeStampPeace = -1;
 
   float fillPotential = -1, fillMagField = -1, fillAuxGSE = -1;

   ReadFile peaceRead1, peaceRead2, potentialRead, magRead, gseRead;

   int iWhat1[50] = {0}, iWhat2[50] = {0};
   int iWhatP[50] = {0}, iWhatB[50] = {0}, iWhatG[50] = {0};
   int peaceFileType1 = -1, peaceFileType2 = -1;
   int potentialFileType = -1, magFieldFileType = -1, auxGSEFileType = -1;
   int peaceFileScn1 = -1, peaceFileScn2 = -1, potentialFileScn = -1, magFieldFileScn = -1, auxGSEFileScn = -1;

   int peaceFileSensors1 = -1, peaceFileSensors2 = -1; // specifies if we have data from one or two sensors in a single file

// initialize all What arrays to -1

   for (int i=0; i<50; i++)
   {
      iWhat1[i] = -1;
      iWhat2[i] = -1;
      iWhatP[i] = -1;
      iWhatB[i] = -1;
      iWhatG[i] = -1;
   }

// azimuth bins for PITCH_3D & PITCH_FULL only

   float aziBins[64], aziBinsLower[64], aziBinsUpper[64];

// list containing currently required S/C potential data

   QList<dtp> potlist;

//-----------------------------------------------------------------------------
//
// open cef file source for PEACE file 1
//
//-----------------------------------------------------------------------------

   if (dl) std::cout << "[peacemoments] open source for PEACE file 1" << std::endl;

   QString qObservatory;

   if (usePeace1)
   {
      if (peaceRead1.open(filePeace1) == _FAILED)
      {
         return(_FAILED_OPEN_FILE_IN_PEACE_1);
      }
      peaceRead1.readHeader();

      beginDateTimePeace1 = peaceRead1.getFileBeginDateTime();
      endDateTimePeace1 = peaceRead1.getFileEndDateTime();
      datasetIdPeace1 = peaceRead1.getGlobalAttr("DATASET_ID");
      peaceFileType1 = checkForFileType(datasetIdPeace1);
      peaceFileScn1 = returnSpacecraftNumber(datasetIdPeace1);

      if (peaceFileType1 != _LEEA && peaceFileType1 != _HEEA)
      {
         if (peaceFileType1 != _LEEA_HEEA) return(_NOT_VALID_PEACE_FILE);
         else
         {
            peaceFileSensors1 = _TWO_SENSOR;
         }
      }
      else peaceFileSensors1 = _ONE_SENSOR;
   
      getPeaceLocations(&peaceRead1, iWhat1);

      if (peaceFileSensors1 != _TWO_SENSOR)
      {
         QString units = peaceRead1.getVariableAttr(iWhat1[_Data], "UNITS");
         if (units != "s^3/km^6")
         {
	    return(_NOT_VALID_PEACE_UNITS);
         }
      }
      else
      {
         QString units = peaceRead1.getVariableAttr(iWhat1[_Data_Overlap_LEEA], "UNITS");
         if (units != "s^3/km^6")
         {
            return(_NOT_VALID_PEACE_UNITS);
         }  
      }
      
      qObservatory = peaceRead1.getGlobalAttr("OBSERVATORY");
      
   }

//-----------------------------------------------------------------------------
//
// open cef file source for PEACE file 2
//
//-----------------------------------------------------------------------------

   if (dl) std::cout << "[peacemoments] open source for PEACE file 2" << std::endl;

   if (usePeace2)
   {
      if (peaceRead2.open(filePeace2) == _FAILED)
      {
         return(_FAILED_OPEN_FILE_IN_PEACE_2);
      }
      peaceRead2.readHeader();

      beginDateTimePeace2 = peaceRead2.getFileBeginDateTime();
      endDateTimePeace2 = peaceRead2.getFileEndDateTime();
      datasetIdPeace2 = peaceRead2.getGlobalAttr("DATASET_ID");
      peaceFileType2 = checkForFileType(datasetIdPeace2);
      peaceFileScn2 = returnSpacecraftNumber(datasetIdPeace2);

      if (peaceFileType2 != _LEEA && peaceFileType2 != _HEEA)
      {
         if (peaceFileType2 != _LEEA_HEEA) return(_NOT_VALID_PEACE_FILE);
         else
         {
            peaceFileSensors2 = _TWO_SENSOR;
         }
      }
      else peaceFileSensors2 = _ONE_SENSOR;

      getPeaceLocations(&peaceRead2, iWhat2);

      if (peaceFileSensors2 == _ONE_SENSOR)
      {
         QString units = peaceRead2.getVariableAttr(iWhat2[_Data], "UNITS");
         if (units != "s^3/km^6")
         {
	    return(_NOT_VALID_PEACE_UNITS);
         }
      }
      else
      {
         return(_ONLY_SINGLE_FILE_REQUIRED);
      }
      
      qObservatory = peaceRead2.getGlobalAttr("OBSERVATORY");
   }

   if (!usePeace1 && !usePeace2) return(_NO_VALID_PEACE_FILES_SPECIFIED);

//-----------------------------------------------------------------------------
//
// some simple sensor combination testing
//
//-----------------------------------------------------------------------------

   if (sensorCombination == _LEEA && peaceFileType1 != _LEEA && peaceFileType2 != _LEEA)
   {
      return(_LEEA_DATA_REQUIRED_FOR_REQ_SENS_COMBI);
   }

   if (sensorCombination == _HEEA && peaceFileType1 != _HEEA && peaceFileType2 != _HEEA)
   {
      return(_HEEA_DATA_REQUIRED_FOR_REQ_SENS_COMBI);
   }

   if (sensorCombination == -1)
   {
      if (usePeace1 && !usePeace2) sensorCombination = peaceFileType1;
      if (!usePeace1 && usePeace2) sensorCombination = peaceFileType2;
   }

//-----------------------------------------------------------------------------
//
// open cef file source for S/C potential
//
//-----------------------------------------------------------------------------

   if (usePotential)
   {
      if (potentialRead.open(filePotential) == _FAILED)
      {
         return(_FAILED_OPEN_FILE_IN_POTENTIAL);
      }
      potentialRead.readHeader();

      beginDateTimePotential = potentialRead.getFileBeginDateTime();
      endDateTimePotential = potentialRead.getFileEndDateTime();
      datasetIdPotential = potentialRead.getGlobalAttr("DATASET_ID");
      if (datasetIdPotential == "") datasetIdPotential = potentialRead.getGlobalAttr("LOGICAL_SOURCE");
      potentialFileType = checkForFileType(datasetIdPotential);

      potentialFileScn = returnSpacecraftNumber(datasetIdPotential);

      if (potentialFileType != _POTENTIAL_SPIN && potentialFileType != _POTENTIAL_HIGH)
      {
	 return(_NOT_VALID_POTENTIAL_FILE);
      }

      getPotentialLocations(&potentialRead, iWhatP);
   }

//-----------------------------------------------------------------------------
//
// open cef file source for B-field
//
//-----------------------------------------------------------------------------

   if (useMagField)
   {
      if (magRead.open(fileMagField) == _FAILED)
      {
         return(_FAILED_OPEN_FILE_IN_MAG_FIELD);
      }
      magRead.readHeader();

      beginDateTimeMagField = magRead.getFileBeginDateTime();
      endDateTimeMagField = magRead.getFileEndDateTime();
      datasetIdMagField = magRead.getGlobalAttr("DATASET_ID");
      if (datasetIdMagField == "") datasetIdMagField = magRead.getGlobalAttr("LOGICAL_SOURCE");
      magFieldFileType = checkForFileType(datasetIdMagField);

      magFieldFileScn = returnSpacecraftNumber(datasetIdMagField);

      if (magFieldFileType != _BFIELD_SPIN && magFieldFileType != _BFIELD_HIGH)
      {
	 return(_NOT_VALID_MAG_FIELD_FILE);
      }

      getMagFieldLocations(&magRead, iWhatB);
   }

//-----------------------------------------------------------------------------
//
// open cef file source for AUX GSE
//
//-----------------------------------------------------------------------------

   if (useAuxGSE)
   {
      if (gseRead.open(fileAuxGSE) == _FAILED)
      {
         return(_FAILED_OPEN_FILE_IN_AUX_GSE);
      }
      gseRead.readHeader();

      beginDateTimeAuxGSE = gseRead.getFileBeginDateTime();
      endDateTimeAuxGSE = gseRead.getFileEndDateTime();
      datasetIdAuxGSE = gseRead.getGlobalAttr("DATASET_ID");
      if (datasetIdAuxGSE == "") datasetIdAuxGSE = gseRead.getGlobalAttr("LOGICAL_SOURCE");
      auxGSEFileType = checkForFileType(datasetIdAuxGSE);
      auxGSEFileScn = returnSpacecraftNumber(datasetIdAuxGSE);

      if (auxGSEFileScn == _FAILED) // the SP AUX files contain all 4 S/C
      {
         if (datasetIdAuxGSE == "CL_SP_AUX") auxGSEFileScn = peaceFileScn1;
      }

      if (auxGSEFileType != _GSE_SPINAXIS_SP)
      {
	 return(_NOT_VALID_AUX_GSE_FILE);
      }
      getAuxGSELocations(&gseRead, iWhatG);

   }

//-----------------------------------------------------------------------------
//
// checking before we attempt to calculate moments
//
//-----------------------------------------------------------------------------

   if (dl) std::cout << "[peacemoments] checking before calculations..." << std::endl;

// check that one file is LEEA and the other is HEEA if the user has specified two files

   if (usePeace1 && usePeace2)
   {
      if (peaceFileType1 == peaceFileType2)
      {
	 return(_NOT_VALID_TWO_PEACE_FILES);
      }
   }

// check that the date/time overlaps if the user has specified two files

   if (usePeace1 && usePeace2)
   {
      bool check = timeDateOverlapCheck(beginDateTimePeace1, endDateTimePeace1,
		                        beginDateTimePeace2, endDateTimePeace2);
      if (!check)
      {
	 return(_NOT_VALID_TIME_OVERLAP_PEACE);
      }
   }

// check the the two PEACE files are for the same spacecraft

   if (usePeace1 && usePeace2)
   {
      if (peaceFileScn1 != peaceFileScn2)
      {
         return(_NOT_VALID_SCN_PEACE_FILES);
      }
   }

// if used, check that the S/C potential file overlaps the PEACE data

   if (usePotential && usePeace1)
   {
      bool check = timeDateOverlapCheck(beginDateTimePeace1, endDateTimePeace1,
		                        beginDateTimePotential, endDateTimePotential);

      if (!check)
      {
	 return(_NOT_VALID_TIME_OVERLAP_POTENTIAL);
      }
   }

// if used, check that the S/C potential file is for the correct spacecraft
// - don't need this as there might be a reason for using a different spacecraft

   if (usePotential && usePeace1)
   {
      if (peaceFileScn1 != potentialFileScn)
      {
//         return(_NOT_VALID_SCN_POTENTIAL_FILE);
      }
   }

// if used, check that the AUX GSE file overlaps the PEACE data

   if (useAuxGSE && usePeace1)
   {
      bool check = timeDateOverlapCheck(beginDateTimePeace1, endDateTimePeace1,
		                        beginDateTimeAuxGSE, endDateTimeAuxGSE);

      if (!check)
      {
	 return(_NOT_VALID_TIME_OVERLAP_AUX_GSE);
      }
   }

// if used, check that the AUX GSE file contains data for the correct spacecraft

   if (useAuxGSE && usePeace1)
   {
      if (peaceFileScn1 != auxGSEFileScn)
      {
         return(_NOT_VALID_SCN_AUX_GSE_FILE);
      }
   }

// if used, check that the B-field file overlaps the PEACE data

   if (useMagField && usePeace1)
   {
      bool check = timeDateOverlapCheck(beginDateTimePeace1, endDateTimePeace1,
		                        beginDateTimeMagField, endDateTimeMagField);

      if (!check)
      {
	 return(_NOT_VALID_TIME_OVERLAP_MAG_FIELD);
      }
   }

// if used, check that the B-field file is for the correct spacecraft

   if (useMagField && usePeace1)
   {
      if (peaceFileScn1 != magFieldFileScn)
      {
         return(_NOT_VALID_SCN_MAG_FIELD_FILE);
      }
   }

//-----------------------------------------------------------------------------
//
// preparation before entering main loop
//
//-----------------------------------------------------------------------------

   if (dl) std::cout << "[peacemoments] preparation before main loop" << std::endl;

// determine begin & end times for FILE_TIME_SPAN

   if (usePeace1 && !usePeace2)
   {
      fileTimeSpanBegin = beginDateTimePeace1;
      fileTimeSpanEnd = endDateTimePeace1;
   }
   else if (!usePeace1 && usePeace2)
   {
      fileTimeSpanBegin = beginDateTimePeace2;
      fileTimeSpanEnd = endDateTimePeace2;
   }
   else if (usePeace1 && usePeace2)
   {
      if (qTimeToDec(beginDateTimePeace1) < qTimeToDec(beginDateTimePeace2))
         fileTimeSpanBegin = beginDateTimePeace1;
      else
         fileTimeSpanBegin = beginDateTimePeace2;

      if (qTimeToDec(endDateTimePeace1) > qTimeToDec(endDateTimePeace2))
         fileTimeSpanEnd = endDateTimePeace1;
      else
         fileTimeSpanEnd = endDateTimePeace2;

   }

// generate output filename

   QString outMomentsFile = fileName;

// read first record

   int firstUsePeace1 = -1, firstUsePeace2 = -1;
   int firstUsePotential = -1, firstUseMagField = -1, firstUseAuxGSE = -1;

   if (usePeace1) firstUsePeace1 = peaceRead1.readRecord();
   if (usePeace2) firstUsePeace2 = peaceRead2.readRecord();
   if (usePotential) firstUsePotential = potentialRead.readRecord();
   if (useMagField) firstUseMagField = magRead.readRecord();
   if (useAuxGSE) firstUseAuxGSE = gseRead.readRecord();

// check for errors

   if (usePeace1 && firstUsePeace1 != _SUCCESS)
   {
      return(_FAILED_READ_DATA_REC_PEACE_1);
   }

   if (usePeace2 && firstUsePeace2 != _SUCCESS)
   {
      return(_FAILED_READ_DATA_REC_PEACE_2);
   }

   if (usePotential && firstUsePotential != _SUCCESS)
   {
      return(_FAILED_READ_DATA_REC_POTENTIAL);
   }

   if (useMagField && firstUseMagField != _SUCCESS)
   {
      return(_FAILED_READ_DATA_REC_MAG_FIELD);
   }

   if (useAuxGSE && firstUseAuxGSE != _SUCCESS)
   {
      return(_FAILED_READ_DATA_REC_AUX_GSE);
   }

// timing

   QString timeDatePeace1, timeDatePeace2;
   QString timeDatePotential, timeDateMagField, timeDateAuxGSE;

   bool retval = true;
   if (usePeace1) retval = peaceRead1.getDataEpoch(iWhat1[_time_tags], &timeDatePeace1);
   if (usePeace2) retval = peaceRead2.getDataEpoch(iWhat2[_time_tags], &timeDatePeace2);
   if (usePotential) retval = potentialRead.getDataEpoch(iWhatP[_time_tags], &timeDatePotential);
   if (useMagField) retval = magRead.getDataEpoch(iWhatB[_time_tags], &timeDateMagField);

   if (useAuxGSE)
   {
      double ftmp8;
      float ftmp4;
      retval = gseRead.getDataEpoch(iWhatG[_time_tags], &timeDateAuxGSE);

      if (gseRead.getVariableDataType(iWhatG[_time_tags_HalfInterval]) == FLOAT)
      {
         retval = gseRead.getDataValue(iWhatG[_time_tags_HalfInterval], &ftmp4);
	 lowerDeltaAuxGSE = ftmp4;
      }
      else
      {
         retval = gseRead.getDataValue(iWhatG[_time_tags_HalfInterval], &ftmp8);
	 lowerDeltaAuxGSE = ftmp8;
      }
      upperDeltaAuxGSE = lowerDeltaAuxGSE;
      getSpinStartEnd(timeDateAuxGSE, (double)lowerDeltaAuxGSE, (double)upperDeltaAuxGSE,
		      &beginTimeAuxGSE, &endTimeAuxGSE);
   } 

// fill values

   if (usePotential)
   {
      fillPotential = potentialRead.getVariableAttr(iWhatP[_Spacecraft_potential], "FILLVAL").toFloat();
   }

   if (useMagField)
   {
      fillMagField = magRead.getVariableAttr(iWhatB[_MagneticField], "FILLVAL").toFloat();
   }

   if (useAuxGSE)
   {
      fillAuxGSE = gseRead.getVariableAttr(iWhatG[peaceFileScn1*2-1], "FILLVAL").toFloat();
   }

//=============================================================================
//
// main loop
// 
//=============================================================================

   if (dl) std::cout << "[peacemoments] about to enter main loop..." << std::endl;

   float potentialForPeace1[64] = {0}, potentialForPeace2[64] = {0};   
	 
// open an output cef file

   WriteCef outMoments(coordinateSystem);
   if (outMoments.open(outMomentsFile) == _FAILED)
   {
      return(_FAILED_OPEN_FILE_OUT_MOMENTS);
   }
   
// write header

   if (dl) std::cout << "[peacemoments] writing header..." << std::endl;

   outMoments.writeInitial(); 
   outMoments.writeFileName((QString)fileName);
   outMoments.writeMetaStatic();
   outMoments.writeObservatory(qObservatory);
   outMoments.writeExperiment();
   outMoments.writeGenerationDate();
   outMoments.writeMetaDatasetVersion();
   outMoments.writeMetaLogicalFileId((QString)fileName);
   outMoments.writeMetaVersionNumber(1);
   outMoments.writeMetaFileTimeSpan(fileTimeSpanBegin, fileTimeSpanEnd);
   outMoments.writeMetaMomentsSetup(datasetIdPeace1, datasetIdPeace2, sensorCombination);

   if (usePeace1) outMoments.writeMetaFileUsed("INPUT_FILE_PEACE_1", filePeace1);
   if (usePeace2) outMoments.writeMetaFileUsed("INPUT_FILE_PEACE_2", filePeace2);
   if (usePotential) outMoments.writeMetaFileUsed("INPUT_FILE_POTENTIAL", filePotential);
   if (useAuxGSE) outMoments.writeMetaFileUsed("INPUT_FILE_AUX_GSE", fileAuxGSE);
   if (useMagField) outMoments.writeMetaFileUsed("INPUT_FILE_MAG_FIELD", fileMagField);

   outMoments.writeMetaSetupPotential(potentialOffset1, potentialOffset2, potentialOffset2units, usePotential);
   outMoments.writeMetaSetupRestrInt(energyRestrictStatus, energyRestrictType, energyRestrictScpType,
                                     energyRestrictLow, energyRestrictHigh,
                                     polarRestrictStatus, polarRestrictLow, polarRestrictHigh);

   outMoments.writeTimeTagsMeta();
   outMoments.writeUsefulInfoHeader();

   if (WML.n) outMoments.writeDensityMeta();
   if (WML.v) outMoments.writeVelocityMeta();
   if (WML.vpara) outMoments.writeVelocityParaMeta();
   if (WML.vperp) outMoments.writeVelocityPerpMeta();
   if (WML.q) outMoments.writeEnergyFluxMeta();
   if (WML.qpara) outMoments.writeEnergyFluxParaMeta();
   if (WML.qperp) outMoments.writeEnergyFluxPerpMeta();
   if (WML.h) outMoments.writeHeatFluxMeta();
   if (WML.hpara) outMoments.writeHeatFluxParaMeta();
   if (WML.hperp) outMoments.writeHeatFluxPerpMeta();
   if (WML.p) outMoments.writePressureMeta();
   if (WML.t) outMoments.writeTempMeta();
   if (WML.tpara) outMoments.writeTempParaMeta();
   if (WML.tperp) outMoments.writeTempPerpMeta();
   if (WML.j) outMoments.writeCurrentMeta();
   if (WML.jpara) outMoments.writeCurrentParaMeta();
   if (WML.jperp) outMoments.writeCurrentPerpMeta();
   if (WML.ce) outMoments.writeCharacteristicEnergyMeta();
   if (WML.b) outMoments.writeMagFieldMeta();
   if (WML.scp) outMoments.writePotentialMeta();
   outMoments.writeMetaStatusVariables();
   outMoments.beginData(); 

// filePeace1 is used as the master time source. Other files, including a second
// PEACE file, are synchronized to filePeace1.

   double spinCentreTimeCurrent = -1;
   double spinCentreTimePrev = -1;
   double spinCentreTimeNext = -1;
   double spinDuration = -1;
   
   int currentProgress = 0;

   int counter = 0;
   bool stillRunning = true;
   while (stillRunning)
   {
      if (dl) std::cout << "[peacemoments] _TOP_OF_WHILE_LOOP_" << std::endl;

//-----------------------------------------------------------------------------
//
// if there are 2 PEACE files, synchronize them
//
//-----------------------------------------------------------------------------

      bool calculateMoments = true;
      bool fatalError = false;
      if (usePeace1 && usePeace2)
      {
         if (qTimeToDec(timeDatePeace1) < qTimeToDec(timeDatePeace2))
	 {
            bool failed = false, leave = false;
            while((qTimeToDec(timeDatePeace2) - qTimeToDec(timeDatePeace1)) > CLOSE_ENOUGH && !failed && !leave)
	    {
               int retNextRec = peaceRead1.readRecord();
	       if (retNextRec == _SUCCESS) retval = peaceRead1.getDataEpoch(iWhat1[_time_tags], &timeDatePeace1);
	       if (retNextRec != _SUCCESS || !retval) failed = true;
	       if (fabs(qTimeToDec(timeDatePeace1) - qTimeToDec(timeDatePeace2)) < CLOSE_ENOUGH) leave = true;
	    }
	 }
	 else if (qTimeToDec(timeDatePeace1) > qTimeToDec(timeDatePeace2))
	 {
            bool failed = false, leave = false;
	    while((qTimeToDec(timeDatePeace1) - qTimeToDec(timeDatePeace2)) > CLOSE_ENOUGH && !failed && !leave)
            {
               int retNextRec = peaceRead2.readRecord();
               if (retNextRec == _SUCCESS) retval = peaceRead2.getDataEpoch(iWhat2[_time_tags], &timeDatePeace2);
               if (retNextRec != _SUCCESS || !retval) failed = true;
	       if (fabs(qTimeToDec(timeDatePeace1) - qTimeToDec(timeDatePeace2)) < CLOSE_ENOUGH) leave = true;
            }
	 }

// final check that all is okay

         if (fabs(qTimeToDec(timeDatePeace1) - qTimeToDec(timeDatePeace2)) < CLOSE_ENOUGH)
            calculateMoments = true;
	 else calculateMoments = false;

// set the current spin centre time

         spinCentreTimeCurrent = qTimeToDec(timeDatePeace1);
         float ttDL, ttDU;
         peaceRead1.getDataValue(iWhat1[_time_tags_DeltaLower], &ttDL);
         peaceRead1.getDataValue(iWhat1[_time_tags_DeltaUpper], &ttDU);
         spinDuration = ttDL + ttDU;

      }
      else if (usePeace1 && !usePeace2) 
      {
         spinCentreTimeCurrent = qTimeToDec(timeDatePeace1);
         float ttDL, ttDU;
         peaceRead1.getDataValue(iWhat1[_time_tags_DeltaLower], &ttDL);
         peaceRead1.getDataValue(iWhat1[_time_tags_DeltaUpper], &ttDU);
         spinDuration = ttDL + ttDU;
      }
      else if (!usePeace1 && usePeace2)
      {
         spinCentreTimeCurrent = qTimeToDec(timeDatePeace2);
         float ttDL, ttDU;
         peaceRead2.getDataValue(iWhat2[_time_tags_DeltaLower], &ttDL);
         peaceRead2.getDataValue(iWhat2[_time_tags_DeltaUpper], &ttDU);
         spinDuration = ttDL + ttDU;
      }

      spinCentreTimeNext = spinCentreTimeCurrent + spinDuration/86400.0;

      if (dl) std::cout << "[peacemoments] RECORD date/time = " << timeDatePeace1.toStdString() << " : " << timeDatePeace2
.toStdString() << std::endl;

      //cout << endl << endl << endl << "RECORD date/time = " << timeDatePeace1.toStdString() << " : " << timeDatePeace2.toStdString() << " qual = " << calculateMoments << " || " << spinCentreTimeCurrent << " dur = " << spinDuration << endl;
      //cout << "   GSE = " << timeDateAuxGSE.toStdString() << " : FGM = " << timeDateMagField.toStdString() << " : EFW = " << timeDatePotential.toStdString() << endl;


// determine the current progress and emit a signal

      currentProgress = (int)(100.*(spinCentreTimeCurrent - qTimeToDec(fileTimeSpanBegin))/
                             (qTimeToDec(fileTimeSpanEnd) - qTimeToDec(fileTimeSpanBegin)));
			     
      emit progress(currentProgress);			     

//-----------------------------------------------------------------------------
//
// prepare all required data for moments
//
//-----------------------------------------------------------------------------    
 
      if (calculateMoments)
      {
         if (dl) std::cout << "[peacemoments] inside calculateMoments" << std::endl;

         int varSize_1[3] = {0}, realEnSize_1 = -1, sensor_1 = -1, varSize2_1[2] = {0};
	 int varSize_2[3] = {0}, realEnSize_2 = -1, sensor_2 = -1, varSize2_2[2] = {0};

	 int energyLevelLowLEEA = 0, energyLevelHighLEEA = 0;
	 int energyLevelLowHEEA = 0, energyLevelHighHEEA = 0;
	 
	 float time_tags_DeltaLower, time_tags_DeltaUpper; 
	 double spinStart, spinEnd;

	 double azimuthTimesPeace1[65], azimuthTimesPeace2[65];

	 int aBinsLEEA = 0, pBinsLEEA = 0, eBinsLEEA = 0, efBinsLEEA = 0;
	 int aBinsHEEA = 0, pBinsHEEA = 0, eBinsHEEA = 0, efBinsHEEA = 0;
         QString datasetIdPeaceL, datasetIdPeaceH;

	 bool usingLEEA = false, usingHEEA = false;

	 int resPeace1 = 0, resPeace2 = 0;

         float status1_Quality[3], status2_Quality[3];
         float status1_CountStats[3], status2_CountStats[3];
         float status1_EFW[3], status2_EFW[3];
         float status1_WHI[3], status2_WHI[3];
         float status1_DataStream[3], status2_DataStream[3];
         float status1_Eclipse[3], status2_Eclipse[3];

         float potentialSpinAvLEEA = FILL_VALUE_PM, potentialSpinAvHEEA = FILL_VALUE_PM;

         int actualEnSizeLEEA = -1, actualEnSizeHEEA = -1;

         int padTimingMaster = -1;

         if (usePeace1 && peaceFileSensors1 == _TWO_SENSOR)
         {
            bool r1, r2, r3, r4, r5, r6;
            r1 = peaceRead1.getDataValue(iWhat1[_Mode_RealSize_Overlap_LEEA], &varSize_1[0]);
            r2 = peaceRead1.getDataValue(iWhat1[_Mode_EnergyLevelRange_LEEA], &varSize2_1[0]);
            r3 = peaceRead1.getDataValue(iWhat1[_Mode_RealBinNum_Overlap_LEEA], &realEnSize_1);
            r4 = peaceRead1.getDataValue(iWhat1[_time_tags_DeltaLower], &time_tags_DeltaLower);
            r5 = peaceRead1.getDataValue(iWhat1[_time_tags_DeltaUpper], &time_tags_DeltaUpper);
            r6 = peaceRead1.getDataValue(iWhat1[_Mode_Sensor], &sensor_1);
            int tSize = peaceRead1.getVariableSize(iWhat1[_Energy_Overlap_LEEA], 0);

            // check that Mode_Sensor is valid; if it's a fill value then something is wrong

            if (sensor_1 != 0 && sensor_1 != 1 && sensor_1 != 2) fatalError = true;

            // decide what sensor to use for timing

            if (sensor_1 == 2) padTimingMaster = 1;
            else if (sensor_1 == 1) padTimingMaster = 0;
            else if (sensor_1 == 0) padTimingMaster = 0;

            // get the azimthal angles, assuming that overlap LEEA & HEEA azimuthal angles are
            // the same. This is possibly NOT TRUE if HAR sweep mode is used and we use a 
            // different set of results (remember with HAR we have 4 azimuths, not 2)
            
            if (padTimingMaster == 1)
            {
               peaceRead1.getDataValue(iWhat1[_Azimuth_Overlap_HEEA], aziBins);
               peaceRead1.getDataValue(iWhat1[_Azimuth_Overlap_HEEA_DeltaLower], aziBinsLower);
               peaceRead1.getDataValue(iWhat1[_Azimuth_Overlap_HEEA_DeltaUpper], aziBinsUpper);
            }
            else
            {
               peaceRead1.getDataValue(iWhat1[_Azimuth_Overlap_LEEA], aziBins);
               peaceRead1.getDataValue(iWhat1[_Azimuth_Overlap_LEEA_DeltaLower], aziBinsLower);
               peaceRead1.getDataValue(iWhat1[_Azimuth_Overlap_LEEA_DeltaUpper], aziBinsUpper);
            }


            usingLEEA = true;
            aBinsLEEA = varSize_1[0];
            pBinsLEEA = varSize_1[1];
            eBinsLEEA = varSize_1[2];
            efBinsLEEA = realEnSize_1;
            energyLevelLowLEEA = varSize2_1[1];
            energyLevelHighLEEA = varSize2_1[0];
            datasetIdPeaceL = datasetIdPeace1;
            actualEnSizeLEEA = tSize;

            bool r1a, r2a, r3a;
            r1a = peaceRead1.getDataValue(iWhat1[_Mode_RealSize_Overlap_HEEA], &varSize_2[0]);
            r2a = peaceRead1.getDataValue(iWhat1[_Mode_EnergyLevelRange_HEEA], &varSize2_2[0]);
            r3a = peaceRead1.getDataValue(iWhat1[_Mode_RealBinNum_Overlap_HEEA], &realEnSize_2);
            tSize = peaceRead1.getVariableSize(iWhat1[_Energy_Overlap_HEEA], 0);

            usingHEEA = true;
            aBinsHEEA = varSize_2[0];
            pBinsHEEA = varSize_2[1];
            eBinsHEEA = varSize_2[2];
            efBinsHEEA = realEnSize_2;
            energyLevelLowHEEA = varSize2_2[1];
            energyLevelHighHEEA = varSize2_2[0];
            datasetIdPeaceH = datasetIdPeace2;
            actualEnSizeHEEA = tSize;
         }
	 
         if (usePeace1 && peaceFileSensors1 == _ONE_SENSOR)
	 {
            bool r1, r2, r3, r4, r5, r6;
            r1 = peaceRead1.getDataValue(iWhat1[_Mode_RealSize], &varSize_1[0]);
            r2 = peaceRead1.getDataValue(iWhat1[_Mode_EnergyLevelRange], &varSize2_1[0]);
            r3 = peaceRead1.getDataValue(iWhat1[_Mode_RealBinNum], &realEnSize_1);	 
            r4 = peaceRead1.getDataValue(iWhat1[_time_tags_DeltaLower], &time_tags_DeltaLower);
            r5 = peaceRead1.getDataValue(iWhat1[_time_tags_DeltaUpper], &time_tags_DeltaUpper);	
	    r6 = peaceRead1.getDataValue(iWhat1[_Mode_Sensor], &sensor_1);     
	    int tSize = peaceRead1.getVariableSize(iWhat1[_Energy], 0);

	    if (sensor_1 == _LEEA)
	    {
               usingLEEA = true;
	       aBinsLEEA = varSize_1[0];
	       pBinsLEEA = varSize_1[1];
	       eBinsLEEA = varSize_1[2];
	       efBinsLEEA = realEnSize_1;
	       energyLevelLowLEEA = varSize2_1[1];
	       energyLevelHighLEEA = varSize2_1[0];
	       datasetIdPeaceL = datasetIdPeace1;
	       actualEnSizeLEEA = tSize;
	    }
	    else
	    {
               usingHEEA = true;
	       aBinsHEEA = varSize_1[0];
	       pBinsHEEA = varSize_1[1];
	       eBinsHEEA = varSize_1[2];
	       efBinsHEEA = realEnSize_1;
	       energyLevelLowHEEA = varSize2_1[1];
	       energyLevelHighHEEA = varSize2_1[0];
	       datasetIdPeaceH = datasetIdPeace1;
	       actualEnSizeHEEA = tSize;
	    }
	    if (r1 && r2 && r3 && r4 && r5 && r6) resPeace1 = _SUCCESS;
	    else resPeace1 = _FAILED;
	 }
	 if (usePeace2 && peaceFileSensors2 == _ONE_SENSOR)
	 {
            peaceRead2.getDataValue(iWhat2[_Status_CountsStats], status2_CountStats);
            peaceRead2.getDataValue(iWhat2[_Status_EFW], status2_EFW);
            peaceRead2.getDataValue(iWhat2[_Status_WHI], status2_WHI);
            peaceRead2.getDataValue(iWhat2[_Status_DataStream], status2_DataStream);

            bool r1, r2, r3, r4, r5, r6;
            r1 = peaceRead2.getDataValue(iWhat2[_Mode_RealSize], &varSize_2[0]);
            r2 = peaceRead2.getDataValue(iWhat2[_Mode_EnergyLevelRange], &varSize2_2[0]);
            r3 = peaceRead2.getDataValue(iWhat2[_Mode_RealBinNum], &realEnSize_2);	 
            r4 = peaceRead2.getDataValue(iWhat2[_time_tags_DeltaLower], &time_tags_DeltaLower);
            r5 = peaceRead2.getDataValue(iWhat2[_time_tags_DeltaUpper], &time_tags_DeltaUpper);
	    r6 = peaceRead2.getDataValue(iWhat2[_Mode_Sensor], &sensor_2); 	    
	    int tSize = peaceRead2.getVariableSize(iWhat2[_Energy], 0);

            if (sensor_2 == _LEEA)
            {
               usingLEEA = true;
	       aBinsLEEA = varSize_2[0];
	       pBinsLEEA = varSize_2[1];
	       eBinsLEEA = varSize_2[2];
	       efBinsLEEA = realEnSize_2;
	       energyLevelLowLEEA = varSize2_2[1];
	       energyLevelHighLEEA = varSize2_2[0];
	       datasetIdPeaceL = datasetIdPeace2;
	       actualEnSizeLEEA = tSize;
	    }
            else
            {
               usingHEEA = true;
	       aBinsHEEA = varSize_2[0];
               pBinsHEEA = varSize_2[1];
               eBinsHEEA = varSize_2[2];
               efBinsHEEA = realEnSize_2;
	       energyLevelLowHEEA = varSize2_2[1];
	       energyLevelHighHEEA = varSize2_2[0];
	       datasetIdPeaceH = datasetIdPeace2;
	       actualEnSizeHEEA = tSize;
            }
	    if (r1 && r2 && r3 && r4 && r5 && r6) resPeace2 = _SUCCESS;
            else resPeace2 = _FAILED;
	 }

         if (dl) std::cout << "[peacemoments] initializing PeaceData peaceSpin" << std::endl;
         PeaceData peaceSpin(usingLEEA, usingHEEA, actualEnSizeLEEA, actualEnSizeHEEA,
	                     aBinsLEEA, pBinsLEEA, eBinsLEEA, efBinsLEEA, 
			     energyLevelLowLEEA, energyLevelHighLEEA, datasetIdPeaceL,
                             aBinsHEEA, pBinsHEEA, eBinsHEEA, efBinsHEEA, 
			     energyLevelLowHEEA, energyLevelHighHEEA, datasetIdPeaceH);

         peaceSpin.padTimingMaster = padTimingMaster;

         peaceSpin.testingIntValue = oneCountValue;

	 if (usePeace1 && peaceFileSensors1 == _ONE_SENSOR)
	 {
            if (sensor_1 == _LEEA) putPeaceDataIntoLEEA(&peaceRead1, &peaceSpin, iWhat1, modeBackgroundLevel);
	    else putPeaceDataIntoHEEA(&peaceRead1, &peaceSpin, iWhat1, modeBackgroundLevel);
	 }
         else if (usePeace1 && peaceFileSensors1 == _TWO_SENSOR)
         {
            putPeaceDataIntoLEEAo(&peaceRead1, &peaceSpin, iWhat1);
            putPeaceDataIntoHEEAo(&peaceRead1, &peaceSpin, iWhat1);
         }
      
         if (usePeace2 && peaceFileSensors2 == _ONE_SENSOR)
	 {
            if (sensor_2 == _LEEA) putPeaceDataIntoLEEA(&peaceRead2, &peaceSpin, iWhat2, modeBackgroundLevel);
	    else putPeaceDataIntoHEEA(&peaceRead2, &peaceSpin, iWhat2, modeBackgroundLevel);
	 }

// determine timestamps for the PEACE spin start & end

         getSpinStartEnd(timeDatePeace1, time_tags_DeltaLower, time_tags_DeltaUpper,
                         &spinStart, &spinEnd);

   //cout << "time = " << spinStart << " : " << spinEnd << " from " << time_tags_DeltaLower << " : " << time_tags_DeltaUpper << endl;

// get the SR2 angle of the timestamp or the SPOS if it exists

	 if (iWhat1[_Mode_SPOS] == -1)
	 {

// SPOS is not in the PEACE data file, so determine it from Angle_SR2phi. Note that
// this will not be exact.

            peaceRead1.getDataValue(iWhat1[_Angle_SR2phi], &angleTimeStampPeace);
	    double SUNAZ = 180.0 - angleTimeStampPeace - 30.0;
	    if (sensor_1 == _LEEA) SUNAZ = SUNAZ + 180.0;
	    peaceSpin.spos = (26.2 - SUNAZ)/(360.0/(16.0*1024.0));
	 }
	 else
	 {

// use the actual SPOS

            int actualSPOS;
	    peaceRead1.getDataValue(iWhat1[_Mode_SPOS], &actualSPOS);
            peaceSpin.spos = (double)actualSPOS;
         }

//-----------------------------------------------------------------------------
//
// if required, synchronize the potential data to the PEACE data
//
//-----------------------------------------------------------------------------

         if (usePotential)
         {

// interval for which we want to have S/C potential data

            double current_interval_begin = spinStart - 8.0/86400.0;
            double current_interval_end = spinEnd + 8.0/86400.0;

// add S/C potential data to the list up until the end of current interval under 
// consideration, or until we run out of data

            bool p_done = false;

            // firstly, check if we really need to do anything

            potentialRead.getDataEpoch(iWhatP[_time_tags], &timeDatePotential);
            
            if (qTimeToDec(timeDatePotential) > current_interval_end) p_done = true;

            while (!p_done)
            {
               potentialRead.getDataEpoch(iWhatP[_time_tags], &timeDatePotential);
               double p_datetime = qTimeToDec(timeDatePotential);
               float f_pval;
               potentialRead.getDataValue(iWhatP[_Spacecraft_potential], &f_pval);
               if (f_pval == fillPotential) f_pval = FILL_VALUE_PM;

// check if we should leave the while loop
               
               if (p_datetime > current_interval_end) p_done = true;

// add new data to the list if necessary

               if (p_datetime >= current_interval_begin && p_datetime < current_interval_end)
               {
//cout << "ADDING TO LIST : " << p_datetime << " / " << f_pval << endl;
                  dtp dtp_tmp;
                  dtp_tmp.datetime = p_datetime;
                  dtp_tmp.potential = f_pval;
                  potlist.prepend(dtp_tmp);
               }

               if (potentialRead.readRecord() != _SUCCESS) p_done = true;
            }

// remove data no longer required from the list

            if (!potlist.isEmpty())
            {
               p_done = false;
               while (!p_done)
               {
                  if (potlist.last().datetime < current_interval_begin && potlist.size() > 2) potlist.removeLast();
                  else p_done = true;
               }
            }

// now we have the required S/C potential data. The next step is do determine the required potential for
// each azimuth. This may involve either linear interpolation, or for high time resolution potentials,
// averaging all FGM within a single PEACE azimuth bin.


           //cout << "CURRENT POTENTIAL DATA::" << endl;

/*
            for (int i=0; i< potlist.size(); ++i)
               cout << i << " : " << potlist.at(i).datetime << " / " << potlist.at(i).potential << endl;
*/


            if (usePeace1)
            {
               int aBins;
               if (sensor_1 == _LEEA) aBins = aBinsLEEA;
               else aBins = aBinsHEEA;
               setAzimuthTimes(spinStart, spinEnd, aBins, azimuthTimesPeace1);
	       
	       for (int ia=0; ia<aBins; ia++)
	       {
	          double azimuthBegin = azimuthTimesPeace1[ia];
		  double azimuthEnd = azimuthTimesPeace1[ia+1];

// if possible, average all potential data during the current azimuth under consideration

                  double ptmp = 0.0;
                  int itmp = 0;

                  for (int ip=0; ip<potlist.size(); ip++) 
                  {
                     if (potlist.at(ip).datetime >= azimuthBegin && potlist.at(ip).datetime < azimuthEnd &&
                         fabs(potlist.at(ip).potential) < MAX_VALUE_PM)
                     {
                        itmp++;
                        ptmp += potlist.at(ip).potential;
                     }
                  }                 

                  potentialForPeace1[ia] = (itmp > 0) ? ptmp/((double)itmp) : ptmp;

// otherwise, interpolate nearest potential values just before & after the azimuth centre time

                  if (itmp == 0)
                  {
                     double potentialBefore = FILL_VALUE_PM, potentialAfter = FILL_VALUE_PM;
                     double timeBefore = 0, timeAfter = 0;
                     
// find the potential value just before the current azimuth

                     p_done = false;
                     itmp = 0;
                     while (!p_done && itmp < potlist.size())
                     {
                        if (potlist.at(itmp).datetime <= azimuthBegin &&
                            fabs(potlist.at(itmp).potential) < MAX_VALUE_PM)
                        {
                           potentialBefore = potlist.at(itmp).potential;
                           timeBefore = potlist.at(itmp).datetime;
                           p_done = true;
                        }
                        else
                        {
                           itmp++;
                        }
                     }

// find the potential value just after the current azimuth

                     p_done = false;
                     itmp = potlist.size() - 1;
                     while (!p_done && itmp >= 0)
                     {
                        if (potlist.at(itmp).datetime >= azimuthEnd &&
                            fabs(potlist.at(itmp).potential) < MAX_VALUE_PM)
                        {
                           potentialAfter = potlist.at(itmp).potential;
                           timeAfter = potlist.at(itmp).datetime;
                           p_done = true;
                        }
                        else
                        {
                           itmp--;
                        }
                     }

// interpolate if required

                     if (fabs(potentialBefore) < MAX_VALUE_PM && fabs(potentialAfter) < MAX_VALUE_PM)
                     {
                        potentialForPeace1[ia] = interpolateLinear(timeBefore, timeAfter,
                                                                   potentialBefore, potentialAfter,
                                                                   azimuthBegin); // using begin time to be consistent with PEACEmoments IDFS
                     }
                     else potentialForPeace1[ia] = FILL_VALUE_PM;

                  }

	       }

// copy data to the main storage array
 
               if (sensor_1 == _LEEA)
               {
                  potentialSpinAvLEEA = 0;
                  int iccc = 0;
                  for (int id=0; id<aBinsLEEA; id++)
                  {
                     peaceSpin.potentialLEEA[id] = potentialForPeace1[id];
//cout << "STORE>> " << id << " : " << peaceSpin.potentialLEEA[id] << endl;
                     if (fabs(potentialForPeace1[id]) < MAX_VALUE_PM)
                     {
                        iccc++;
                        potentialSpinAvLEEA += potentialForPeace1[id];
                     }
                  }
                  potentialSpinAvLEEA = (iccc>0) ? potentialSpinAvLEEA/((float)iccc) : potentialSpinAvLEEA;
               } 
               else
               {
                  potentialSpinAvLEEA = 0;
                  int iccc = 0;
                  for (int id=0; id<aBinsHEEA; id++)
                  {
                     peaceSpin.potentialHEEA[id] = potentialForPeace1[id];
                     if (fabs(potentialForPeace1[id]) < MAX_VALUE_PM)
                     {
                        iccc++; 
                        potentialSpinAvHEEA += potentialForPeace1[id];
                     }
                  }
                  potentialSpinAvHEEA = (iccc>0) ? potentialSpinAvHEEA/((float)iccc) : potentialSpinAvHEEA;
               }
            } // <-- end if using peace1
            
            if (usePeace2)
            {
               int aBins;
               if (sensor_2 == _LEEA) aBins = aBinsLEEA;
               else aBins = aBinsHEEA;
               setAzimuthTimes(spinStart, spinEnd, aBins, azimuthTimesPeace2);
	       
	       for (int ia=0; ia<aBins; ia++)
	       {
	          double azimuthBegin = azimuthTimesPeace2[ia];
		  double azimuthEnd = azimuthTimesPeace2[ia+1];

// if possible, average all potential data during the current azimuth under consideration

                  double ptmp = 0.0;
                  int itmp = 0;

                  for (int ip=0; ip<potlist.size(); ip++) 
                  {
                     if (potlist.at(ip).datetime >= azimuthBegin && potlist.at(ip).datetime < azimuthEnd &&
                         fabs(potlist.at(ip).potential) < MAX_VALUE_PM)
                     {
                        itmp++;
                        ptmp += potlist.at(ip).potential;
                     }
                  }                 

                  potentialForPeace2[ia] = (itmp > 0) ? ptmp/((double)itmp) : ptmp;

// otherwise, interpolate nearest potential values just before & after the azimuth centre time

                  if (itmp == 0)
                  {
                     double potentialBefore = FILL_VALUE_PM, potentialAfter = FILL_VALUE_PM;
                     double timeBefore = 0, timeAfter = 0;
                     
// find the potential value just before the current azimuth

                     p_done = false;
                     itmp = 0;
                     while (!p_done && itmp < potlist.size())
                     {
                        if (potlist.at(itmp).datetime <= azimuthBegin &&
                            fabs(potlist.at(itmp).potential) < MAX_VALUE_PM)
                        {
                           potentialBefore = potlist.at(itmp).potential;
                           timeBefore = potlist.at(itmp).datetime;
                           p_done = true;
                        }
                        else
                        {
                           itmp++;
                        }
                     }

// find the potential value just after the current azimuth

                     p_done = false;
                     itmp = potlist.size() - 1;
                     while (!p_done && itmp >= 0)
                     {
                        if (potlist.at(itmp).datetime >= azimuthEnd &&
                            fabs(potlist.at(itmp).potential) < MAX_VALUE_PM)
                        {
                           potentialAfter = potlist.at(itmp).potential;
                           timeAfter = potlist.at(itmp).datetime;
                           p_done = true;
                        }
                        else
                        {
                           itmp--;
                        }
                     }

// interpolate if required

                     if (fabs(potentialBefore) < MAX_VALUE_PM && fabs(potentialAfter) < MAX_VALUE_PM)
                     {
                        potentialForPeace2[ia] = interpolateLinear(timeBefore, timeAfter,
                                                                   potentialBefore, potentialAfter,
                                                                   azimuthBegin); // using begin time to be consistent with PEACEmoments IDFS
                     }
                     else potentialForPeace2[ia] = FILL_VALUE_PM;

                  }

	       }

// copy data to the main storage array
 
               if (sensor_2 == _LEEA)
               {
                  potentialSpinAvLEEA = 0;
                  int iccc = 0;
                  for (int id=0; id<aBinsLEEA; id++)
                  {
                     peaceSpin.potentialLEEA[id] = potentialForPeace2[id];
//cout << "STORE>> " << id << " : " << peaceSpin.potentialLEEA[id] << endl;
                     if (fabs(potentialForPeace2[id]) < MAX_VALUE_PM)
                     {
                        iccc++;
                        potentialSpinAvLEEA += potentialForPeace2[id];
                     }
                  }
                  potentialSpinAvLEEA = (iccc>0) ? potentialSpinAvLEEA/((float)iccc) : potentialSpinAvLEEA;
               } 
               else
               {
                  potentialSpinAvLEEA = 0;
                  int iccc = 0;
                  for (int id=0; id<aBinsHEEA; id++)
                  {
                     peaceSpin.potentialHEEA[id] = potentialForPeace2[id];
                     if (fabs(potentialForPeace1[id]) < MAX_VALUE_PM)
                     {
                        iccc++; 
                        potentialSpinAvHEEA += potentialForPeace2[id];
                     }
                  }
                  potentialSpinAvHEEA = (iccc>0) ? potentialSpinAvHEEA/((float)iccc) : potentialSpinAvHEEA;
               }
            } // <-- end if using peace2
            
            
         } // <-- end if using potential

//-----------------------------------------------------------------------------
//       
// if required, synchronize the FGM B-field data to the PEACE data
//
//-----------------------------------------------------------------------------

	 if (useMagField && usePeace1)
	 {
            if (fabs(qTimeToDec(timeDatePeace1) - qTimeToDec(timeDateMagField)) > CLOSE_ENOUGH)
	    {
               if (qTimeToDec(timeDateMagField) < qTimeToDec(timeDatePeace1))
	       {
                  bool failed = false, leave = false;
		  while (fabs(qTimeToDec(timeDatePeace1) - qTimeToDec(timeDateMagField)) > CLOSE_ENOUGH && !failed && !leave)
		  {
                     int retNextRec = magRead.readRecord();
                     if (retNextRec == _SUCCESS) magRead.getDataEpoch(iWhatB[_time_tags], &timeDateMagField);
                     if (retNextRec != _SUCCESS || !retval) failed = true;
		     if (fabs(qTimeToDec(timeDatePeace1) - qTimeToDec(timeDateMagField)) < CLOSE_ENOUGH) leave = true;
		     if (qTimeToDec(timeDateMagField) > qTimeToDec(timeDatePeace1)) leave = true;
		  }
	       }
	    }

// check that all is okay - if it is, set the B-field in the PeaceData class. If not,
// use a fill value to ensure B-field aligned quantities are not calculated.

            if (fabs(qTimeToDec(timeDatePeace1) - qTimeToDec(timeDateMagField)) < CLOSE_ENOUGH)
	    {
               retval = magRead.getDataValue(iWhatB[_MagneticField], currentMagneticField);
	       for (int i=0; i < 3; i++)
	       {           
                  peaceSpin.fgmB[i] =
                    (currentMagneticField[i] != fillMagField) ? currentMagneticField[i] : FILL_VALUE_PM;
               } 
	    }
	    else
	    {
               for (int i=0; i<3; i++)
               {     
                  peaceSpin.fgmB[i] = FILL_VALUE_PM;
               } 
	    }
	 }

	 //cout << "   FGM = " << timeDateMagField.toStdString() << " " << peaceSpin.fgmB[0] << " : " << peaceSpin.fgmB[1] << " : " << peaceSpin.fgmB[2] << endl;

//-----------------------------------------------------------------------------
//       
// if required, synchronize the AUX GSE data to the PEACE data
//          
//-----------------------------------------------------------------------------

	 if (useAuxGSE && usePeace1)
	 {

// check if the current PEACE spin lies within the current AUX GSE record time span

            if (qTimeToDec(timeDatePeace1) >= beginTimeAuxGSE && qTimeToDec(timeDatePeace1) <= endTimeAuxGSE)
	    {

// everything is okay, we don't need to do anything

	    }
	    else
	    {

// the PEACE data is ahead of the AUX GSE data, so increment the AUX GSE data

               if (endTimeAuxGSE < qTimeToDec(timeDatePeace1))
	       {
                  bool failed = false, leave = false;
		  while (endTimeAuxGSE < qTimeToDec(timeDatePeace1) && !failed && !leave)
		  {
                     int retNextRec = gseRead.readRecord();
		     if (retNextRec == _SUCCESS) retval = gseRead.getDataEpoch(iWhatG[_time_tags], &timeDateAuxGSE);
		     if (retNextRec != _SUCCESS || !retval) failed = true;
		     if (gseRead.getVariableDataType(iWhatG[_time_tags_HalfInterval]) == FLOAT)
		     {
                        float ftmp4;
			retval = gseRead.getDataValue(iWhatG[_time_tags_HalfInterval], &ftmp4);
			lowerDeltaAuxGSE = ftmp4;
		     }
		     else
		     {
                        double ftmp8;
			retval = gseRead.getDataValue(iWhatG[_time_tags_HalfInterval], &ftmp8);
			lowerDeltaAuxGSE = ftmp8;
		     }
		     lowerDeltaAuxGSE = lowerDeltaAuxGSE;
		     upperDeltaAuxGSE = lowerDeltaAuxGSE;
		     getSpinStartEnd(timeDateAuxGSE, (double)lowerDeltaAuxGSE, (double)upperDeltaAuxGSE,
		                     &beginTimeAuxGSE, &endTimeAuxGSE);
		     if (endTimeAuxGSE > qTimeToDec(timeDatePeace1)) leave = true;

		  }
	       }
	    }

// final check that all is okay

            if (qTimeToDec(timeDatePeace1) >= beginTimeAuxGSE && qTimeToDec(timeDatePeace1) <= endTimeAuxGSE)
            {
               retval = gseRead.getDataValue(iWhatG[peaceFileScn1*2-1], &currentSpinAxisGseLat);
	       retval = gseRead.getDataValue(iWhatG[peaceFileScn1*2], &currentSpinAxisGseLong);
	       peaceSpin.spinAxisLatitudeGSE = currentSpinAxisGseLat;
	       peaceSpin.spinAxisLongitudeGSE = currentSpinAxisGseLong;
            }
	    else
	    {
               peaceSpin.spinAxisLatitudeGSE = FILL_VALUE_PM;
	       peaceSpin.spinAxisLongitudeGSE = FILL_VALUE_PM;
	    }

	 }
	 //cout << "   GSE = " << timeDateAuxGSE.toStdString() << " vals (lat/long) = " << currentSpinAxisGseLat << " : " << currentSpinAxisGseLong << " filescn1 = " << peaceFileScn1 << endl;

//-----------------------------------------------------------------------------
//
// calculate moments   
//
//-----------------------------------------------------------------------------

// get Status information

         int Status_EFW=-1, Status_WHI=-1, Status_QUA = -1, Status_STA = -1, Status_DSE = -1;
	 int lStatus_EFW=-1, lStatus_WHI=-1, lStatus_QUA = -1, lStatus_STA = -1, lStatus_DSE = -1;
	 int hStatus_EFW=-1, hStatus_WHI=-1, hStatus_QUA = -1, hStatus_STA = -1, hStatus_DSE = -1;
         int Status_ECL=-1, lStatus_ECL=-1, hStatus_ECL=-1;

         if (usePeace1)
         {
            peaceRead1.getDataValue(iWhat1[_Status_Quality], status1_Quality);
            peaceRead1.getDataValue(iWhat1[_Status_CountsStats], status1_CountStats);
            peaceRead1.getDataValue(iWhat1[_Status_EFW], status1_EFW);
            peaceRead1.getDataValue(iWhat1[_Status_WHI], status1_WHI);
            peaceRead1.getDataValue(iWhat1[_Status_DataStream], status1_DataStream);
            peaceRead1.getDataValue(iWhat1[_Status_Eclipse], status1_Eclipse);

            Status_QUA = status1_Quality[0];
            Status_STA = status1_CountStats[0];
            Status_EFW = status1_EFW[0];
            Status_WHI = status1_WHI[0];
            Status_DSE = status1_DataStream[0];
            Status_ECL = status1_Eclipse[0];
	    
	    if ((sensor_1 == _LEEA && peaceFileSensors1 == _ONE_SENSOR) || peaceFileSensors1 == _TWO_SENSOR)
	    {
	       lStatus_QUA = status1_Quality[0];
	       lStatus_WHI = status1_WHI[0];
	       lStatus_EFW = status1_EFW[0];
	       lStatus_STA = status1_CountStats[0];
	       lStatus_DSE = status1_DataStream[0];
               lStatus_ECL = status1_Eclipse[0];
            }
	    else
	    {
	       hStatus_QUA = status1_Quality[0];
	       hStatus_WHI = status1_WHI[0];
	       hStatus_EFW = status1_EFW[0];
	       hStatus_STA = status1_CountStats[0];
	       hStatus_DSE = status1_DataStream[0];	    
               hStatus_ECL = status1_Eclipse[0];
	    }
         }
         if (usePeace2)
         {
            peaceRead2.getDataValue(iWhat2[_Status_Quality], status2_Quality);
            peaceRead2.getDataValue(iWhat2[_Status_CountsStats], status2_CountStats);
            peaceRead2.getDataValue(iWhat2[_Status_EFW], status2_EFW);
            peaceRead2.getDataValue(iWhat2[_Status_WHI], status2_WHI);
            peaceRead2.getDataValue(iWhat2[_Status_DataStream], status2_DataStream);
            peaceRead2.getDataValue(iWhat2[_Status_Eclipse], status2_Eclipse);
            if (status2_Quality[0] != -1 && status2_Quality[0] > 0) Status_QUA = status2_Quality[0];
            if (status2_CountStats[0] != -1 && status2_CountStats[0] > 0) Status_STA = status2_CountStats[0];
            if (status2_EFW[0] != -1 && status2_EFW[0] > 0) Status_EFW = status2_EFW[0];
            if (status2_WHI[0] != -1 && status2_WHI[0] > 0) Status_WHI = status2_WHI[0];
            if (status2_DataStream[0] != -1 && status2_DataStream[0] > 0) Status_DSE = status2_DataStream[0];
            if (status2_Eclipse[0] != -1 && status2_Eclipse[0] == 1) Status_ECL = status2_Eclipse[0];

//cout << "L, h = " << lStatus_ECL << " : " << hStatus_ECL << endl;
	    
	    if (sensor_2 == _LEEA)
	    {
	       lStatus_QUA = status2_Quality[0];
	       lStatus_WHI = status2_WHI[0];
	       lStatus_EFW = status2_EFW[0];
	       lStatus_STA = status2_CountStats[0];
	       lStatus_DSE = status2_DataStream[0];
               lStatus_ECL = status2_Eclipse[0];
            }
	    else
	    {
	       hStatus_QUA = status2_Quality[0];
	       hStatus_WHI = status2_WHI[0];
	       hStatus_EFW = status2_EFW[0];
	       hStatus_STA = status2_CountStats[0];
	       hStatus_DSE = status2_DataStream[0];	    
               hStatus_ECL = status2_Eclipse[0];
	    }	    
         }

// set spin-averaged potential

         if (fabs(potentialSpinAvLEEA) < MAX_VALUE_PM) peaceSpin.potentialSpinAv = -potentialSpinAvLEEA;
         else if (fabs(potentialSpinAvHEEA) < MAX_VALUE_PM) peaceSpin.potentialSpinAv = -potentialSpinAvHEEA;
         else peaceSpin.potentialSpinAv = FILL_VALUE_PM;

//cout << "pot = " << peaceSpin.potentialSpinAv << endl;

// settings

         peaceSpin.potentialOffset1 = potentialOffset1;
	 peaceSpin.potentialOffset2 = potentialOffset2;
	 peaceSpin.potentialOffset2units = potentialOffset2units;

         peaceSpin.setRestrictedEnergyIntegration(energyRestrictStatus, energyRestrictType, energyRestrictScpType,
                                                  energyRestrictLow, energyRestrictHigh);
         peaceSpin.setRestrictedPolarIntegration(polarRestrictStatus, polarRestrictLow, polarRestrictHigh);

// perform integration

	 int iApplyPotentialCorrection = 0;
	 if (usePotential) iApplyPotentialCorrection = 1;

         int max_int = 1;
         if (sensorCombination == _OVERLAP_TWO_SEC)
         {
            max_int = max(aBinsLEEA, aBinsHEEA);
         }

         if (dl) std::cout << "[peacemoments] about to do integration with max_int = " << max_int  << std::endl;

// loop - required in order to handle 2-sec time-resolution PAD moments

         for (int i_int=0; i_int<max_int; i_int++)
         {
            if (dl) std::cout << "[peacemoments] about to do integration for i_int = " << i_int << std::endl;

            int intReturn = peaceSpin.integrate(sensorCombination, iApplyPotentialCorrection, i_int, testingInt);
            
            if (dl) std::cout << "[peacemoments] integration done" << std::endl;

	    if (intReturn == _SUCCESS)
	    {
      
               // convert from standard moments to moments relative to the bulk flow     
      
               peaceSpin.convertToRelativeMoments();
      
               // convert to required coordinate system      
      
               peaceSpin.applyRotation(coordinateSystem);

               // generate currents

               peaceSpin.calculateCurrents(); 

               // calculate characteristic energy

               if (WML.ce)
               {
                  int ret = peaceSpin.calculateCharacteristicEnergy();
                  if (ret != _SUCCESS) return ret;
               }

               // convert to required units

               peaceSpin.adjustUnits();

	    }
	    else
	    {
	       peaceSpin.setAllMomentsToFills();
	       if (sensorCombination == _OVERLAP_TWO_SEC) fatalError = true;
	    }

            // remove data if necessary - output filtering

            if ((Status_QUA > 0 && filterQuality) ||
                (Status_STA > 0 && filterStats) ||
                (Status_EFW > 0 && filterEFW) ||
                (Status_WHI > 0 && filterWHI) ||
                (Status_DSE > 0 && filterDataStream))
               peaceSpin.setAllMomentsToFills();

            // remove data if density is zero & all other moments are fills

            if (peaceSpin.n == 0 && fabs(peaceSpin.v[0]) > MAX_VALUE_PM &&
                fabs(peaceSpin.v[1]) > MAX_VALUE_PM && fabs(peaceSpin.v[2]) > MAX_VALUE_PM &&
                fabs(peaceSpin.T) > MAX_VALUE_PM)
               peaceSpin.setAllMomentsToFills();
            
            // prepare some mode information for writing to output file

            int whichSensorType = -1;
            if (peaceFileSensors1 == _TWO_SENSOR) whichSensorType = 2;
            else
            {
   	       if (usingLEEA && usingHEEA) whichSensorType = 2;
	       else if (usingLEEA && !usingHEEA) whichSensorType = 1;
	       else if (usingHEEA && !usingLEEA) whichSensorType = 0;
            }

            // write out moments to output file

            // prepare the date/time - for PITCH_FULL & PITCH_3D moments, we
            // calculate the date/time from the azimuthal angle

            QString timeDateToWrite = timeDatePeace1;
            float lowerDeltaToWrite = time_tags_DeltaLower;
            float upperDeltaToWrite = time_tags_DeltaUpper;
	    //cout << "writing " << i_int << endl;

            if (sensorCombination == _OVERLAP_TWO_SEC)
            {
               generateDateTime(timeDatePeace1, time_tags_DeltaLower, time_tags_DeltaUpper, peaceSpin.spos,
                                aziBins[i_int], aziBinsLower[i_int], aziBinsUpper[i_int],
                                &timeDateToWrite, &lowerDeltaToWrite, &upperDeltaToWrite, peaceSpin.padTimingMaster);
	       //cout << "  " << timeDateToWrite.toStdString() << endl;
            }

            if (!fatalError) // sometimes things are so bad we don't even want to write a fill value record :-)
            {
               if (dl) std::cout << "[peacemoments] actually writing record for " << timeDateToWrite.toStdString() << std::endl;
	 
               outMoments.writeDataVariable(timeDateToWrite);
               outMoments.writeDataVariable(1, &lowerDeltaToWrite);
               outMoments.writeDataVariable(1, &upperDeltaToWrite);
	 
	 // Mode_ variables

	       outMoments.writeDataVariable(1, &whichSensorType);
	       outMoments.writeDataVariable(1, &peaceSpin.sweepModeHEEA);
	       outMoments.writeDataVariable(1, &peaceSpin.sweepModeLEEA);

	       outMoments.writeDataVariable(1, &peaceSpin.sensorUsedTopRegion);
	       outMoments.writeDataVariable(1, &peaceSpin.sensorUsedOverlapRegion);
	       outMoments.writeDataVariable(1, &peaceSpin.sensorUsedBottomRegion);

	 // Data_ variables 

               if (WML.n) outMoments.writeDataVariable(1, &peaceSpin.n);
	       if (WML.v) outMoments.writeDataVariable(3, peaceSpin.v);
	       if (WML.vpara) outMoments.writeDataVariable(3, peaceSpin.vpara);
	       if (WML.vperp) outMoments.writeDataVariable(3, peaceSpin.vperp);
	       if (WML.q) outMoments.writeDataVariable(3, peaceSpin.Q);
	       if (WML.qpara) outMoments.writeDataVariable(3, peaceSpin.Qpara);
	       if (WML.qperp) outMoments.writeDataVariable(3, peaceSpin.Qperp);
	       if (WML.h) outMoments.writeDataVariable(3, peaceSpin.H);
               if (WML.hpara) outMoments.writeDataVariable(3, peaceSpin.Hpara);
	       if (WML.hperp) outMoments.writeDataVariable(3, peaceSpin.Hperp);
	       if (WML.p) outMoments.writeDataVariable(9, &peaceSpin.P[0][0]);
	       if (WML.t) outMoments.writeDataVariable(1, &peaceSpin.T);
	       if (WML.tpara) outMoments.writeDataVariable(1, &peaceSpin.Tpara);
	       if (WML.tperp) outMoments.writeDataVariable(1, &peaceSpin.Tperp);
               if (WML.j) outMoments.writeDataVariable(3, peaceSpin.j);
               if (WML.jpara) outMoments.writeDataVariable(3, peaceSpin.jpara);
               if (WML.jperp) outMoments.writeDataVariable(3, peaceSpin.jperp);
               if (WML.ce) outMoments.writeDataVariable(1, &peaceSpin.ce);
               if (WML.b) outMoments.writeDataVariable(3, peaceSpin.fgmB); 
               if (WML.scp) outMoments.writeDataVariable(1, &peaceSpin.potentialSpinAv);

         // Status_ variables

               outMoments.writeDataVariable(1, &hStatus_QUA);
               outMoments.writeDataVariable(1, &hStatus_STA);
               outMoments.writeDataVariable(1, &hStatus_DSE);
            
               outMoments.writeDataVariable(1, &lStatus_QUA);
               outMoments.writeDataVariable(1, &lStatus_STA);
               outMoments.writeDataVariable(1, &lStatus_DSE);

               // prepare status variables (EFW, WHI, Eclipse) then write

               int myStatus_EFW = 0, myStatus_WHI = 0, myStatus_ECL = 0;

               if (lStatus_EFW > 0) myStatus_EFW = lStatus_EFW;
               if (hStatus_EFW > 0) myStatus_EFW = hStatus_EFW;
               if (lStatus_WHI > 0) myStatus_WHI = lStatus_WHI;
               if (hStatus_WHI > 0) myStatus_WHI = hStatus_WHI;
               if (lStatus_ECL > 0) myStatus_ECL = lStatus_ECL;
               if (hStatus_ECL > 0) myStatus_ECL = hStatus_ECL;

               if (lStatus_EFW < 0 && hStatus_EFW < 0) myStatus_EFW = -1;
               if (lStatus_WHI < 0 && hStatus_WHI < 0) myStatus_WHI = -1;
               if (lStatus_ECL < 0 && hStatus_ECL < 0) myStatus_ECL = -1;

               outMoments.writeDataVariable(1, &myStatus_EFW);
               outMoments.writeDataVariable(1, &myStatus_WHI);
               outMoments.writeDataVariable(1, &myStatus_ECL);

	 
	       // end current record
	 
	       outMoments.endCurrentDataRecord(); 

               if (dl) std::cout << "[peacemoments] completed writing record" << std::endl;

            } // <-- end check for fatalError
	 
	 // increment counter
	 
	    counter++;	 
         } // <-- end of do loop 
      }

//-----------------------------------------------------------------------------
//
// prepare for the next record
//
//-----------------------------------------------------------------------------

      if (dl) std::cout << "[peacemoments] preparing for next record" << std::endl;

      if (usePeace1)
      {
         int retNextRec = peaceRead1.readRecord();
	 if (retNextRec == _SUCCESS) retval = peaceRead1.getDataEpoch(iWhat1[_time_tags], &timeDatePeace1);
	 if (retNextRec != _SUCCESS || !retval) stillRunning = false;
      }
      if (usePeace2)
      {
         int retNextRec = peaceRead2.readRecord(); 
	 if (retNextRec == _SUCCESS) retval = peaceRead2.getDataEpoch(iWhat2[_time_tags], &timeDatePeace2);
	 if (retNextRec != _SUCCESS || !retval) stillRunning = false;
      }      

      spinCentreTimePrev = spinCentreTimeCurrent;

      if (dl) std::cout << "[peacemoments] about to do next record..." << std::endl;

   } 
   
//-----------------------------------------------------------------------------
//
// tidy up
//
//-----------------------------------------------------------------------------

   *numRecordsReturned = counter;

   if (!guiOnOff) cout << "Done, written " << counter << " records to the output file." << endl;

   outMoments.endData();
   outMoments.close();
   
   if (usePeace1) peaceRead1.close();
   if (usePeace2) peaceRead2.close();
   if (usePotential) potentialRead.close();
   if (useMagField) magRead.close();
   if (useAuxGSE) gseRead.close();
   
   if (counter == 0) return(_NO_MOMENTS_CALCULATED);
   return(_SUCCESS);

}

