/*******************************************************************************
 **
 ** $Id: plotmaker.C 151 2008-11-28 22:26:47Z AndrewLahiff $
 **
 ******************************************************************************
 **
 ** Copyright (C) 2007-2008 Andrew Lahiff.
 **
 ** This file is part of PEACEmoments CAA.
 **
 ** PEACEmoments CAA is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** PEACEmoments CAA is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with PEACEmoments CAA.  If not, see <http://www.gnu.org/licenses/>.
 **
 *******************************************************************************/

#include "peacemoments_caa.h"
#include "peacemoments_gui.h"
#include "plotmaker.h"

#define MAX_NUMBER_PLOTS 30
#define MAX_NUMBER_PLOTS_PER_PANEL 10

struct PlotPanel
{
   int numTimeSeries;
   int numSpectrograms;
   int tsVar[MAX_NUMBER_PLOTS_PER_PANEL];
   int spVar[MAX_NUMBER_PLOTS_PER_PANEL];
   double tsFill[MAX_NUMBER_PLOTS_PER_PANEL];
   double spFill[MAX_NUMBER_PLOTS_PER_PANEL];
   int tsVarComp[MAX_NUMBER_PLOTS_PER_PANEL];
   QString tsLegend[MAX_NUMBER_PLOTS_PER_PANEL];
   float multFactor[MAX_NUMBER_PLOTS_PER_PANEL];
};

//-----------------------------------------------------------------------------
//
// constructor
//
//-----------------------------------------------------------------------------

PlotMaker::PlotMaker()
{

}

//-----------------------------------------------------------------------------
//
// destructor
//
//-----------------------------------------------------------------------------

PlotMaker::~PlotMaker()
{

}

//-----------------------------------------------------------------------------
//
// make plot
//
//-----------------------------------------------------------------------------

int PlotMaker::makePlot(QString fileName, QString fileNamePotential)
{

   fileNamePotential = QString(""); // remove compiler warning

// initialization

   double userDefinedStartTime = -1, userDefinedStopTime = -1;

// CAA CEF/CDF reader

   ReadFile caaRead;

// open file

   if (caaRead.open(fileName) == _READ_FAILED) return(_ERROR_OPENING_FILE);

// read the header
      
   caaRead.readHeader();
   QString myAutoPlotTitle = caaRead.getGlobalAttr("DATASET_TITLE");

// setup the plot

    QsiPlot myPlot(mySettings->plotWidth, mySettings->plotHeight, myAutoPlotTitle, -1, false);
    myPlot.setPlotLineWidth(1);
    myPlot.setCurveLineWidth(1);
    myPlot.setBackgroundColor(Qt::white);

// setup structure for plot panels & initialize   
   
   struct PlotPanel plotPanels[MAX_NUMBER_PLOTS];
   for (int i=0; i<MAX_NUMBER_PLOTS; i++)
   {
      plotPanels[i].numTimeSeries = 0;
      plotPanels[i].numSpectrograms = 0;
   }
   int numPlotPanels = 0, numDataVars = 0, numDataVarsActual = 0;
   int numDataVarsSize[MAX_NUMBER_PLOTS], numDataVarsVar[MAX_NUMBER_PLOTS];
   double numDataFillValue[MAX_NUMBER_PLOTS] = {0};
   
   int varDefinedTpara = -1, varDefinedTperp = -1;
   int varDefinedVpara = -1;
   int varDefinedQpara = -1;
   int varDefinedHpara = -1;
   int varDefinedJpara = -1;
   
   double multFactor[200];

// loop over variables in the moments file, determine what to plot   

   for (int i = 0; i < caaRead.getNumVariables(); i++)
   {
      QString name = caaRead.getVariableAttr(i, "NAME");
      double currentFillValue = (caaRead.getVariableAttr(i, "FILLVAL")).toFloat();

      if (mySettings->n && name == "Data_Density")
      {	 
	 plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = -1;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 multFactor[i] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	
	 numDataVarsSize[numDataVars] = 1; 
	 numDataVarsVar[numDataVars] = i;
	 numDataVars++;	 
      }
      else if (mySettings->v && name == "Data_Velocity")
      {      
         multFactor[i] = 1.0;
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 0;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;	 
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 1;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	 
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 2;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	
	 numDataVarsSize[numDataVars] = 3; 
	 numDataVarsVar[numDataVars] = i;
	 numDataVars++;	 
      }
      else if (mySettings->vpara && name == "Data_Velocity_ComponentParallelToMagField")
      {
         multFactor[i] = 1.0;
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 0;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 varDefinedVpara = numPlotPanels;
	 numPlotPanels++;
	 numDataVarsActual++;
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 1;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	 
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 2;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	
	 numDataVarsSize[numDataVars] = 3; 
	 numDataVarsVar[numDataVars] = i;
	 numDataVars++;	 
      }
      else if (mySettings->vperp && name == "Data_Velocity_ComponentPerpendicularToMagField")
      {
         multFactor[i] = 1.0;
         int tsNum = 0;
	 if (varDefinedVpara > -1)
	 {
	    tsNum = 1;
	    numPlotPanels = varDefinedVpara;
	 }      
         plotPanels[numPlotPanels].tsVar[tsNum] = i;
	 plotPanels[numPlotPanels].tsVarComp[tsNum] = 0;
	 plotPanels[numPlotPanels].tsFill[tsNum] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[tsNum] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;
         plotPanels[numPlotPanels].tsVar[tsNum] = i;
	 plotPanels[numPlotPanels].tsVarComp[tsNum] = 1;
	 plotPanels[numPlotPanels].tsFill[tsNum] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[tsNum] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	 
         plotPanels[numPlotPanels].tsVar[tsNum] = i;
	 plotPanels[numPlotPanels].tsVarComp[tsNum] = 2;
	 plotPanels[numPlotPanels].tsFill[tsNum] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[tsNum] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	
	 numDataVarsSize[numDataVars] = 3; 
	 numDataVarsVar[numDataVars] = i;
	 numDataVars++;	 
      }      
      else if (mySettings->t && name == "Data_Temperature")
      {
         multFactor[i] = 1.0;
	 plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = -1;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	
	 numDataVarsSize[numDataVars] = 1; 
	 numDataVarsVar[numDataVars] = i;
	 numDataVars++;	
      }
      else if (mySettings->tpara && name == "Data_Temperature_ComponentParallelToMagField")
      {
         multFactor[i] = 1.0;
	 plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = -1;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 varDefinedTpara = numPlotPanels;
	 numPlotPanels++;
	 numDataVarsActual++;	
	 numDataVarsSize[numDataVars] = 1; 
	 numDataVarsVar[numDataVars] = i;
	 numDataVars++;	 
      }
      else if (mySettings->tperp && name == "Data_Temperature_ComponentPerpendicularToMagField")
      {  
         multFactor[i] = 1.0;
         int tsNum = 0;
	 if (varDefinedTpara > -1)
	 {
	    tsNum = 1;
	    numPlotPanels = varDefinedTpara;
	 }
	 plotPanels[numPlotPanels].tsVar[tsNum] = i;
	 plotPanels[numPlotPanels].tsVarComp[tsNum] = -1;
	 plotPanels[numPlotPanels].tsFill[tsNum] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[tsNum] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 varDefinedTperp = numPlotPanels;
	 numPlotPanels++;
	 numDataVarsActual++;	
	 numDataVarsSize[numDataVars] = 1; 
	 numDataVarsVar[numDataVars] = i;
	 numDataVars++;	
      }  
      else if (mySettings->q && name == "Data_EnergyFlux")
      {  
         multFactor[i] = 1.0;
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 0;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 1;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	 
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 2;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	
	 numDataVarsSize[numDataVars] = 3; 
	 numDataVarsVar[numDataVars] = i;
	 numDataVars++;	 
      }
      else if (mySettings->qpara && name == "Data_EnergyFlux_ComponentParallelToMagField")
      {
         multFactor[i] = 1.0;
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 0;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 varDefinedQpara = numPlotPanels;
	 numPlotPanels++;
	 numDataVarsActual++;
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 1;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	 
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 2;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	
	 numDataVarsSize[numDataVars] = 3; 
	 numDataVarsVar[numDataVars] = i;
	 numDataVars++;	  
      }      
      else if (mySettings->qperp && name == "Data_EnergyFlux_ComponentPerpendicularToMagField")
      {
         multFactor[i] = 1.0;
         int tsNum = 0;
	 if (varDefinedQpara > -1)
	 {
	    tsNum = 1;
	    numPlotPanels = varDefinedQpara;
	 }           
         plotPanels[numPlotPanels].tsVar[tsNum] = i;
	 plotPanels[numPlotPanels].tsVarComp[tsNum] = 0;
	 plotPanels[numPlotPanels].tsFill[tsNum] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[tsNum] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;
         plotPanels[numPlotPanels].tsVar[tsNum] = i;
	 plotPanels[numPlotPanels].tsVarComp[tsNum] = 1;
	 plotPanels[numPlotPanels].tsFill[tsNum] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[tsNum] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	 
         plotPanels[numPlotPanels].tsVar[tsNum] = i;
	 plotPanels[numPlotPanels].tsVarComp[tsNum] = 2;
	 plotPanels[numPlotPanels].tsFill[tsNum] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[tsNum] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	
	 numDataVarsSize[numDataVars] = 3; 
	 numDataVarsVar[numDataVars] = i;
	 numDataVars++;	  
      }       
      else if (mySettings->h && name == "Data_HeatFlux")
      {
         multFactor[i] = 1.0;
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 0;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 1;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	 
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 2;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	
	 numDataVarsSize[numDataVars] = 3; 
	 numDataVarsVar[numDataVars] = i;
	 numDataVars++;	  
      }
      else if (mySettings->hpara && name == "Data_HeatFlux_ComponentParallelToMagField")
      {
         multFactor[i] = 1.0;
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 0;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 varDefinedHpara = numPlotPanels;
	 numPlotPanels++;
	 numDataVarsActual++;
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 1;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	 
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 2;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	
	 numDataVarsSize[numDataVars] = 3; 
	 numDataVarsVar[numDataVars] = i;
	 numDataVars++;	  
      }      
      else if (mySettings->hperp && name == "Data_HeatFlux_ComponentPerpendicularToMagField")
      {
         multFactor[i] = 1.0;
         int tsNum = 0;
	 if (varDefinedHpara > -1)
	 {
	    tsNum = 1;
	    numPlotPanels = varDefinedHpara;
	 }           
         plotPanels[numPlotPanels].tsVar[tsNum] = i;
	 plotPanels[numPlotPanels].tsVarComp[tsNum] = 0;
	 plotPanels[numPlotPanels].tsFill[tsNum] = currentFillValue;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;
         plotPanels[numPlotPanels].tsVar[tsNum] = i;
	 plotPanels[numPlotPanels].tsVarComp[tsNum] = 1;
	 plotPanels[numPlotPanels].tsFill[tsNum] = currentFillValue;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	 
         plotPanels[numPlotPanels].tsVar[tsNum] = i;
	 plotPanels[numPlotPanels].tsVarComp[tsNum] = 2;
	 plotPanels[numPlotPanels].tsFill[tsNum] = currentFillValue;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	
	 numDataVarsSize[numDataVars] = 3; 
	 numDataVarsVar[numDataVars] = i;
	 numDataVars++;	  
      }                           
      else if (mySettings->b && name == "Data_MagField")
      {
         multFactor[i] = 1.0;
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = -1;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numDataVarsActual++;
         plotPanels[numPlotPanels].tsVar[1] = i;
	 plotPanels[numPlotPanels].tsVarComp[1] = -1;
	 plotPanels[numPlotPanels].tsFill[1] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numDataVarsActual++;	 
         plotPanels[numPlotPanels].tsVar[2] = i;
	 plotPanels[numPlotPanels].tsVarComp[2] = -1;
	 plotPanels[numPlotPanels].tsFill[2] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	
	 numDataVarsSize[numDataVars] = 3; 
	 numDataVarsVar[numDataVars] = i;
	 numDataVars++;	
      }
      else if (mySettings->scp && name == "Data_Potential")
      {
         multFactor[i] = 1.0;
	 plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = -1;	 
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	
	 numDataVarsSize[numDataVars] = 1; 
	 numDataVarsVar[numDataVars] = i;
	 numDataVars++;	
      }
      else if (mySettings->j && name == "Data_Current")
      {      
         multFactor[i] = 1.0e+06;
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 0;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1000000.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 1;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1000000.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	 
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 2;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1000000.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	
	 numDataVarsSize[numDataVars] = 3; 
	 numDataVarsVar[numDataVars] = i;
	 numDataVars++;	 
      }      
      else if (mySettings->jpara && name == "Data_Current_ComponentParallelToMagField")
      {
         multFactor[i] = 1.0e+06;
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 0;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1000000.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 varDefinedJpara = numPlotPanels;
	 numPlotPanels++;
	 numDataVarsActual++;
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 1;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1000000.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	 
         plotPanels[numPlotPanels].tsVar[0] = i;
	 plotPanels[numPlotPanels].tsVarComp[0] = 2;
	 plotPanels[numPlotPanels].tsFill[0] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[0] = 1000000.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	
	 numDataVarsSize[numDataVars] = 3; 
	 numDataVarsVar[numDataVars] = i;
	 numDataVars++;	 
      }
      else if (mySettings->jperp && name == "Data_Current_ComponentPerpendicularToMagField")
      {
         multFactor[i] = 1.0e+06;
         int tsNum = 0;
	 if (varDefinedJpara > -1)
	 {
	    tsNum = 1;
	    numPlotPanels = varDefinedJpara;
	 }      
         plotPanels[numPlotPanels].tsVar[tsNum] = i;
	 plotPanels[numPlotPanels].tsVarComp[tsNum] = 0;
	 plotPanels[numPlotPanels].tsFill[tsNum] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[tsNum] = 1000000.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;
         plotPanels[numPlotPanels].tsVar[tsNum] = i;
	 plotPanels[numPlotPanels].tsVarComp[tsNum] = 1;
	 plotPanels[numPlotPanels].tsFill[tsNum] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[tsNum] = 1000000.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	 
         plotPanels[numPlotPanels].tsVar[tsNum] = i;
	 plotPanels[numPlotPanels].tsVarComp[tsNum] = 2;
	 plotPanels[numPlotPanels].tsFill[tsNum] = currentFillValue;
	 plotPanels[numPlotPanels].multFactor[tsNum] = 1000000.0;
	 numDataFillValue[numDataVarsActual] = currentFillValue;	 
	 plotPanels[numPlotPanels].numTimeSeries++;
	 numPlotPanels++;
	 numDataVarsActual++;	
	 numDataVarsSize[numDataVars] = 3; 
	 numDataVarsVar[numDataVars] = i;
	 numDataVars++;	 
      }           
   }

// set plot layout

   myPlot.setLayout(numPlotPanels, 1, myAutoPlotTitle);

   for (int i=0; i<numPlotPanels; i++)
   {
      if (i < numPlotPanels-1)  myPlot.showValuesX(i, 0, pFalse);
      else myPlot.showValuesX(i, 0, pTrue);

      myPlot.setPlotType(i, 0, TimeSeries);
      myPlot.setScaleTypeY(i, 0, Linear);
      myPlot.setScaleTypeX(i, 0, Linear);
   }

// allocate storage for the time series plots

   double *x = new double[86400];
   double vMinStore[MAX_NUMBER_PLOTS], vMaxStore[MAX_NUMBER_PLOTS];

   double **aa;
   aa = new double*[numDataVarsActual];
   for (register int i=0; i<numDataVarsActual; i++) aa[i] = new double [86400];

// initialize lower/upper y-axis limits for each plot panel

   for (int i=0; i<numDataVarsActual; i++)
   {
      vMinStore[i] = 3.0e+30;
      vMaxStore[i] = -3.0e+30;
   }

// read in the data

   QDate initialDate;
   bool doneOnce = false;

   int arraySize = 0;
   while (caaRead.readRecord() != _READ_FAILED)
   {
      QString timeData;
      float dataF[12], dataIn;
      caaRead.getDataEpoch(0, &timeData);
      x[arraySize] = time_to_tis((char *)(timeData.mid(11,12).toStdString()).c_str());

      if (!doneOnce)
      {
         doneOnce = true;
         initialDate = toQDate(timeData.mid(0, 10));
      }

      int jj = 0;
      for (int i=0; i<numDataVars; i++)
      {
         caaRead.getDataValue(numDataVarsVar[i], dataF);

         for (int j=0; j<numDataVarsSize[i]; j++)
         {
            dataIn = multFactor[numDataVarsVar[i]]*dataF[j];
            aa[jj][arraySize] = (dataIn == numDataFillValue[jj] || fabs(dataIn) > MaxAllowedValue) ? FillValue : dataIn;
            if (aa[jj][arraySize] < vMinStore[jj] && fabs(aa[jj][arraySize]) < MaxAllowedValue) vMinStore[jj] = aa[jj][arraySize];
            if (aa[jj][arraySize] > vMaxStore[jj] && fabs(aa[jj][arraySize]) < MaxAllowedValue) vMaxStore[jj] = aa[jj][arraySize];
            jj++;
         }
      }
      arraySize++;
   }      

   myPlot.initialDate = initialDate;

// auto determine x-axis limits

   int nMinX;
   double tMin, tMax, durMajX;
   myPlot.getAutoScaleX(x, arraySize, userDefinedStartTime, userDefinedStopTime, &durMajX, &nMinX, &tMin, &tMax);

   if (mySettings->includeSpectrogram)
   {
      vMinStore[0] = mySettings->spectroEmin;
      vMaxStore[0] = mySettings->spectroEmax;
   }

// auto determine y-axis limits

   int ic = 0;
   for (int i=0; i<numPlotPanels; i++)
   {
      int nMaj, nMin;
      double vMin, vMax, vvMin = 0, vvMax = 0;
      for (int j=0; j<plotPanels[i].numTimeSeries; j++)
      {
         if (j == 0)
	 {
	   vvMin = vMinStore[ic];
	   vvMax = vMaxStore[ic];	   
	 }
	 else
	 {
	    if (vMinStore[ic] < vvMin) vvMin = vMinStore[ic];
	    if (vMaxStore[ic] > vvMax) vvMax = vMaxStore[ic];
	 }
	 ic++;
      }

      if (fabs(vvMin) < 1.0e-05 && fabs(vvMax) < 1.0e-05)
      {
         vvMin = -5.0;
         vvMax = 5.0;
      }

      myPlot.getAutoScaleY(vvMin, vvMax, &nMaj, &nMin, &vMin, &vMax, 0);
      myPlot.setPlotAxisLimits(i, 0, tMin, tMax, vMin, vMax);
      myPlot.drawFrame(i, 0);
      myPlot.setNumTicksY(i, 0, nMaj, nMin);
      myPlot.setNumTicksX(i, 0, durMajX, nMinX);
   }   

// determine & set the y-axis labels

    if (mySettings->includeSpectrogram) myPlot.setLabelY(0, 0, "energy (eV)");

    int ia = 0;
    for (int i=0; i<numPlotPanels; i++)
    {           
       QString label1 = caaRead.getVariableAttr(plotPanels[i].tsVar[0], "REPRESENTATION_1");
       QStringList label1List = label1.split(",", QString::SkipEmptyParts);

       QString unitsLabel = caaRead.getVariableAttr(plotPanels[i].tsVar[0], "UNITS");
       
       if (unitsLabel.contains("A/m^2"))
       {
          unitsLabel = "A/micro-m^2"; 
       }
       
       plotPanels[i].tsLegend[0] = "";
       QString axisName = caaRead.getVariableAttr(plotPanels[i].tsVar[0], "LABLAXIS");
       
       if (plotPanels[i].numTimeSeries == 2 && axisName.contains("Te (para)"))
       {
          axisName = "Te"; // correct y-axis label when showing Tpara & Tperp
       }
       
       if (plotPanels[i].tsVarComp[0] > -1)
       {
          if (plotPanels[i].numTimeSeries == 1)
	  {
             axisName = axisName + "(" + label1List.at(plotPanels[i].tsVarComp[0]) + ")";	
	  }    
       }
       
       if (plotPanels[i].numTimeSeries == 2 && axisName.contains("Ve (para)"))
       {
          axisName = "Ve"; // correct y-axis label when showing Vpara & Vperp
	  axisName = axisName + "(" + label1List.at(plotPanels[i].tsVarComp[0]) + ")";
       }     
       
       if (plotPanels[i].numTimeSeries == 2 && axisName.contains("He (para)"))
       {
          axisName = "Ve"; // correct y-axis label when showing Hpara & Hperp
	  axisName = axisName + "(" + label1List.at(plotPanels[i].tsVarComp[0]) + ")";
       }
       
       if (plotPanels[i].numTimeSeries == 2 && axisName.contains("Qe (para)"))
       {
          axisName = "Qe"; // correct y-axis label when showing Qpara & Qperp
	  axisName = axisName + "(" + label1List.at(plotPanels[i].tsVarComp[0]) + ")";
       }                
       
       if (plotPanels[i].numTimeSeries == 2 && axisName.contains("Je (para)"))
       {
          axisName = "Je"; // correct y-axis label when showing Jpara & Jperp
	  axisName = axisName + "(" + label1List.at(plotPanels[i].tsVarComp[0]) + ")";
       }
       	  
       axisName = axisName + " (" + unitsLabel + ")";
       myPlot.setLabelY(i, 0, axisName);	              
      
       for (int j=0; j<plotPanels[i].numTimeSeries; j++)
       {  	  
	  if (plotPanels[i].numTimeSeries > 1)
	  {
	     if (axisName.contains("Ve") || axisName.contains("He") || axisName.contains("Qe") ||
	         axisName.contains("Je"))
	     {
	        plotPanels[i].tsLegend[j] = caaRead.getVariableAttr(plotPanels[i].tsVar[j], "LABLAXIS");        
	     }
	     else if (label1List.size() > 1)
	     {
	        plotPanels[i].tsLegend[j] = caaRead.getVariableAttr(plotPanels[i].tsVar[j], "LABLAXIS") +
	                                    "(" + label1List.at(j) + ")";
             }
	     else
	     {
	        plotPanels[i].tsLegend[j] = caaRead.getVariableAttr(plotPanels[i].tsVar[j], "LABLAXIS");
	     }		  
	  }	  
          ia++;
       }
    }   

// draw the colour box

   if (mySettings->includeSpectrogram)
   {
      myPlot.drawColourBox(0, 0, mySettings->spectroCmin, mySettings->spectroCmax, "Diff. energy flux");
   }

// set colour list for plots

   QColor colorList[10];
   colorList[0].setNamedColor("black");
   colorList[1].setNamedColor("blue");
   colorList[2].setNamedColor("red");
   colorList[3].setNamedColor("lime");
   colorList[4].setNamedColor("magenta");
   colorList[5].setNamedColor("cyan");
   colorList[6].setNamedColor("navy");
   colorList[7].setNamedColor("darkorange");
   colorList[8].setNamedColor("blueviolet");
   
   myPlot.setLegendLocation(TopLeft);

// make plot

   int ii=0;
   for (int i=0; i<numPlotPanels; i++)
   {  
      for (int j=0; j<plotPanels[i].numTimeSeries; j++)
      {      
         myPlot.drawCurve(i, 0, colorList[j], arraySize, x, &aa[ii][0], plotPanels[i].tsLegend[j], PLOT_LINE);
	 if (plotPanels[i].numTimeSeries > 1 && plotPanels[i].tsVarComp[0] < 1) myPlot.showLegend(i, 0, j, PLOT_LINE);
	 ii++;
      }
   }

// make the QImage available

   qPlot = myPlot.plot->copy(0, 0, myPlot.plot->width(), myPlot.plot->height());

// save plot to file if necessary

   if (mySettings->saveToFile) myPlot.saveToFile(mySettings->plotFileName);

// deallocate memory

   for (register int i=numDataVarsActual-1; i>=0; i--) delete [] aa[i]; delete [] aa;
   delete [] x;

// close file

   caaRead.close();

// return

   return(_SUCCESS);

}

//-----------------------------------------------------------------------------
// 
// 
// 
//-----------------------------------------------------------------------------
   
bool PlotMaker::checkIfHave(int n, int *a, int type)
{
   bool have = false;
   for (int i=0; i<n; i++)
   {
      if (*(a+i) == type) have = true;
   }
   return(have);
}

