/*******************************************************************************
 **
 ** $Id: plotwindow.C 149 2008-11-28 20:51:29Z AndrewLahiff $
 **
 ******************************************************************************
 **
 ** Copyright (C) 2007-2008 Andrew Lahiff.
 **
 ** This file is part of PEACEmoments CAA.
 **
 ** PEACEmoments CAA is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** PEACEmoments CAA is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with PEACEmoments CAA.  If not, see <http://www.gnu.org/licenses/>.
 **
 *******************************************************************************/

#include "peacemoments_caa.h"
#include "peacemoments_gui.h"
#include "window.h"

#define SPECTRO_AXIS_WIDTH 70
#define PLOT_BUTTONS_WIDTH 65

//-----------------------------------------------------------------------------
//
// constructor
//
//-----------------------------------------------------------------------------

PlotWindow::PlotWindow(QWidget *parent)
   : QWidget(parent)
{

//   setPalette(QPalette(QColor("#c0c0c0")));
//   setAutoFillBackground(true);
   
// moments   
   
   plotSettingsMoments = new QGroupBox("Moments plot");
   plotSettingsMoments->setCheckable(true);
   plotSettingsMoments->setToolTip("Generate moments plot automatically (only data in the moments output file can be plotted)");
 
   iPlotS = new QCheckBox("Spectrogram");
   iPlotN = new QCheckBox("Density");
   iPlotV = new QCheckBox("Velocity");
   iPlotQ = new QCheckBox("Energy flux");
   iPlotH = new QCheckBox("Heat flux");
   iPlotVpara = new QCheckBox("Velocity (para)");
   iPlotVperp = new QCheckBox("Velocity (perp)");
   iPlotQpara = new QCheckBox("Energy flux (para)");
   iPlotQperp = new QCheckBox("Energy flux (perp)");
   iPlotHpara = new QCheckBox("Heat flux (para)");
   iPlotHperp = new QCheckBox("Heat flux (perp)");
   iPlotP = new QCheckBox("Pressure");
   iPlotT = new QCheckBox("Temperature");
   iPlotTpara = new QCheckBox("Temperature (para)");
   iPlotTperp = new QCheckBox("Temperature (perp)");
   
   iPlotJ = new QCheckBox("Current");
   iPlotJpara = new QCheckBox("Current (para)");
   iPlotJperp = new QCheckBox("Current (perp)");
   
   iPlotB = new QCheckBox("Magnetic field");
   iPlotScP = new QCheckBox("Potential");
   
   iPlotN->setChecked(true);
   iPlotV->setChecked(true);
   iPlotT->setChecked(true);
   iPlotS->setEnabled(false);
     
   QGridLayout *mainLayout = new QGridLayout;
   mainLayout->addWidget(iPlotS, 0, 0);
   mainLayout->addWidget(iPlotN, 0, 1);
   mainLayout->addWidget(iPlotV, 0, 2);
   mainLayout->addWidget(iPlotQ, 0, 3);
   mainLayout->addWidget(iPlotH, 1, 0);
   mainLayout->addWidget(iPlotVpara, 1, 1);
   mainLayout->addWidget(iPlotVperp, 1, 2);
   mainLayout->addWidget(iPlotQpara, 1, 3);
   mainLayout->addWidget(iPlotQperp, 2, 0);
   mainLayout->addWidget(iPlotHpara, 2, 1);
   mainLayout->addWidget(iPlotHperp, 2, 2);
   mainLayout->addWidget(iPlotP, 2, 3);
   mainLayout->addWidget(iPlotT, 3, 0);
   mainLayout->addWidget(iPlotTpara, 3, 1);
   mainLayout->addWidget(iPlotTperp, 3, 2);
   mainLayout->addWidget(iPlotJ, 3, 3);
   mainLayout->addWidget(iPlotJpara, 4, 0);
   mainLayout->addWidget(iPlotJperp, 4, 1);
   mainLayout->addWidget(iPlotB, 4, 2);
   mainLayout->addWidget(iPlotScP, 4, 3);
 
   saveAsDefaultsButton = new QPushButton("&Save as defaults");
   saveAsDefaultsButton->setToolTip("Save the current settings as the default when making moments plots");

   connect(saveAsDefaultsButton, SIGNAL(clicked()),
           this, SLOT(saveAsDefaultsSlot()));
   
   QVBoxLayout *mplotPV = new QVBoxLayout;
   QHBoxLayout *mplotPH = new QHBoxLayout;
   mplotPH->addWidget(saveAsDefaultsButton);
   mplotPH->addStretch(1);

   saveMomentsToFile = new QPushButton("Save as...");
   saveMomentsToFile->setFixedWidth(PLOT_BUTTONS_WIDTH);
   saveMomentsToFile->setToolTip("Save current moments plot as a png");
   mplotPH->addWidget(saveMomentsToFile);

   connect(saveMomentsToFile, SIGNAL(clicked()),
           this, SLOT(slotSaveMomentsToFile()));
   
   replotMoments = new QPushButton("&Replot");
   replotMoments->setFixedWidth(PLOT_BUTTONS_WIDTH);
   replotMoments->setToolTip("Regenerate moments plot");
   mplotPH->addWidget(replotMoments);

   connect(replotMoments, SIGNAL(clicked()),
           this, SLOT(slotReplotMoments()));
   
   mplotPV->addLayout(mainLayout);
   mplotPV->addLayout(mplotPH);
   
   plotSettingsMoments->setLayout(mplotPV);
   
// spectrogram

   plotSettingsSpectro = new QGroupBox("Spectrogram");
   
   spectroYmin = new QLineEdit();
   spectroYmin->setAlignment(Qt::AlignRight);
   spectroYmin->setFixedWidth(SPECTRO_AXIS_WIDTH);
   spectroYmin->setText("1");
   
   spectroYmax = new QLineEdit();
   spectroYmax->setAlignment(Qt::AlignRight);
   spectroYmax->setFixedWidth(SPECTRO_AXIS_WIDTH);
   spectroYmax->setText("26000");
   
   spectroCmin = new QLineEdit();
   spectroCmin->setAlignment(Qt::AlignRight);
   spectroCmin->setFixedWidth(SPECTRO_AXIS_WIDTH);
   spectroCmin->setText("1e-06");
   
   spectroCmax = new QLineEdit();
   spectroCmax->setAlignment(Qt::AlignRight);
   spectroCmax->setFixedWidth(SPECTRO_AXIS_WIDTH);
   spectroCmax->setText("1e-03");
   
   QGroupBox *spectroYaxis = new QGroupBox("Y-axis limits");
   QGroupBox *spectroCaxis = new QGroupBox("Colour scale");
   QGroupBox *spectroUnits = new QGroupBox("Units");
   QGroupBox *spectroAnodes = new QGroupBox("Anodes");
   QGroupBox *spectroEnergy = new QGroupBox("Energy");
   QGroupBox *spectroSensors = new QGroupBox("Sensors");

   spectroSensors->setToolTip("Specify what sensors to show in the spectrogram (depends on files specified by user)");

   spectroUnitsCombo = new QComboBox;
   spectroUnitsCombo->addItem("Differential energy flux");
   spectroUnitsCombo->addItem("Differential number flux");
   spectroUnitsCombo->addItem("Phase space density");

   spectroAnodesCombo = new QComboBox;
   spectroAnodesCombo->addItem("All anodes averaged");

   spectroEnergyCombo = new QComboBox;
   spectroEnergyCombo->addItem("Show all energy bins");
   spectroEnergyCombo->addItem("Show energy bins above potential only");
   spectroEnergyCombo->addItem("Potential corrected spectrogram");

   spectroSensorCombo = new QComboBox;
   spectroSensorCombo->addItem("Show all sensors");
   spectroSensorCombo->addItem("Show LEEA sensor only");
   spectroSensorCombo->addItem("Show HEEA sensor only");
   
   QLabel *labelYmin = new QLabel("Min");
   QLabel *labelYmax = new QLabel("Max");
   QLabel *labelCmin = new QLabel("Min");
   QLabel *labelCmax = new QLabel("Max");
   
   QHBoxLayout *layoutYaxis = new QHBoxLayout;
   QHBoxLayout *layoutCaxis = new QHBoxLayout;
   QHBoxLayout *layoutUnits = new QHBoxLayout;
   QHBoxLayout *layoutAnodes = new QHBoxLayout;
   QHBoxLayout *layoutEnergy = new QHBoxLayout;
   QHBoxLayout *layoutSensors = new QHBoxLayout;

   layoutUnits->addWidget(spectroUnitsCombo);
   layoutAnodes->addWidget(spectroAnodesCombo);
   layoutEnergy->addWidget(spectroEnergyCombo);
   layoutSensors->addWidget(spectroSensorCombo);
   
   layoutYaxis->addWidget(labelYmin);
   layoutYaxis->addWidget(spectroYmin);
   layoutYaxis->addStretch(1);
   layoutYaxis->addWidget(labelYmax);
   layoutYaxis->addWidget(spectroYmax);
  
   layoutCaxis->addWidget(labelCmin);
   layoutCaxis->addWidget(spectroCmin);
   layoutCaxis->addStretch(1);
   layoutCaxis->addWidget(labelCmax);
   layoutCaxis->addWidget(spectroCmax);
   
   spectroYaxis->setLayout(layoutYaxis);
   spectroCaxis->setLayout(layoutCaxis);
   spectroUnits->setLayout(layoutUnits);
   spectroAnodes->setLayout(layoutAnodes);
   spectroEnergy->setLayout(layoutEnergy);
   spectroSensors->setLayout(layoutSensors); 
   
   saveSpectroToFile = new QPushButton("Save as...");
   saveSpectroToFile->setFixedWidth(PLOT_BUTTONS_WIDTH);
   saveSpectroToFile->setToolTip("Save current spectrogram as a png");

   connect(saveSpectroToFile, SIGNAL(clicked()),
           this, SLOT(slotSaveSpectroToFile()));
   
   replotSpectro = new QPushButton("&Plot");
   replotSpectro->setFixedWidth(PLOT_BUTTONS_WIDTH);
   replotSpectro->setToolTip("Generate a spectrogram");

   connect(replotSpectro, SIGNAL(clicked()),
           this, SLOT(slotPlotSpectro()));
   
   QHBoxLayout *spectroButtonsH = new QHBoxLayout;
   spectroButtonsH->addStretch(1);
   spectroButtonsH->addWidget(saveSpectroToFile);
   spectroButtonsH->addWidget(replotSpectro);

   QGridLayout *spectroMainLayoutG = new QGridLayout;
   spectroMainLayoutG->addWidget(spectroYaxis, 0, 0);
   spectroMainLayoutG->addWidget(spectroCaxis, 0, 1);
   spectroMainLayoutG->addWidget(spectroUnits, 1, 0);
   spectroMainLayoutG->addWidget(spectroAnodes, 1, 1);
   spectroMainLayoutG->addWidget(spectroEnergy, 2, 0);
   spectroMainLayoutG->addWidget(spectroSensors, 2, 1);
   
   QVBoxLayout *spectroMainLayoutV = new QVBoxLayout;
   spectroMainLayoutV->addLayout(spectroMainLayoutG);
   spectroMainLayoutV->addLayout(spectroButtonsH);
   
   plotSettingsSpectro->setLayout(spectroMainLayoutV);
   
// the window

   QVBoxLayout *plotPV = new QVBoxLayout;
   QHBoxLayout *plotPH = new QHBoxLayout;
   
   closePlotWindowButton = new QPushButton("&Close");
   closePlotWindowButton->setToolTip("Close this window");
   
   connect(closePlotWindowButton, SIGNAL(clicked()),
           this, SLOT(closePlotWindow()));
   
   plotPH->addStretch(1);
   plotPH->addWidget(closePlotWindowButton);   
   
   plotPV->addWidget(plotSettingsSpectro);
   plotPV->addWidget(plotSettingsMoments);
   plotPV->addLayout(plotPH); 
   
   setLayout(plotPV);  

   setWindowTitle("Plot settings");
}

//------------------------------------------------------------------------------
//
// close plot window
//
//------------------------------------------------------------------------------
   
void PlotWindow::closePlotWindow()
{     
   hide();
}  

//-----------------------------------------------------------------------------
//
// save as defaults
//
//-----------------------------------------------------------------------------

void PlotWindow::saveAsDefaultsSlot()
{
   QSettings settings(QSettings::NativeFormat, QSettings::UserScope, "mssl-adl", "peacemoments_caa");
   settings.setValue("plot-mom/on-off", plotSettingsMoments->isChecked());
   settings.setValue("plot-mom/spectro", iPlotS->isChecked());
   settings.setValue("plot-mom/n", iPlotN->isChecked());
   settings.setValue("plot-mom/v", iPlotV->isChecked());
   settings.setValue("plot-mom/q", iPlotQ->isChecked());
   settings.setValue("plot-mom/h", iPlotH->isChecked());
   settings.setValue("plot-mom/vpara", iPlotVpara->isChecked());
   settings.setValue("plot-mom/vperp", iPlotVperp->isChecked());
   settings.setValue("plot-mom/qpara", iPlotQpara->isChecked());
   settings.setValue("plot-mom/qperp", iPlotQperp->isChecked());
   settings.setValue("plot-mom/hpara", iPlotHpara->isChecked());
   settings.setValue("plot-mom/hperp", iPlotHperp->isChecked());
   settings.setValue("plot-mom/t", iPlotT->isChecked());
   settings.setValue("plot-mom/tpara", iPlotTpara->isChecked());
   settings.setValue("plot-mom/tperp", iPlotTperp->isChecked());
   settings.setValue("plot-mom/j", iPlotJ->isChecked());
   settings.setValue("plot-mom/jpara", iPlotJpara->isChecked());
   settings.setValue("plot-mom/jperp", iPlotJperp->isChecked());
   settings.setValue("plot-mom/b", iPlotB->isChecked());
   settings.setValue("plot-mom/scp", iPlotScP->isChecked());
}

//-----------------------------------------------------------------------------
//
// replot moments
//
//-----------------------------------------------------------------------------

void PlotWindow::slotReplotMoments()
{
   emit remakeMomentsPlot("");
}

//-----------------------------------------------------------------------------
//
// plot spectrogram
//
//-----------------------------------------------------------------------------

void PlotWindow::slotPlotSpectro()
{
   replotSpectro->setEnabled(false);
   emit makeSpectrogramPlot("");
}

//-----------------------------------------------------------------------------
//
// save moments plot as png
//
//-----------------------------------------------------------------------------

void PlotWindow::slotSaveMomentsToFile()
{
   QString tmp = QFileDialog::getSaveFileName(this, "", "", "png files (*.png)");
   emit remakeMomentsPlot(tmp);
}

//-----------------------------------------------------------------------------
//
// save spectrogram as png
//
//-----------------------------------------------------------------------------

void PlotWindow::slotSaveSpectroToFile()
{
   QString tmp = QFileDialog::getSaveFileName(this, "", "", "png files (*.png)");
   emit makeSpectrogramPlot(tmp);
}

