/*******************************************************************************
 **
 ** $Id: sc2gse.C 149 2008-11-28 20:51:29Z AndrewLahiff $
 **
 ** Uses code based on transformations & rotations by Steve Schwartz.
 **
 ******************************************************************************
 **
 ** Copyright (C) 2007-2008 Andrew Lahiff.
 **
 ** This file is part of PEACEmoments CAA.
 **
 ** PEACEmoments CAA is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** PEACEmoments CAA is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with PEACEmoments CAA.  If not, see <http://www.gnu.org/licenses/>.
 **
 *******************************************************************************/

#include <math.h>
#define  TRDEG2RAD 0.0174532925199432958

void sc2gse(double lambda, double phi, double TR[3][3], double spos) 
{
   double sunaz;
   double lambda_rad, phi_rad, sunaz_rad;
   double Rot[3][3], T[3][3];

   double coslam, sinlam, cosphi, sinphi, beta;

   int i,j,k;
   
   sunaz = 26.2 - spos*360.0/(16.0*1024.0); // Cluster only

   lambda_rad = TRDEG2RAD * lambda;
   phi_rad    = TRDEG2RAD * phi;
   sunaz_rad  = TRDEG2RAD * sunaz;   
   
   Rot[0][0] = 1.0; Rot[0][1] = 0.0;              Rot[0][2] = 0.0;
   Rot[1][0] = 0.0; Rot[1][1] = cos (sunaz_rad);  Rot[1][2] = sin (sunaz_rad);
   Rot[2][0] = 0.0; Rot[2][1] = -sin (sunaz_rad); Rot[2][2] = cos (sunaz_rad);   

   coslam = cos(lambda_rad);
   sinlam = sin(lambda_rad);
   cosphi = cos(phi_rad);
   sinphi = sin(phi_rad);
   
   beta = 1.0/(sqrt(sinlam*sinlam + coslam*coslam*sinphi*sinphi));

   T[0][0] = coslam*cosphi;
   T[0][1] = beta*(sinlam*sinlam+coslam*coslam*sinphi*sinphi);
   T[0][2] = 0.0;

   T[1][0] = coslam*sinphi;
   T[1][1] = -beta*coslam*coslam*cosphi*sinphi;
   T[1][2] = beta*sinlam;

   T[2][0] = sinlam;
   T[2][1] = -beta*coslam*sinlam*cosphi;
   T[2][2] = -beta*coslam*sinphi;
        

// multiply T * Rot to form complete rotation matrix 

   for (i = 0; i<3; i++)
   {
      for (j=0; j<3; j++)
      {
         TR[i][j] = 0.0;
         for (k=0; k<3; k++)
	     {
            TR[i][j] += T[i][k]*Rot[k][j];
         }
      }
   }
}

