/*******************************************************************************
 **
 ** $Id: spectromaker.C 151 2008-11-28 22:26:47Z AndrewLahiff $
 **
 ******************************************************************************
 **
 ** Copyright (C) 2007-2008 Andrew Lahiff.
 **
 ** This file is part of PEACEmoments CAA.
 **
 ** PEACEmoments CAA is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** PEACEmoments CAA is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with PEACEmoments CAA.  If not, see <http://www.gnu.org/licenses/>.
 **
 *******************************************************************************/

#include "peacemoments_caa.h"
#include "peacemoments_gui.h"
#include "spectromaker.h"

#define ALMOST_ENOUGH 6.0

//-----------------------------------------------------------------------------
//
// constructor
//
//-----------------------------------------------------------------------------

SpectroMaker::SpectroMaker()
{
	
}

//-----------------------------------------------------------------------------
// 
// destructor
//
//-----------------------------------------------------------------------------

SpectroMaker::~SpectroMaker()
{
	
}

//-----------------------------------------------------------------------------
// 
// make spectrogram plot
//
//-----------------------------------------------------------------------------

int SpectroMaker::makePlot()
{

// how many files

   int iUseFile[3] = {0};
   if (!filePeace1.isEmpty()) iUseFile[0] = 1;
   if (!filePeace2.isEmpty()) iUseFile[1] = 1;
   if (!filePotential.isEmpty()) iUseFile[2] = 1;

// CAA CEF/CDF readers

   ReadFile readPeace[2], readPotential;
   
// open the files

   if (iUseFile[0] == 1)
   {
      if (readPeace[0].open(filePeace1) == _READ_FAILED) return(_ERROR_OPENING_FILE);
   }      

   if (iUseFile[1] == 1)
   {
      if (readPeace[1].open(filePeace2) == _READ_FAILED) return(_ERROR_OPENING_FILE);
   }
   
   if (iUseFile[2] == 1)
   {
      if (readPotential.open(filePotential) == _READ_FAILED) return(_ERROR_OPENING_FILE);	
   }

// read the headers

   for (int i=0; i<2; i++)
   {
      if (iUseFile[i] == 1) readPeace[i].readHeader();
   }
   if (iUseFile[2] == 1) readPotential.readHeader();

// check if the user has specified a valid potential file

   QString datasetIdPot;
   if (iUseFile[2] == 1)
   {
      datasetIdPot = readPotential.getGlobalAttr("DATASET_ID");
      int ic = checkForFileType(datasetIdPot);
      if (ic != _POTENTIAL_SPIN && ic != _POTENTIAL_HIGH) return(_NOT_VALID_POTENTIAL_FILE);
   }

// check if the user has specified valid PEACE files; also check if each
// file contains sensors appropriate to the user's sensor choice

   QString datasetId[2], datasetIdNU[2];

   int twoSensorsInSingleFile = 0;
   
   for (int i=0; i<2; i++)
   {
      if (iUseFile[i] == 1)
      {
         datasetId[i] = readPeace[i].getGlobalAttr("DATASET_ID");
         QStringList SL = datasetId[i].split("_");
         if (datasetId[i].contains("PITCH"))
         {
            datasetIdNU[i] = SL[0] + "_" + SL[1] + "_" + SL[2] + "_" + SL[3] + "_" + SL[4] +
               " (energy overlap region only shown)";
         }
         else datasetIdNU[i] = SL[0] + "_" + SL[1] + "_" + SL[2] + "_" + SL[3];
         int ic = checkForFileType(datasetId[i]);
         if (ic != _LEEA && ic != _HEEA && ic != _LEEA_HEEA) return(_NOT_VALID_PEACE_FILE);
         if (ic == _LEEA && spectroSensors == 2) iUseFile[i] = 0; // don't plot LEEA if user requested HEEA only
         if (ic == _HEEA && spectroSensors == 1) iUseFile[i] = 0; // don't plot HEEA if user requested LEEA only
         if (ic == _LEEA_HEEA) twoSensorsInSingleFile = 1;
      }
   }   

// determine what variable to plot from each PEACE file

   int iWhat[50] = {0};
   for (int i=0; i<50; i++) iWhat[i] = -1;
   bool done = false;
   for (int i=0; i<2; i++)
   {
      if (iUseFile[i] == 1 && !done)
      {
      	 getPeaceLocations(&readPeace[i], iWhat);
      	 done = true;
      }
   }     

// determine what variable to plot from the potential file

   int iWhatP[20] = {0};
   for (int i=0; i<20; i++) iWhatP[i] = -1;
   if (iUseFile[2] == 1) getPotentialLocations(&readPotential, iWhatP);   

// determine the begin & end date/time from the PEACE file(s)

   QString beginDateTime[2], endDateTime[2];
   double beginDateTimeDec[2], endDateTimeDec[2];
   
   for (int i=0; i<2; i++)
   {
      if (iUseFile[i] == 1)
      {
         beginDateTime[i] = readPeace[i].getFileBeginDateTime();
         beginDateTimeDec[i] = qTimeToDec(beginDateTime[i]);
         endDateTime[i] = readPeace[i].getFileEndDateTime();	
         endDateTimeDec[i] = qTimeToDec(endDateTime[i]);
      }
   }

   QString titlePart;
   double beginTimeOnly = 0, endTimeOnly = 0;
   QString beginDateTimeUsed;

   if (iUseFile[0] == 1 && iUseFile[1] != 1)
   {
      beginTimeOnly = time_to_tis((char *)(beginDateTime[0].mid(11, 12).toStdString().c_str()));
      endTimeOnly = time_to_tis((char *)(endDateTime[0].mid(11, 12).toStdString().c_str()));
      titlePart = datasetIdNU[0];
      beginDateTimeUsed = beginDateTime[0];
   }
   else if (iUseFile[1] == 1 && iUseFile[0] != 1)
   {
      beginTimeOnly = time_to_tis((char *)(beginDateTime[1].mid(11, 12).toStdString().c_str()));
      endTimeOnly = time_to_tis((char *)(endDateTime[1].mid(11, 12).toStdString().c_str()));   
      titlePart = datasetIdNU[1];	
      beginDateTimeUsed = beginDateTime[1];
   }
   else if (iUseFile[0] == 1 && iUseFile[1] == 1)
   {
     if (beginDateTimeDec[0] < beginDateTimeDec[1])
     {
        beginTimeOnly = time_to_tis((char *)(beginDateTime[0].mid(11, 12).toStdString().c_str()));
        beginDateTimeUsed = beginDateTime[0];
     }
     else
     {
        beginTimeOnly = time_to_tis((char *)(beginDateTime[1].mid(11, 12).toStdString().c_str()));
        beginDateTimeUsed = beginDateTime[1];
     }
        
     if (endDateTimeDec[0] > endDateTimeDec[1])
        endTimeOnly = time_to_tis((char *)(endDateTime[0].mid(11, 12).toStdString().c_str()));
     else
        endTimeOnly = time_to_tis((char *)(endDateTime[1].mid(11, 12).toStdString().c_str())); 
     titlePart = datasetIdNU[0] + " + " + datasetIdNU[1];
   }

   QDate initialDate = toQDate(beginDateTimeUsed.mid(0, 10));

// setup the plot

   QsiPlot myPlot(plotWidth, plotHeight, "", -1, false);
   myPlot.initialDate = initialDate;
   
   myPlot.setPlotLineWidth(1);
   myPlot.setCurveLineWidth(1);
   myPlot.setBackgroundColor(Qt::white);
   myPlot.setLayout(1, 1, titlePart);
   myPlot.setLegendLocation(TopRight);
   
   myPlot.showValuesX(0, 0, pTrue);
   myPlot.setPlotType(0, 0, Spectrogram);
   myPlot.setScaleTypeY(0, 0, Log);
   myPlot.setScaleTypeX(0, 0, Linear);

// allocate storage for the potential data

   double *pX = new double[86400];
   double *pA = new double[86400];
   
// read in spacecraft potential

   int iCountP = 0;
   if (iUseFile[2] == 1)
   {
      double fillVal = (readPotential.getVariableAttr(iWhatP[_Spacecraft_potential], "FILLVAL")).toFloat();
      while (readPotential.readRecord() != _READ_FAILED)
      {
      	 float dF;
         QString timeData;
         readPotential.getDataEpoch(iWhatP[_time_tags], &timeData);
         readPotential.getDataValue(iWhatP[_Spacecraft_potential], &dF);
         
         if (dF != fillVal)
         {
            pA[iCountP] = -dF + scpOff1;
            pX[iCountP] = time_to_tis((char *)(timeData.mid(11,12).toStdString()).c_str());
            iCountP++;	
         }    
      }
   }

// plot axis setup

   int nMaj, nMin, nMinX;
   double vMin, vMax, tMin, tMax, durMajX;
   
   myPlot.getAutoScaleX(pX, -1, beginTimeOnly, endTimeOnly, &durMajX, &nMinX, &tMin, &tMax);
   myPlot.getAutoScaleY(yMin, yMax, &nMaj, &nMin, &vMin, &vMax, Log);
   myPlot.setPlotAxisLimits(0, 0, tMin, tMax, vMin, vMax);
   myPlot.drawFrame(0, 0, false);
   myPlot.setLabelY(0, 0, "energy (eV)");
   
   QString myPlotTitle;
   if (unitsSpectro == _PSD) myPlotTitle = "Phase space density (s^3/km^6)";
   else if (unitsSpectro == _DNF) myPlotTitle = "Differential number flux";
   else myPlotTitle = "Differential energy flux (ergs/cm^2-s-str-eV)";
   myPlot.drawColourBox(0, 0, cMin, cMax, myPlotTitle);

// prepare units
// - conversion factors required for PSD (sec**3/km**6) to DEF & DNF
// - using numbers obtained directly from PEACE VIDFs

   double cPtoD = (1602.0*1.0e-15)*(1602.0*1.0e-15)*(1602.0*1.0e-15)/(4149605.0*1.0e-31);
   double cPtoN = 1.0; // not done yet!

   double cPtoDorN = 1.0;
   if (unitsSpectro == _DEF) cPtoDorN = cPtoD;
   else if (unitsSpectro == _DNF) cPtoN = 1.0; // not completed yet!
   
// read in PEACE data and plot it

   for (int i=0; i<2; i++)
   {
      int iScpCount = 0;
      if (iUseFile[i] == 1)
      {

// determine the fill value

         double fillVal = -1;
         if (twoSensorsInSingleFile == 0)
         {
            fillVal = (readPeace[i].getVariableAttr(iWhat[_Data], "FILLVAL")).toFloat();    	
         }
         else
         {
            fillVal = (readPeace[i].getVariableAttr(iWhat[_Data_Overlap_LEEA], "FILLVAL")).toFloat();
         }
  
// loop over all data  
      	      		    	
         while (readPeace[i].readRecord() != _READ_FAILED)
         {

            // various declarations

            float *energyCentre, *energyLower, *energyUpper;
            float *energyCentre1, *energyLower1, *energyUpper1;

            float *azimuthCentre, *azimuthLower, *azimuthUpper;
            float *azimuthCentre1, *azimuthLower1, *azimuthUpper1;

            float *data, *data1;

            int sizeE, sizeE1;
            int sizeA, sizeA1; 
            int sizeP = 0, sizeP1;
      
            int sizeD[3] = {0}, sizeEr;
            int sizeD1[3] = {0}, sizeEr1;

            if (twoSensorsInSingleFile == 0)
            {
      	       sizeE = readPeace[i].getVariableSize(iWhat[_Energy], 0);
      	       sizeA = readPeace[i].getVariableSize(iWhat[_Azimuth], 0);
      	 
               energyCentre = new float[sizeE];
      	       energyLower = new float[sizeE];
      	       energyUpper = new float[sizeE];
      	 
       	       azimuthCentre = new float[sizeA];
      	       azimuthLower = new float[sizeA];
      	       azimuthUpper = new float[sizeA];    
      	 
      	       readPeace[i].getDataValue(iWhat[_Mode_RealSize], &sizeD[0]);
      	       readPeace[i].getDataValue(iWhat[_Mode_RealBinNum], &sizeEr);
      	    
               data = new float[sizeD[0]*sizeD[1]*sizeD[2]];
            }
            else
            {
               sizeE = readPeace[i].getVariableSize(iWhat[_Energy_Overlap_LEEA], 0);
               sizeA = readPeace[i].getVariableSize(iWhat[_Azimuth_Overlap_LEEA], 0);

               energyCentre = new float[sizeE];
               energyLower = new float[sizeE];
               energyUpper = new float[sizeE];

               azimuthCentre = new float[sizeA];
               azimuthLower = new float[sizeA];
               azimuthUpper = new float[sizeA];

               readPeace[i].getDataValue(iWhat[_Mode_RealSize_Overlap_LEEA], &sizeD[0]);
               readPeace[i].getDataValue(iWhat[_Mode_RealBinNum_Overlap_LEEA], &sizeEr);

               data = new float[sizeD[0]*sizeD[1]*sizeD[2]];
               sizeP = sizeD[1];

               sizeE1 = readPeace[i].getVariableSize(iWhat[_Energy_Overlap_HEEA], 0);
               sizeA1 = readPeace[i].getVariableSize(iWhat[_Azimuth_Overlap_HEEA], 0);

               energyCentre1 = new float[sizeE1];
               energyLower1 = new float[sizeE1];
               energyUpper1 = new float[sizeE1];

               azimuthCentre1 = new float[sizeA1];
               azimuthLower1 = new float[sizeA1];
               azimuthUpper1 = new float[sizeA1];

               readPeace[i].getDataValue(iWhat[_Mode_RealSize_Overlap_HEEA], &sizeD1[0]);
               readPeace[i].getDataValue(iWhat[_Mode_RealBinNum_Overlap_HEEA], &sizeEr1);

               data1 = new float[sizeD1[0]*sizeD1[1]*sizeD1[2]];
               sizeP1 = sizeD1[1];
            }

// get timing info
         	
            QString time_stamp;
            float time_tags_DeltaLower, time_tags_DeltaUpper, anglePhi;

            readPeace[i].getDataEpoch(iWhat[_time_tags], &time_stamp);
            readPeace[i].getDataValue(iWhat[_time_tags_DeltaLower], &time_tags_DeltaLower);
            readPeace[i].getDataValue(iWhat[_time_tags_DeltaUpper], &time_tags_DeltaUpper); 	
            readPeace[i].getDataValue(iWhat[_Angle_SR2phi], &anglePhi);
            
            double tisSpinCentre = time_to_tis((char *)(time_stamp.mid(11,12).toStdString()).c_str());
            double spinDur = time_tags_DeltaLower + time_tags_DeltaUpper;
	    double tisSpinBegin = tisSpinCentre - time_tags_DeltaLower;
 
// get energy info 
            
            if (twoSensorsInSingleFile == 0)
            {
               readPeace[i].getDataValue(iWhat[_Energy], energyCentre);
               readPeace[i].getDataValue(iWhat[_Energy_DeltaLower], energyLower);
               readPeace[i].getDataValue(iWhat[_Energy_DeltaUpper], energyUpper);
            }
            else
            {
               readPeace[i].getDataValue(iWhat[_Energy_Overlap_LEEA], energyCentre);
               readPeace[i].getDataValue(iWhat[_Energy_DeltaLower_Overlap_LEEA], energyLower);
               readPeace[i].getDataValue(iWhat[_Energy_DeltaUpper_Overlap_LEEA], energyUpper);

               readPeace[i].getDataValue(iWhat[_Energy_Overlap_HEEA], energyCentre1);
               readPeace[i].getDataValue(iWhat[_Energy_DeltaLower_Overlap_HEEA], energyLower1);
               readPeace[i].getDataValue(iWhat[_Energy_DeltaUpper_Overlap_HEEA], energyUpper1);
            }
            
// get the azimuth info    

            if (twoSensorsInSingleFile == 0)
            {
               readPeace[i].getDataValue(iWhat[_Azimuth], azimuthCentre);
               readPeace[i].getDataValue(iWhat[_Azimuth_DeltaLower], azimuthLower);
               readPeace[i].getDataValue(iWhat[_Azimuth_DeltaUpper], azimuthUpper); 
            }
            else
            {
               readPeace[i].getDataValue(iWhat[_Azimuth_Overlap_LEEA], azimuthCentre);
               readPeace[i].getDataValue(iWhat[_Azimuth_Overlap_LEEA_DeltaLower], azimuthLower);
               readPeace[i].getDataValue(iWhat[_Azimuth_Overlap_LEEA_DeltaUpper], azimuthUpper);

               readPeace[i].getDataValue(iWhat[_Azimuth_Overlap_HEEA], azimuthCentre1);
               readPeace[i].getDataValue(iWhat[_Azimuth_Overlap_HEEA_DeltaLower], azimuthLower1);
               readPeace[i].getDataValue(iWhat[_Azimuth_Overlap_HEEA_DeltaUpper], azimuthUpper1);
            }

// get the data

            if (twoSensorsInSingleFile == 0)
            {
               readPeace[i].getDataValue(iWhat[_Data], data);
            }
            else
            {
               readPeace[i].getDataValue(iWhat[_Data_Overlap_LEEA], data);
               readPeace[i].getDataValue(iWhat[_Data_Overlap_HEEA], data1);
            }
            
// generate spectrogram slice

            for (int jA=0; jA<sizeD[0]; jA++)
            {
               double xBegin = tisSpinBegin + ((double)jA)*spinDur/((double)sizeD[0]);
               double xEnd = tisSpinBegin + ((double)(jA + 1))*spinDur/((double)sizeD[0]);
	       
	       double currentPotential = 0.0;
	       if (spectroEnergy > 0)
	       {
	          getSpacecraftPotential(xBegin, xEnd, &iScpCount, iCountP,
		                         &pX[0], &pA[0], &currentPotential);
//					 cout << "pot = " << currentPotential << " " << iScpCount << endl;
	       }
	       
               for (int jE=0; jE<sizeD[2]; jE++)
               {
               	  float thisEnergyLower = energyCentre[jE] - energyLower[jE];
               	  float thisEnergyUpper = energyCentre[jE] + energyUpper[jE];
		  
		  if (spectroEnergy == 2 && fabs(currentPotential) < MAX_VALUE_PM)
		  {
		     thisEnergyLower -= currentPotential;
		     thisEnergyUpper -= currentPotential;
		     
		     if (thisEnergyUpper < 0)
		     {
		        thisEnergyLower = -100;
			thisEnergyUpper = -100;
		     } 
		     else if (thisEnergyLower < 0) thisEnergyLower = 0;
		  }   
		  else if (spectroEnergy == 1 && fabs(currentPotential) < MAX_VALUE_PM)
		  {
		     if (thisEnergyLower <= currentPotential)
	             {
		        thisEnergyLower = -100;
			thisEnergyUpper = -100;		         
		     }		 
		  }
		  
                  double conv = energyCentre[jE]*energyCentre[jE]*cPtoDorN; // conversion from PSD to DEF/DNF
                  if (unitsSpectro == _PSD) conv = 1.0;
               	  if (energyCentre[jE] >= 0) // handle flybacks
               	  {
               	     double zval = 0;
                     for (int jP=0; jP<sizeD[1]; jP++)
                     {
                        double dc = data[jE + jP*sizeE + jA*sizeP*sizeE]*conv;
                        if (dc > 0 && dc != fillVal) zval = zval + dc;
                     } // <-- polar loop
                     zval = zval/((double)sizeD[1]);
                     myPlot.drawSpectrogramSlice(0, 0, zval, xBegin, xEnd, thisEnergyLower, thisEnergyUpper,
                                                 cMin, cMax);
               	  }
               } // <-- energy loop	
            } // <-- azimuth loop
            
            delete [] data;
            delete [] energyCentre;
            delete [] energyLower;
            delete [] energyUpper;
            delete [] azimuthCentre;
            delete [] azimuthLower;
            delete [] azimuthUpper;            
            
         } // <-- while loop                     
      }
   }   

   
// draw potential if necessary
// - only needs to be shown if user hasn't select spacecraft-potential corrected spectrogram

   if (iCountP > 0 && spectroEnergy != 2) myPlot.drawCurve(0, 0, Qt::black, iCountP, pX, pA, "");   

// finish making the plot

   myPlot.setNumTicksY(0, 0, nMaj, nMin);
   myPlot.setNumTicksX(0, 0, durMajX, nMinX);
   myPlot.drawFrame(0, 0);

// make the QImage available

   qPlot = myPlot.plot->copy(0, 0, myPlot.plot->width(), myPlot.plot->height());

// save plot to file if necessary

   if (!fileSave.isEmpty()) myPlot.saveToFile(fileSave);
   
// close files & tidy up

   if (iUseFile[0] == 1) readPeace[0].close();
   if (iUseFile[1] == 1) readPeace[1].close();
   if (iUseFile[2] == 1) readPotential.close();

   delete [] pX;
   delete [] pA;
   
   return(_SUCCESS);      
}

//-----------------------------------------------------------------------------
// 
// determine spacecraft potential
//
//-----------------------------------------------------------------------------

void SpectroMaker::getSpacecraftPotential(double azBegin, double azEnd, int *iScpCount, int iMax,
                                          double *pX, double *pA, double *potential)
{

   double azCentre = (azBegin + azEnd)/2.0;
   

   int found = 0, iFoundC = -1, iFoundL = -1, iFoundH = -1;
   while (found == 0 && *iScpCount < iMax)
   {
//      cout << "WHILE " << *iScpCount << " iMax = " << iMax;
//      cout << " :: " << pX[*iScpCount] << " beg = " << azBegin << " end = " << azEnd << endl;
      if (pX[*iScpCount] >= azBegin && pX[*iScpCount] < azEnd)
      {
         iFoundC = *iScpCount;
         found = 1;
         *iScpCount = *iScpCount - 8;
	 if (*iScpCount < 0) *iScpCount = 0;
      }
      else if (pX[*iScpCount] < azBegin)
      {
         iFoundL = *iScpCount;
      }
      else if (pX[*iScpCount] > azEnd)
      {
         iFoundH = *iScpCount;
      }
      
      *iScpCount = *iScpCount + 1;
		  
      if (iFoundL > -1 && iFoundH > -1)
      {
         if (fabs(pX[iFoundL] - azBegin) < ALMOST_ENOUGH &&
             fabs(pX[iFoundH] - azEnd) < ALMOST_ENOUGH)
         {
            found = 2;
            *iScpCount = *iScpCount - 8;
            if (*iScpCount < 0) *iScpCount = 0;
         }
      }
   }
   
//   cout << "** FOUND = " << found << endl;
	       
   if (found == 1)
   {
      *potential = pA[iFoundC];
   }	 
   else if (found == 2)
   {
      *potential = interpolateLinear(pX[iFoundL], pX[iFoundH],
		                     pA[iFoundL], pA[iFoundH], azCentre);
   }
   else *potential = 0.0;
}	    
