/*******************************************************************************
 **
 ** $Id: statushandler.C 171 2008-12-03 11:42:42Z AndrewLahiff $
 **
 ******************************************************************************
 **
 ** Copyright (C) 2007-2008 Andrew Lahiff.
 **
 ** This file is part of PEACEmoments CAA.
 **
 ** PEACEmoments CAA is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** PEACEmoments CAA is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with PEACEmoments CAA.  If not, see <http://www.gnu.org/licenses/>.
 **
 *******************************************************************************/

#include "peacemoments_caa.h"

int statusHandler (int status, char message[])
{
   switch (status)
   {
      case _NOT_VALID_PEACE_FILE:
         sprintf(message, "Not a valid PEACE data file");
         break;
      case _NOT_VALID_PEACE_UNITS:
	 sprintf(message, "Not valid PEACE units for moments calculation");
	 break;
      case _NOT_VALID_POTENTIAL_FILE:
	 sprintf(message, "Not a valid potential file");
	 break;
      case _NOT_VALID_MAG_FIELD_FILE:
	 sprintf(message, "Not a valid magnetic field file");
	 break;
      case _NOT_VALID_AUX_GSE_FILE:
         sprintf(message, "Not a valid AUX GSE spin-axis angle file");
	 break;
      case _NOT_VALID_TWO_PEACE_FILES:
	 sprintf(message, "Invalid combination of PEACE data files");
	 break;
      case _FAILED_READ_DATA_REC_PEACE_1:
	 sprintf(message, "Failed to read a data record from PEACE data file");
	 break;
      case _FAILED_READ_DATA_REC_PEACE_2:
	 sprintf(message, "Failed to read a data record from PEACE data file");
	 break;
      case _FAILED_READ_DATA_REC_POTENTIAL:
	 sprintf(message, "Failed to read a data record from spacecraft potential file");
	 break;
      case _FAILED_READ_DATA_REC_MAG_FIELD:
	 sprintf(message, "Failed to read a data record from magnetic field file");
	 break;
      case _FAILED_READ_DATA_REC_AUX_GSE:
	 sprintf(message, "Failed to read a data record from AUX GSE spin-axis angle file");
	 break;
      case _FAILED_OPEN_FILE_OUT_MOMENTS:
	 sprintf(message, "Failed to open moments output file");
	 break;
      case _FAILED_OPEN_FILE_IN_PEACE_1:
	 sprintf(message, "Failed to open PEACE data file");
	 break;
      case _FAILED_OPEN_FILE_IN_PEACE_2:
	 sprintf(message, "Failed to open PEACE data file");
	 break;
      case _FAILED_OPEN_FILE_IN_POTENTIAL:
	 sprintf(message, "Failed to open spacecraft potential file");
	 break;
      case _FAILED_OPEN_FILE_IN_MAG_FIELD:
	 sprintf(message, "Failed to open magnetic field file");
	 break;
      case _FAILED_OPEN_FILE_IN_AUX_GSE:
	 sprintf(message, "Failed to open AUX GSE spin-axis angle file");
	 break;
      case _NO_MOMENTS_CALCULATED:
	 sprintf(message, "Unable to calculate any moments");
	 break;
      case _NOT_VALID_SCN_PEACE_FILES:
	 sprintf(message, "The two PEACE files are not for the same spacecraft");
	 break;
      case _NOT_VALID_SCN_POTENTIAL_FILE:
	 sprintf(message, "The spacecraft potential file is not for the correct spacecraft");
	 break;
      case _NOT_VALID_SCN_MAG_FIELD_FILE:
	 sprintf(message, "The magetic field file is not for the correct spacecraft");
	 break;
      case _NOT_VALID_SCN_AUX_GSE_FILE:
	 sprintf(message, "The AUX GSE spin axis file is not for the correct spacecraft");
	 break;
      case _NO_VALID_PEACE_FILES_SPECIFIED:
         sprintf(message, "No valid PEACE data files specified");
         break;
      case _AUX_GSE_FILE_REQUIRED_FOR_COORD:
         sprintf(message, "AUX GSE spin-axis angle data is required for the selected coordinate system");
         break;
      case _MAG_FIELD_FILE_REQUIRED_FOR_COORD:
         sprintf(message, "Magnetic field data is required for the selected coordinate system");
         break;
      case _ERROR_OPENING_FILE:
         sprintf(message, "Unable to open file");
         break;
      case _LEEA_DATA_REQUIRED_FOR_REQ_SENS_COMBI:
         sprintf(message, "LEEA data is required for the requested sensor combination");
         break;
      case _HEEA_DATA_REQUIRED_FOR_REQ_SENS_COMBI:
         sprintf(message, "HEEA data is required for the requested sensor combination");
         break;
      case _ONLY_SINGLE_FILE_REQUIRED:
         sprintf(message, "A second PEACE file is not needed since the first file contains both sensors");
         break;
      case _INCORRECT_COORD_SYSTEM_FOR_CE:
         sprintf(message, "Incorrect coordinate system for calculating characteristic energy");
         break;
      case _MAG_FIELD_REQ_FOR_CE:
         sprintf(message, "Magnetic field is required for calculating characteristic energy");
         break;
      default:
	 sprintf(message, "Unexpected error");
	 return(_FAILED);
   }
   return(_SUCCESS);
}


