/*******************************************************************************
 **
 ** $Id: thread.C 149 2008-11-28 20:51:29Z AndrewLahiff $
 **
 ******************************************************************************
 **
 ** Copyright (C) 2007-2008 Andrew Lahiff.
 **
 ** This file is part of PEACEmoments CAA.
 **
 ** PEACEmoments CAA is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** PEACEmoments CAA is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with PEACEmoments CAA.  If not, see <http://www.gnu.org/licenses/>.
 **
 *******************************************************************************/

#include "peacemoments_caa.h"
#include "peacemoments_gui.h"
#include "thread.h"
#include "plotmaker.h"

//-----------------------------------------------------------------------------
// 
// constructor
//         
//-----------------------------------------------------------------------------

calculateMomentsThread::calculateMomentsThread(QObject *parent)
   : QThread(parent)
{
   peaceMoments = new PeaceMoments;
}

//-----------------------------------------------------------------------------
// 
// destructor
//         
//-----------------------------------------------------------------------------

calculateMomentsThread::~calculateMomentsThread()
{
   delete peaceMoments;
}

//-----------------------------------------------------------------------------
// 
// thread run() method
//         
//-----------------------------------------------------------------------------

void calculateMomentsThread::run()
{
   bool usePeace1 = false, usePeace2 = false;
   bool useScPot = false, useGSE = false, useB = false;

// specify what files are needed
   
   if (filePeace1->size() > 1) usePeace1 = true;
   if (filePeace2->size() > 1) usePeace2 = true;
   if (filePotential->size() > 1) useScPot = true;
   if (fileAuxGSE->size() > 1) useGSE = true;
   if (fileMagField->size() > 1) useB = true;

// output file; take into account user-specified output directory

   QString myFileOut;
   if (dirOut->isEmpty()) myFileOut = *fileOut;
   else myFileOut = *dirOut + "/" + *fileOut;

// has the user switched off spacecraft potential correction
   
   if (!myParams->potentialCorrectionOnOff) useScPot = false;

// prepare to begin...

   int numRecords = 0;

   if (whatToDo == 0) numRecords = -1;

//------------------
// calculate moments
//------------------

   int ret = _SUCCESS;
   char message[100];

   if (whatToDo == 1)
   {

// announce that we are currently calculating moments

      emit currentlyDoing(_CURRENTLY_DOING_MOMENTS);

// now we actually run the moments code
       
      ret = peaceMoments->run_calc(0, usePeace1, usePeace2, useScPot, useB, useGSE,
                   (char*)((filePeace1->toStdString()).c_str()),
                   (char*)((filePeace2->toStdString()).c_str()),
                   (char*)((filePotential->toStdString()).c_str()),
                   (char*)((fileAuxGSE->toStdString()).c_str()),
                   (char*)((fileMagField->toStdString()).c_str()),
                   myParams->sensorCombination, 
                   myParams->coordinateSystem, 
                   *myWML, *myPML,
                   myParams->potentialOffset1, 
                   myParams->potentialOffset2,           		
                   myParams->potentialOffset2Units,
                   myParams->energyRestrictStatus,
		   myParams->energyRestrictScpType,
                   myParams->energyRestrictType,
                   myParams->energyRestrictLow,
                   myParams->energyRestrictHigh,
                   myParams->polarRestrictStatus,
                   myParams->polarRestrictLow,
                   myParams->polarRestrictHigh,
                   myParams->filterQuality,
                   myParams->filterEFW,
                   myParams->filterWHI,
                   myParams->filterStats,
                   myParams->filterDataStream,
                   (char*)((myFileOut.toStdString()).c_str()),
                   &numRecords, true, false, 0.0);

      if (ret != _SUCCESS) statusHandler(ret, message);
      else sprintf(message, "");
   }

//-----------------------------------
// generate moments plot if necessary
//-----------------------------------

   if (myPML->on && ret == _SUCCESS)
   {

// announce that we are currently making the plot

      emit currentlyDoing(_CURRENTLY_DOING_MOMENTS_PLOT);

// make the plot

      PlotMaker myPlot;
      myPlot.mySettings = myPML;
      int iPlotStatus = myPlot.makePlot(*fileOut, *filePotential);

      momentsQImage = myPlot.qPlot.copy(0, 0, myPlot.qPlot.width(), myPlot.qPlot.height());

// send the QImage if the plot was successfully generated

      if (iPlotStatus == _SUCCESS) emit thePlot(momentsQImage);
   }

// announce that the moments calculation & plotting has finished

   emit returnedMessage(ret, message, numRecords);

}

