/*******************************************************************************
 **
 ** $Id: threadspectro.C 149 2008-11-28 20:51:29Z AndrewLahiff $
 **
 ******************************************************************************
 **
 ** Copyright (C) 2007-2008 Andrew Lahiff.
 **
 ** This file is part of PEACEmoments CAA.
 **
 ** PEACEmoments CAA is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** PEACEmoments CAA is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with PEACEmoments CAA.  If not, see <http://www.gnu.org/licenses/>.
 **
 *******************************************************************************/

#include "peacemoments_caa.h"
#include "peacemoments_gui.h"
#include "threadspectro.h"
#include "spectromaker.h"

//-----------------------------------------------------------------------------
// 
// constructor
//         
//-----------------------------------------------------------------------------

generateSpectrogramThread::generateSpectrogramThread(QObject *parent)
   : QThread(parent)
{

}

//-----------------------------------------------------------------------------
// 
// destructor
//         
//-----------------------------------------------------------------------------

generateSpectrogramThread::~generateSpectrogramThread()
{

}

//-----------------------------------------------------------------------------
// 
// thread run() method
//         
//-----------------------------------------------------------------------------

void generateSpectrogramThread::run()
{
   SpectroMaker mySpectro;
   mySpectro.yMin = yMin;
   mySpectro.yMax = yMax;
   mySpectro.cMin = cMin;
   mySpectro.cMax = cMax;
   mySpectro.plotHeight = plotHeight;
   mySpectro.plotWidth = plotWidth;
   
   mySpectro.filePeace1 = filePeace1;
   mySpectro.filePeace2 = filePeace2;
   mySpectro.filePotential = filePotential;
   mySpectro.fileSave = fileToSave;

   mySpectro.scpOff1 = scpOff1;
   mySpectro.scpOff2 = scpOff2;
   mySpectro.scpOff2units = scpOff2units;

   mySpectro.spectroEnergy = spectroEnergy;    // how energy bins are plotted
   mySpectro.spectroPolars = spectroPolars;    // how polars are considered
   mySpectro.spectroSensors = spectroSensors;  // what sensors are to be plotted

   if (units.contains("Phase space density"))
   {
      mySpectro.unitsSpectro = _PSD;
   }
   else if (units.contains("Differential number flux"))
   {
      mySpectro.unitsSpectro = _DNF;
   }
   else
   {
      mySpectro.unitsSpectro = _DEF;
   }

   int iSpectroStatus = mySpectro.makePlot();
   spectroQImage = mySpectro.qPlot.copy(0, 0, mySpectro.qPlot.width(), mySpectro.qPlot.height());
   
   if (iSpectroStatus == _SUCCESS) emit thePlot(spectroQImage);
   emit returnedStatus(iSpectroStatus);
}

