/*******************************************************************************
 **
 ** $Id: window.h 166 2008-11-30 09:42:37Z AndrewLahiff $
 **
 ******************************************************************************
 **
 ** Copyright (C) 2007-2008 Andrew Lahiff.
 **
 ** This file is part of PEACEmoments CAA.
 **
 ** PEACEmoments CAA is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** PEACEmoments CAA is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with PEACEmoments CAA.  If not, see <http://www.gnu.org/licenses/>.
 **
 *******************************************************************************/

#ifndef WINDOW_H_
#define WINDOW_H_

#include "thread.h"
#include "threadspectro.h"

#include <QtGui>
#include <QHttp>
#include <QDomDocument>

//=============================================================================
//
// Data tab
//
//=============================================================================

class DataTab : public QWidget
{
   Q_OBJECT

   public:
      DataTab(QWidget *parent = 0);
     
      QSpinBox *stYearSpinBox;
      QSpinBox *stDoySpinBox;
      QDateTimeEdit *stDateEdit;
      QDateTimeEdit *stTimeEdit;
      QSpinBox *spYearSpinBox;
      QSpinBox *spDoySpinBox;
      QDateTimeEdit *spDateEdit;
      QDateTimeEdit *spTimeEdit;       
      
      QComboBox *dataScn;
      QComboBox *dataPeace1;
      QComboBox *dataPeace2;
      QComboBox *dataPotential;
      QComboBox *dataAuxGSE;
      QComboBox *dataMagField;
      
      QLineEdit *authUser;
      QLineEdit *authPass;
      
      QPushButton *downloadData;
      QPushButton *availableData;
      QPushButton *copyTimeDate;

      QHttp *http;
      QFile *downloadFile;

      int downloadTracker;

      void downloadActualData();
      void extractFiles();
    
      QTemporaryFile *qfileTemp;
      QString *qfilePeace1, *qfilePeace2, *qfilePotential, *qfileFGM, *qfileGSE;
      QStringList *filesDownloaded;

      QString userDirOut, dataDirectory;
      
   private:
      void errorBox(QStringList);
   
   private slots:
      void copyStartToStop();     
      void modifiedStartDoy(int doy);
      void modifiedStopDoy(int doy);
      void modifiedStartDate(const QDate &startDate);
      void modifiedStopDate(const QDate &stopDate);
      void modifiedStartYear(int year);
      void modifiedStopYear(int year);   
      void slotDownloadData();
      void slotAvailableData();

   signals:
      void downloadAborted();
      
};

//=============================================================================
//
// Files tab
//
//=============================================================================

class FilesTab : public QWidget
{
   Q_OBJECT

   public:
      FilesTab(QWidget *parent = 0);
      
      QLineEdit *editFilePeace1;
      QLineEdit *editFilePeace2;
      QLineEdit *editFilePotential;
      QLineEdit *editFileAuxGSE;
      QLineEdit *editFileMagField;

      QString *userFilePeace1;
      QString *userFilePeace2;
      QString *userFilePotential;
      QString *userFileAuxGSE;
      QString *userFileMagField;

      QString currentDirectory;
      
   private:
      QPushButton *selectEditFilePeace1;
      QPushButton *selectEditFilePeace2;
      QPushButton *selectEditFilePotential;
      QPushButton *selectEditFileAuxGSE;
      QPushButton *selectEditFileMagField;         

   private slots:
      void slotEditFilePeace1();
      void slotEditFilePeace2();
      void slotEditFilePotential();
      void slotEditFileAuxGSE();
      void slotEditFileMagField();
      
};

//=============================================================================
//
// moments plot window
//
//=============================================================================

class PlotMoments : public QWidget
{
   Q_OBJECT

   public:
      PlotMoments(int, int, QWidget *parent = 0);
      void setMomentsWindowSize(int, int);
      QPixmap plot;
      int plotHeight;
      int plotWidth;

   private:

   protected:
      void paintEvent(QPaintEvent *event);

};

//=============================================================================
//
// spectogram plot window
// 
//=============================================================================

class PlotSpectrogram : public QWidget
{
   Q_OBJECT
   
   public:
      PlotSpectrogram(int, int, QWidget *parent = 0);
      void setSpectrogramWindowSize(int, int);
      QPixmap plot;
      int plotHeight;
      int plotWidth;

   private:

   protected:
      void paintEvent(QPaintEvent *event);

};

//=============================================================================
//
// plot window
//
//=============================================================================

class PlotWindow : public QWidget
{
   Q_OBJECT

   public:
      PlotWindow(QWidget *parent = 0);
      
      QGroupBox *plotSettingsMoments;
      QGroupBox *plotSettingsSpectro;
      QPushButton *closePlotWindowButton;
      QPushButton *replotMoments;
      QPushButton *replotSpectro;
      QPushButton *saveAsDefaultsButton;
      QPushButton *saveMomentsToFile;
      QPushButton *saveSpectroToFile;
      
      QCheckBox *iPlotN;
      QCheckBox *iPlotV;
      QCheckBox *iPlotQ;
      QCheckBox *iPlotH;
      QCheckBox *iPlotVpara;
      QCheckBox *iPlotVperp;
      QCheckBox *iPlotQpara;
      QCheckBox *iPlotQperp;
      QCheckBox *iPlotHpara;
      QCheckBox *iPlotHperp;
      QCheckBox *iPlotP;
      QCheckBox *iPlotT;
      QCheckBox *iPlotTpara;
      QCheckBox *iPlotTperp;
      QCheckBox *iPlotJ;
      QCheckBox *iPlotJpara;
      QCheckBox *iPlotJperp;
      QCheckBox *iPlotB;
      QCheckBox *iPlotS;
      QCheckBox *iPlotScP;
      
      QLineEdit *spectroYmin;
      QLineEdit *spectroYmax;
      QLineEdit *spectroCmin;
      QLineEdit *spectroCmax;

      QComboBox *spectroUnitsCombo;
      QComboBox *spectroAnodesCombo;
      QComboBox *spectroEnergyCombo;
      QComboBox *spectroSensorCombo;

   private:

   public slots:

   private slots:
      void closePlotWindow();
      void saveAsDefaultsSlot();
      void slotReplotMoments();
      void slotPlotSpectro();
      void slotSaveSpectroToFile();
      void slotSaveMomentsToFile();

   signals:
      void remakeMomentsPlot(QString);
      void makeSpectrogramPlot(QString);

};

//=============================================================================
//
// integration window
//
//=============================================================================

class IntegrationWindow : public QWidget
{
   Q_OBJECT

   public:
      IntegrationWindow(QWidget *parent = 0);

      QPushButton *energyFileButton;
      QLineEdit *energyLowEdit;
      QLineEdit *energyHighEdit;
      QGroupBox *energyIntGroup;
      QLineEdit *energyFileEdit;
      QComboBox *energyIntSelect;
      QCheckBox *energyIntType;
      QCheckBox *energyIntSpType;

      QLineEdit *polarLowEdit;
      QLineEdit *polarHighEdit;
      QGroupBox *polarIntGroup;

      QPushButton *closeIntegrationButton;

   private:

   public slots:

   private slots:
      void closeIntegrationWindow();
      void slotEnergyIntSelect(int);

};

//=============================================================================
//
// output settings window
//
//=============================================================================

class OutputWindow : public QWidget
{  
   Q_OBJECT
   
   public:
      OutputWindow(QWidget *parent = 0);
      QCheckBox *inclOutN;
      QCheckBox *inclOutV;
      QCheckBox *inclOutQ;
      QCheckBox *inclOutH;
      QCheckBox *inclOutVpara;
      QCheckBox *inclOutVperp;
      QCheckBox *inclOutQpara;
      QCheckBox *inclOutQperp;
      QCheckBox *inclOutHpara;
      QCheckBox *inclOutHperp;
      QCheckBox *inclOutP;
      QCheckBox *inclOutT;
      QCheckBox *inclOutTpara;
      QCheckBox *inclOutTperp;
      QCheckBox *inclOutB;
      QCheckBox *inclOutScP;
      QCheckBox *inclOutJ;
      QCheckBox *inclOutJpara;
      QCheckBox *inclOutJperp;
      QCheckBox *inclOutCE;

      QCheckBox *filterQUA;
      QCheckBox *filterEFW;
      QCheckBox *filterWHI;
      QCheckBox *filterStats;
      QCheckBox *filterDataStream;

      QPushButton *saveAsDefaultsButton;
      QPushButton *closeOutputButton;
 
      QPushButton *selectAllMoments;
      QPushButton *selectAllFilters;

      QLineEdit *outputDirectoryEdit;
      QLineEdit *outputFileEdit;
     
      QString *currentOutputDirectory;
   
   private:
   
   public slots:

   private slots:
      void closeOutputWindow();
      void slotSelectOutputDirectory();
      void saveAsDefaultsSlot();
      void slotSelectAllMoments();
      void slotSelectAllFilters();

};

//=============================================================================
//
// main window class
//
//=============================================================================

class Window : public QMainWindow
{  
   Q_OBJECT
		         
   public: 
      Window(char *fileToLoad);
      ~Window();

      QString *userFilePeace1;
      QString *userFilePeace2;
      QString *userFilePotential;
      QString *userFileAuxGSE;
      QString *userFileMagField;
      QString *userFileOut;
      QString *userDirOut;
      
      QTabWidget *tabWidget;
      FilesTab *filesTabPage;
      DataTab *dataTabPage;

      void generateMomentsDo(int, QString);

      QProgressBar *progressBar;

   public slots:
      void about();
      void loadCPM();
      void saveCPM();
      void generateMoments();
      void generateMomentsPlotOnly(QString);
      void displayMessage(int, QString, int);
      void displayMessageSpectro(int);
      void httpRequestFinished(int, bool);
      void httpRequestStarted(int);
      void downloadHandler(bool);
      void downloadAborted();
      void slotModifiedOutputDir(QString);

   private:
      QMenu *helpMenu;
      QAction *aboutAct;

      QPushButton *generateMomentsAction;
      QPushButton *integrationAction;
      QPushButton *outputAction;
      QPushButton *spectroViewerAction;
      QPushButton *loadCPMAction;
      QPushButton *saveCPMAction;
      QPushButton *quitAction;

      void errorBox(QStringList);

      calculateMomentsThread *calculationThread;
      generateSpectrogramThread *spectrogramThread;

      IntegrationWindow *integrationWindow;
      OutputWindow *outputWindow;
      PlotWindow *plotWindow;
      PlotMoments *momentsWindow;
      PlotSpectrogram *spectrogramWindow;

      QLineEdit *potentialOffset1;
      QLineEdit *potentialOffset2;
      QComboBox *potentialOffset2Units;

      QLineEdit *editFilePeace1;
      QLineEdit *editFilePeace2;
      QLineEdit *editFilePotential;
      QLineEdit *editFileAuxGSE;
      QLineEdit *editFileMagField;
    
      QComboBox *sensorCombiSelect;
      QComboBox *coordSelect;
 
      struct sWML *myWML;
      struct sPML *myPML;
      struct momCalcParams *myParams;

      QGroupBox *potentialGroup;

      void loadCPMonly(QString);
      void loadSettings();

   private slots:
      void slotUpdateStatusBar();
      void slotUpdateStatusBarSpectro();
      void slotUpdateStatusBarProgress(int);
      void displayIntegration();
      void displayOutput();
      void displayPlot();

      void displayMomentsPlotImage(const QImage &);
      void displaySpectrogramPlotImage(const QImage &);
      void makeSpectrogram(QString);

};

#endif /*WINDOW_H_*/
