/*******************************************************************************
 **
 ** $Id: writecef.C 166 2008-11-30 09:42:37Z AndrewLahiff $
 **
 ******************************************************************************
 **
 ** Copyright (C) 2007-2008 Andrew Lahiff.
 **
 ** This file is part of PEACEmoments CAA.
 **
 ** PEACEmoments CAA is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** PEACEmoments CAA is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with PEACEmoments CAA.  If not, see <http://www.gnu.org/licenses/>.
 **
 *******************************************************************************/

#include "peacemoments_caa.h"

//=============================================================================
//
// WriteCef class
//
//=============================================================================

class WriteHeaderMetaVariable
{
   public:
      WriteHeaderMetaVariable(QTextStream *);
      void start(char *);
      void end(char *);
      void param(char *, char *);
      void paramQ(char *, char *);
   private:
      QTextStream *out;
};

WriteHeaderMetaVariable::WriteHeaderMetaVariable(QTextStream *in)
{
   out = in;
}

void WriteHeaderMetaVariable::start(char name[])
{
   *out << "START_VARIABLE = " << name << endl;
}

void WriteHeaderMetaVariable::end(char name[])
{
   *out << "END_VARIABLE = " << name << endl;
   *out << "!" << endl;
}

void WriteHeaderMetaVariable::param(char name[], char str[])
{
   *out << "   " <<  name << " = " << str << endl;
}

void WriteHeaderMetaVariable::paramQ(char name[], char str[])
{
   *out << "   " <<  name << " = \"" << str << "\"" << endl;
}

//-----------------------------------------------------------------------------
//
// constructor
//
//-----------------------------------------------------------------------------

WriteCef::WriteCef(int iCoord)
{  
   spacing = ",";
   
// set FRAME & COORDINATE_SYSTEM strings   
   
   coordinateSystemToStr(iCoord, str_CS, str_F, str_C);
}

//-----------------------------------------------------------------------------
// 
// set filename and open file
//
//-----------------------------------------------------------------------------

int WriteCef::open(QString userFileName)
{  
   file.setFileName(userFileName);
   if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) return(_FAILED);
   out.setDevice(&file);
   return(_SUCCESS);
}

//-----------------------------------------------------------------------------
//
// begin data section
//
//-----------------------------------------------------------------------------

int WriteCef::beginData(void)
{
   out << "!" << endl;
   out << "!------------ BEGINNING OF DATA ------------" << endl;
   out << "!" << endl;
   out << "DATA_UNTIL = \"End_of_Data\"" << endl;
   return(_SUCCESS);
}

//-----------------------------------------------------------------------------
//
// write a single variable to the current data record (double)
//
//-----------------------------------------------------------------------------

int WriteCef::writeDataVariable(int size, double *ptr)
{
   for (int i=0; i<size; i++)
   {
      out << spacing << *(ptr + i);
   }
   return(_SUCCESS);
}

//-----------------------------------------------------------------------------
//
// write a single variable to the current data record (float)
//
//-----------------------------------------------------------------------------

int WriteCef::writeDataVariable(int size, float *ptr)
{
   for (int i=0; i<size; i++)
   {
      out << spacing << *(ptr + i);
   }
   return(_SUCCESS);
}


//-----------------------------------------------------------------------------
//
// write a single variable to the current data record (int)
//
//-----------------------------------------------------------------------------

int WriteCef::writeDataVariable(int size, int *ptr)
{
   for (int i=0; i<size; i++)
   {
      out << spacing << *(ptr + i);
   }
   return(_SUCCESS);
}

//-----------------------------------------------------------------------------
//
// write a single variable to the current data record (QString)
//
//-----------------------------------------------------------------------------

int WriteCef::writeDataVariable(QString ptr)
{
   out << ptr;
   return(_SUCCESS);
}

//-----------------------------------------------------------------------------
//
// end the current data record
//
//-----------------------------------------------------------------------------

int WriteCef::endCurrentDataRecord(void)
{
   out << " $" << endl;
   return(_SUCCESS);
}

//-----------------------------------------------------------------------------
//
// end data section
//
//-----------------------------------------------------------------------------

int WriteCef::endData(void)
{
   out << "End_of_Data" << endl;
   return(_SUCCESS);
}

//-----------------------------------------------------------------------------
//
// close CEF file
//
//-----------------------------------------------------------------------------

void WriteCef::close()
{
   file.close();
}

//-----------------------------------------------------------------------------
//
// write header components
//
//-----------------------------------------------------------------------------

int WriteCef::writeMetaFileUsed(QString name, QString file)
{
   QString fileOnly;
   if (file.contains("/"))
   { 
      QStringList slist = file.split("/");
      fileOnly = slist.at(slist.size()-1);
   }
   else fileOnly = file;

   out << "START_META = " << name << endl;
   out << "   ENTRY = \"" << fileOnly << "\"" << endl;
   out << "END_META = " << name << endl;
   out << "!" << endl;
   return(_SUCCESS);
}

int WriteCef::writeMetaSetupPotential(double offset1, double offset2, int offset2_units, bool usePotential)
{
   if (usePotential)
   {
      out << "START_META = SC_POTENTIAL_CORRECTION" << endl;
      out << "   ENTRY = \"ON\"" << endl;
      out << "END_META = SC_POTENTIAL_CORRECTION" << endl;
      out << "!" << endl;
      out << "START_META = SC_POTENTIAL_OFFSET" << endl;
      out << "   ENTRY =  \"" << offset1 << " eV\"" << endl;
      out << "END_META = SC_POTENTIAL_OFFSET" << endl;
      out << "!" << endl;
      char tmpS[6];
      if (offset2_units == _BINS) sprintf(tmpS, "bins");
      else sprintf(tmpS, "eV");
      out << "START_META = SC_POTENTIAL_LOW_ENERGY_CUTOFF_OFFSET" << endl;
      out << "   ENTRY = \"" << offset2 << " " << tmpS << "\"" << endl;
      out << "END_META = SC_POTENTIAL_LOW_ENERGY_CUTOFF_OFFSET" << endl;
      out << "!" << endl;
   }
   else
   {
      out << "START_META = SC_POTENTIAL_CORRECTION" << endl;
      out << "   ENTRY = \"OFF\"" << endl;
      out << "END_META = SC_POTENTIAL_CORRECTION" << endl;
      out << "!" << endl;
   }
   return(_SUCCESS);
}

int WriteCef::writeMetaSetupRestrInt(int enStatus, int enType, int enScpType, double enLow, double enHigh,
                                     int plStatus, double plLow, double plHigh)
{
   enScpType = 0; // remove compile warning 

   if (enStatus == _OFF)
   {
      out << "START_META = RESTR_ENERGY_INT" << endl;
      out << "   ENTRY = \"OFF\"" << endl;
      out << "END_META = RESTR_ENERGY_INT" << endl;
      out << "!" << endl;      
   }
   else if (enStatus == _ON)
   {
      out << "START_META = RESTR_ENERGY_INT" << endl;
      out << "   ENTRY = \"ON\"" << endl;
      out << "END_META = RESTR_ENERGY_INT" << endl;
      out << "!" << endl;   
      out << "START_META = RESTR_ENERGY_EN_SCP" << endl;
      if (enType == 0)
      {
         out << "   ENTRY = \"NO_POTENTIAL_CORRECTION\"" << endl;     
      }
      else
      {
         out << "   ENTRY = \"POTENTIAL_CORRECTED\"" << endl;      
      }      
      out << "START_META = RESTR_ENERGY_EN_SCP" << endl;
      out << "!" << endl;     
      if (enType == 0)
      {
         out << "   ENTRY = \"NORMAL\"" << endl;     
      }
      else
      {
         out << "   ENTRY = \"INTERPOLATED\"" << endl;      
      }
      out << "END_META = RESTR_ENERGY_TYPE" << endl;
      out << "!" << endl; 
      out << "START_META = RESTR_ENERGY_LOW" << endl;
      out << "   ENTRY =  \"" << enLow << " eV\"" << endl;
      out << "END_META = RESTR_ENERGY_LOW" << endl;
      out << "!" << endl;  
      out << "START_META = RESTR_ENERGY_HIGH" << endl;
      out << "   ENTRY =  \"" << enHigh << " eV\"" << endl;
      out << "END_META = RESTR_ENERGY_HIGH" << endl;
      out << "!" << endl;                  
   }

   int iplLow = (int)plLow;
   int iplHigh = (int)plHigh;

   if (plStatus == _OFF)
   {
      out << "START_META = RESTR_POLAR_INT" << endl;
      out << "   ENTRY = \"OFF\"" << endl;
      out << "END_META = RESTR_POLAR_INT" << endl;
      out << "!" << endl;
   }
   else if (plStatus == _ON)
   {
      out << "START_META = RESTR_POLAR_INT" << endl;
      out << "   ENTRY = \"ON\"" << endl;
      out << "END_META = RESTR_POLAR_INT" << endl;
      out << "!" << endl;
      out << "START_META = RESTR_POLAR_LOW" << endl;
      out << "   ENTRY =  \"" << iplLow << " (bin number)\"" << endl;
      out << "END_META = RESTR_POLAR_LOW" << endl;
      out << "!" << endl;
      out << "START_META = RESTR_POLAR_HIGH" << endl;
      out << "   ENTRY =  \"" << iplHigh << " (bin number)\"" << endl;
      out << "END_META = RESTR_POLAR_HIGH" << endl;
      out << "!" << endl;
   }
   return(_SUCCESS);
}

int WriteCef::writeMetaVersionNumber(int iVersionNumber)
{
   char str[5];
   sprintf(str, "%02d", iVersionNumber);
   out << "START_META = VERSION_NUMBER" << endl;
   out << "   ENTRY =  " << str << endl;
   out << "END_META = VERSION_NUMBER" << endl;
   out << "!" << endl;
   return(_SUCCESS);
}

int WriteCef::writeMetaDatasetVersion(void)
{
   out << "START_META = DATASET_VERSION" << endl;
   out << "   ENTRY = \"Generated by PEACEmoments CAA " << VERSION_PM << "\"" << endl;
   out << "END_META = DATASET_VERSION" << endl;
   out << "!" << endl;
   return(_SUCCESS);
}

int WriteCef::writeMetaFileTimeSpan(QString qStart, QString qEnd)
{
   out << "START_META = FILE_TIME_SPAN" << endl;
   out << "   VALUE_TYPE = ISO_TIME_RANGE" << endl;
   out << "   ENTRY = " << qStart << "/" << qEnd << endl;
   out << "END_META = FILE_TIME_SPAN" << endl;
   out << "!" << endl;
   return(_SUCCESS);
}

int WriteCef::writeMetaLogicalFileId(QString fileId)
{
   fileId.remove(".cef");
   out << "START_META = LOGICAL_FILE_ID" << endl;
   out << "   ENTRY = \"" << fileId << "\"" << endl;
   out << "END_META = LOGICAL_FILE_ID" << endl;
   out << "!" << endl;
   return(_SUCCESS);
}

int WriteCef::writeFileName(QString file)
{
   out << "FILE_NAME = \"" << file << "\"" << endl;
   out << "!" << endl;
   return(_SUCCESS);
}

int WriteCef::writeObservatory(QString tmpStr)
{
   out << "START_META = OBSERVATORY" << endl;
   out << "   ENTRY = \"" << tmpStr << "\"" << endl;
   out << "END_META = OBSERVATORY" << endl;    
   out << "!" << endl;   
   return(_SUCCESS);
}

int WriteCef::writeExperiment()
{
   out << "START_META = EXPERIMENT" << endl;
   out << "   ENTRY = \"PEACE\"" << endl;
   out << "END_META = EXPERIMENT" << endl;   
   out << "!" << endl;   
   return(_SUCCESS);
}

int WriteCef:: writeInitial(void)
{
   out << "!---------------------------------------------------------------|" << endl;
   out << "!                                                               |" << endl;
   out << "! PEACEmoments CAA : electron moments from PEACE CAA data files |" << endl;
   out << "!                                                               |" << endl;
   out << "!---------------------------------------------------------------|" << endl;
   out << "!   ~ this file does not contain official CAA PEACE moments ~   |" << endl;
   out << "!---------------------------------------------------------------|" << endl;
   out << "!" << endl;
   return(_SUCCESS);
}

int WriteCef::writeMetaStatic(void)
{
   out << "FILE_FORMAT_VERSION = \"CEF-2.0\"" << endl;
   out << "END_OF_RECORD_MARKER = \"$\"" << endl;
   out << "!" << endl;
   out << "!-------- Cluster Exchange Version 2 ASCII File --------|" << endl;
   out << "! Free width data entries separated by delimiters       |" << endl;
   out << "!                                                       |" << endl;
   out << "! ASCII Format                                          |" << endl;
   out << "! Native C ordering,                                    |" << endl;
   out << "! Blank lines are ignored                               |" << endl;
   out << "! \"!\" escapes rest of line as comment                   |" << endl;
   out << "!-------------------------------------------------------|" << endl;
   out << "!" << endl;
   out << "START_META = FILE_TYPE" << endl;
   out << "   ENTRY   = \"cef\"" << endl;
   out << "END_META   = FILE_TYPE" << endl;
   out << "!" << endl;
   out << "START_META = METADATA_TYPE" << endl;
   out << "   ENTRY   = \"CAA\"" << endl;
   out << "END_META   = METADATA_TYPE" << endl;
   out << "!" << endl;
   out << "START_META = METADATA_VERSION" << endl;
   out << "   ENTRY   = \"2.0\"" << endl;
   out << "END_META   = METADATA_VERSION" << endl;
   out << "!" << endl;
   return(_SUCCESS);
}

int WriteCef::writeGenerationDate(void)
{
   char TimeDateStr[50];
   time_t tim;
   time (&tim);
   tm *now=localtime(&tim);

   sprintf(TimeDateStr, "%d-%02d-%02dT%02d:%02d:%02d.000Z", now->tm_year+1900,
           now->tm_mon+1, now->tm_mday, now->tm_hour, now->tm_min, now->tm_sec);

   out << "START_META = GENERATION_DATE" << endl;
   out << "   VALUE_TYPE = ISO_TIME" << endl;
   out << "   ENTRY = " << TimeDateStr << endl;
   out << "END_META = GENERATION_DATE" << endl;
   out << "!" << endl;
   return(_SUCCESS);
}

int WriteCef::writeUsefulInfoHeader(void)
{
   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Mode_Sensor");
   metaVar.paramQ("PARAMETER_TYPE", "Support_Data");
   metaVar.paramQ("CATDESC", "PEACE Sensor from which Data is collected (0 = HEEA, 1 = LEEA, 2 = Both)");
   metaVar.paramQ("UNITS", "Unitless");
   metaVar.paramQ("SI_CONVERSION", "1>unitless");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("VALUE_TYPE", "INT");
   metaVar.param("SIGNIFICANT_DIGITS", "1");
   metaVar.param("FILLVAL", "-1");
   metaVar.param("QUALITY", "0");
   metaVar.paramQ("FIELDNAM", "Sensor (0=HEEA,1=LEEA,2=Both)");
   metaVar.paramQ("LABLAXIS", "Sensor");
   metaVar.param("SCALEMIN", "0");
   metaVar.param("SCALEMAX", "2");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.end("Mode_Sensor");

   metaVar.start("Mode_SweepMode_HEEA");
   metaVar.paramQ("PARAMETER_TYPE", "Support_Data");
   metaVar.paramQ("CATDESC", "HEEA Sweep Mode (0 = Off/Fixed, 1 = LAR, 2 = HAR, 3 = MAR)");
   metaVar.paramQ("UNITS", "Unitless");
   metaVar.paramQ("SI_CONVERSION", "1>unitless");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("VALUE_TYPE", "INT");
   metaVar.param("SIGNIFICANT_DIGITS", "1");
   metaVar.param("FILLVAL", "-1");
   metaVar.param("QUALITY", "0");
   metaVar.paramQ("FIELDNAM", "Sweep Mode HEEA");
   metaVar.paramQ("LABLAXIS", "Sweep Mode HEEA");
   metaVar.param("SCALEMIN", "0");
   metaVar.param("SCALEMAX", "2");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.end("Mode_SweepMode_HEEA");

   metaVar.start("Mode_SweepMode_LEEA");
   metaVar.paramQ("PARAMETER_TYPE", "Support_Data");
   metaVar.paramQ("CATDESC", "LEEA Sweep Mode (0 = Off/Fixed, 1 = LAR, 2 = HAR, 3 = MAR)");
   metaVar.paramQ("UNITS", "Unitless");
   metaVar.paramQ("SI_CONVERSION", "1>unitless");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("VALUE_TYPE", "INT");
   metaVar.param("SIGNIFICANT_DIGITS", "1");
   metaVar.param("FILLVAL", "-1");
   metaVar.param("QUALITY", "0");
   metaVar.paramQ("FIELDNAM", "Sweep Mode LEEA");
   metaVar.paramQ("LABLAXIS", "Sweep Mode LEEA");
   metaVar.param("SCALEMIN", "0");
   metaVar.param("SCALEMAX", "2");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.end("Mode_SweepMode_LEEA");

   metaVar.start("Mode_TopRegionSensor");
   metaVar.paramQ("PARAMETER_TYPE", "Support_Data");
   metaVar.paramQ("CATDESC", "Sensor used for Top Energy Region (0 = HEEA, 1 = LEEA, 2 = average)");
   metaVar.paramQ("UNITS", "Unitless");
   metaVar.paramQ("SI_CONVERSION", "1>unitless");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("VALUE_TYPE", "INT");
   metaVar.param("SIGNIFICANT_DIGITS", "1");
   metaVar.param("FILLVAL", "-1");
   metaVar.param("QUALITY", "0");
   metaVar.paramQ("FIELDNAM", "Sensor (top region)");
   metaVar.paramQ("LABLAXIS", "Sensor (top region)");
   metaVar.param("SCALEMIN", "0");
   metaVar.param("SCALEMAX", "2");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.end("Mode_TopRegionSensor");

   metaVar.start("Mode_OverlapRegionSensor");
   metaVar.paramQ("PARAMETER_TYPE", "Support_Data");
   metaVar.paramQ("CATDESC", "Sensor used for Overlap Energy Region (0 = HEEA, 1 = LEEA, 2 = average)");
   metaVar.paramQ("UNITS", "Unitless");
   metaVar.paramQ("SI_CONVERSION", "1>unitless");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("VALUE_TYPE", "INT");
   metaVar.param("SIGNIFICANT_DIGITS", "1");
   metaVar.param("FILLVAL", "-1");
   metaVar.param("QUALITY", "0");
   metaVar.paramQ("FIELDNAM", "Sensor (overlap region)");
   metaVar.paramQ("LABLAXIS", "Sensor (overlap region)");
   metaVar.param("SCALEMIN", "0");
   metaVar.param("SCALEMAX", "2");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.end("Mode_OverlapRegionSensor");

   metaVar.start("Mode_BottomRegionSensor");
   metaVar.paramQ("PARAMETER_TYPE", "Support_Data");
   metaVar.paramQ("CATDESC", "Sensor used for Bottom Energy Region (0 = HEEA, 1 = LEEA, 2 = average)");
   metaVar.paramQ("UNITS", "Unitless");
   metaVar.paramQ("SI_CONVERSION", "1>unitless");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("VALUE_TYPE", "INT");
   metaVar.param("SIGNIFICANT_DIGITS", "1");
   metaVar.param("FILLVAL", "-1");
   metaVar.param("QUALITY", "0");
   metaVar.paramQ("FIELDNAM", "Sensor (bottom region)");
   metaVar.paramQ("LABLAXIS", "Sensor (bottom region)");
   metaVar.param("SCALEMIN", "0");
   metaVar.param("SCALEMAX", "2");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.end("Mode_BottomRegionSensor");

   return(_SUCCESS);
}

//-----------------------------------------------------------------------------
//
// meta data for all the possible variables
//
//-----------------------------------------------------------------------------

int WriteCef::writeTimeTagsMeta()
{
   WriteHeaderMetaVariable metaVar1(&out);
   metaVar1.start("time_tags");
   metaVar1.paramQ("PARAMETER_TYPE", "Support_Data");
   metaVar1.paramQ("CATDESC", "Epoch Time of PEACE data record (for one PEACE spin)");
   metaVar1.paramQ("UNITS", "s");
   metaVar1.paramQ("SI_CONVERSION", "1.0>s");
   metaVar1.param("SIZES", "1");
   metaVar1.paramQ("DELTA_PLUS", "time_tags_DeltaUpper");
   metaVar1.paramQ("DELTA_MINUS", "time_tags_DeltaLower");
   metaVar1.param("VALUE_TYPE", "ISO_TIME");
   metaVar1.param("SIGNIFICANT_DIGITS", "24");
   metaVar1.param("FILLVAL", "9999-12-31T23:59:59.999Z");
   metaVar1.paramQ("LABLAXIS", "Data Epoch");
   metaVar1.end("time_tags");

   WriteHeaderMetaVariable metaVar2(&out);
   metaVar2.start("time_tags_DeltaLower");
   metaVar2.paramQ("PARAMETER_TYPE", "Support_Data");
   metaVar2.paramQ("CATDESC", "Actual Lower Epoch Edge = time_tags - time_tags_DeltaLower");
   metaVar2.paramQ("UNITS", "s");
   metaVar2.paramQ("SI_CONVERSION", "1.0>s");
   metaVar2.param("SIZES", "1");
   metaVar2.param("VALUE_TYPE", "FLOAT");
   metaVar2.param("SIGNIFICANT_DIGITS", "4");
   metaVar2.param("FILLVAL", "-1");
   metaVar2.paramQ("LABLAXIS", "Data Epoch Lower Delta");
   metaVar2.end("time_tags_DeltaLower");

   WriteHeaderMetaVariable metaVar3(&out);
   metaVar3.start("time_tags_DeltaUpper");
   metaVar3.paramQ("PARAMETER_TYPE", "Support_Data");
   metaVar3.paramQ("CATDESC", "Actual Upper Epoch Edge = time_tags + time_tags_DeltaLower");
   metaVar3.paramQ("UNITS", "s");
   metaVar3.paramQ("SI_CONVERSION", "1.0>s");
   metaVar3.param("SIZES", "1");
   metaVar3.param("VALUE_TYPE", "FLOAT");
   metaVar3.param("SIGNIFICANT_DIGITS", "4");
   metaVar3.param("FILLVAL", "-1");
   metaVar3.paramQ("LABLAXIS", "Data Epoch Upper Delta");
   metaVar3.end("time_tags_DeltaUpper");
				    
   return(_SUCCESS);
}

int WriteCef::writeDensityMeta()
{
   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_Density");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");
   metaVar.paramQ("PROPERTY", "Number_Density");
   metaVar.paramQ("CATDESC", "Electron Density");
   metaVar.paramQ("UNITS", "cm^-3");
   metaVar.paramQ("SI_CONVERSION", "1.0E6>(particles)m^-3");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("SIGNIFICANT_DIGITS", "8");
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.param("SCALEMIN", "1E0");
   metaVar.param("SCALEMAX", "1E2");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "Ne");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": Ne").toStdString().c_str());
   metaVar.end("Data_Density");
   
   return(_SUCCESS);
}

int WriteCef::writeVelocityMeta()
{
   char tmp[40];
   sprintf(tmp, "%s Electron Velocity", str_C);
   
   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_Velocity");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");
   metaVar.paramQ("PROPERTY", "Velocity");
   metaVar.paramQ("CATDESC", tmp);
   metaVar.paramQ("UNITS", "km s^-1");
   metaVar.paramQ("SI_CONVERSION", "1.0E3>m s^-1");
   metaVar.paramQ("TENSOR_ORDER", "1");
   metaVar.paramQ("COORDINATE_SYSTEM", str_CS);
   metaVar.paramQ("FRAME", str_F);
   metaVar.param("REPRESENTATION_1", "\"x\", \"y\", \"z\"");
   metaVar.param("SIZES", "3");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("LABEL_1", "\"Vx\", \"Vy\", \"Vz\"");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("SIGNIFICANT_DIGITS", "8");   
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.param("SCALEMIN", "-6E2");
   metaVar.param("SCALEMAX", "6E2");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "Ve");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": Ve").toStdString().c_str());
   metaVar.end("Data_Velocity");
   
   return(_SUCCESS);
}

int WriteCef::writeVelocityParaMeta()
{
   char tmp[40];
   sprintf(tmp, "%s Electron Velocity (para)", str_C);

   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_Velocity_ComponentParallelToMagField");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");
   metaVar.paramQ("PROPERTY", "Velocity");
   metaVar.paramQ("CATDESC", tmp);
   metaVar.paramQ("UNITS", "km s^-1");
   metaVar.paramQ("SI_CONVERSION", "1.0E3>m s^-1");
   metaVar.paramQ("TENSOR_ORDER", "1");
   metaVar.paramQ("COORDINATE_SYSTEM", str_CS);
   metaVar.paramQ("FRAME", str_F);
   metaVar.param("REPRESENTATION_1", "\"x\", \"y\", \"z\"");
   metaVar.param("SIZES", "3");   
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("LABEL_1", "\"Vx\", \"Vy\", \"Vz\"");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("SIGNIFICANT_DIGITS", "8");  
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.param("SCALEMIN", "-6E2");
   metaVar.param("SCALEMAX", "6E2");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "Ve (para)");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": Ve (para)").toStdString().c_str());
   metaVar.end("Data_Velocity_ComponentParallelToMagField");
   
   return(_SUCCESS);
}

int WriteCef::writeVelocityPerpMeta()
{
   char tmp[50];
   sprintf(tmp, "%s Electron Velocity (perp)", str_C);

   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_Velocity_ComponentPerpendicularToMagField");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");
   metaVar.paramQ("PROPERTY", "Velocity");
   metaVar.paramQ("CATDESC", tmp);
   metaVar.paramQ("UNITS", "km s^-1");
   metaVar.paramQ("SI_CONVERSION", "1.0E3>m s^-1");
   metaVar.paramQ("TENSOR_ORDER", "1");
   metaVar.paramQ("COORDINATE_SYSTEM", str_CS);
   metaVar.paramQ("FRAME", str_F);
   metaVar.param("REPRESENTATION_1", "\"x\", \"y\", \"z\"");
   metaVar.param("SIZES", "3");   
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("LABEL_1", "\"Vx\", \"Vy\", \"Vz\"");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("SIGNIFICANT_DIGITS", "8");   
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.param("SCALEMIN", "-6E2");
   metaVar.param("SCALEMAX", "6E2");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "Ve (perp)");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": Ve (perp)").toStdString().c_str());
   metaVar.end("Data_Velocity_ComponentPerpendicularToMagField");
   
   return(_SUCCESS);
}

int WriteCef::writeTempMeta()
{
   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_Temperature");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");
   metaVar.paramQ("PROPERTY", "Temperature");
   metaVar.paramQ("CATDESC", "Electron Temperature");
   metaVar.paramQ("UNITS", "MK");
   metaVar.paramQ("SI_CONVERSION", "1.0E6>K");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("SIGNIFICANT_DIGITS", "8");   
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.paramQ("FIELDNAM", "Temperature");
   metaVar.param("SCALEMIN", "0");
   metaVar.param("SCALEMAX", "5");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "Te");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": Te").toStdString().c_str());
   metaVar.end("Data_Temperature");
   
   return(_SUCCESS);
}

int WriteCef::writeTempParaMeta()
{
   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_Temperature_ComponentParallelToMagField");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");
   metaVar.paramQ("PROPERTY", "Temperature");
   metaVar.paramQ("CATDESC", "Electron Temperature (para)");
   metaVar.paramQ("UNITS", "MK");
   metaVar.paramQ("SI_CONVERSION", "1.0E6>K");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.paramQ("FIELDNAM", "Para Temperature");
   metaVar.param("SCALEMIN", "0");
   metaVar.param("SCALEMAX", "5");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "Te (para)");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": Te (para)").toStdString().c_str());
   metaVar.end("Data_Temperature_ComponentParallelToMagField");
   
   return(_SUCCESS);
}

int WriteCef::writeTempPerpMeta()
{
   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_Temperature_ComponentPerpendicularToMagField");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");
   metaVar.paramQ("PROPERTY", "Temperature");
   metaVar.paramQ("CATDESC", "Electron Temperature (perp)");
   metaVar.paramQ("UNITS", "MK");
   metaVar.paramQ("SI_CONVERSION", "1.0E6>K");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.paramQ("FIELDNAM", "Perp Temperature");
   metaVar.param("SCALEMIN", "0");
   metaVar.param("SCALEMAX", "5");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "Te (perp)");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": Te (perp)").toStdString().c_str());
   metaVar.end("Data_Temperature_ComponentPerpendicularToMagField");
   
   return(_SUCCESS);
}

int WriteCef::writeHeatFluxMeta()
{
   char tmp[50];
   sprintf(tmp, "%s Electron Heat Flux", str_C);

   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_HeatFlux");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");   
   metaVar.paramQ("PROPERTY", "Heat_Flux");
   metaVar.paramQ("CATDESC", tmp);
   metaVar.paramQ("UNITS", "microW m^-2");
   metaVar.paramQ("SI_CONVERSION", "1.0E-6>W m^-2");
   metaVar.param("TENSOR_ORDER", "1");
   metaVar.paramQ("COORDINATE_SYSTEM", str_CS);
   metaVar.paramQ("FRAME", str_F);
   metaVar.param("REPRESENTATION_1", "\"x\", \"y\", \"z\"");
   metaVar.param("SIZES", "3");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("LABEL_1", "\"Hx\", \"Hy\", \"Hz\"");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("SIGNIFICANT_DIGITS", "8");
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");   
   metaVar.paramQ("FIELDNAM", "GSE Heat Flux");
   metaVar.param("SCALEMIN", "-6E2");
   metaVar.param("SCALEMAX", "6E2");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "He");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": He").toStdString().c_str());
   metaVar.end("Data_HeatFlux");
   
   return(_SUCCESS);
}

int WriteCef::writeHeatFluxParaMeta()
{
   char tmp[50];
   sprintf(tmp, "%s Electron Heat Flux (para)", str_C);
   
   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_HeatFlux_ComponentParallelToMagField");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");   
   metaVar.paramQ("PROPERTY", "Heat_Flux");
   metaVar.paramQ("CATDESC", tmp);
   metaVar.paramQ("UNITS", "microW m^-2");
   metaVar.paramQ("SI_CONVERSION", "1.0E-6>W m^-2");
   metaVar.paramQ("TENSOR_ORDER", "1");
   metaVar.paramQ("COORDINATE_SYSTEM", str_CS);
   metaVar.paramQ("FRAME", str_F);
   metaVar.param("REPRESENTATION_1", "\"x\", \"y\", \"z\"");
   metaVar.param("SIZES", "3");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("LABEL_1", "\"Hx\", \"Hy\", \"Hz\"");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("SIGNIFICANT_DIGITS", "8");
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.paramQ("FIELDNAM", "GSE Heat Flux (para)");
   metaVar.param("SCALEMIN", "-6E2");
   metaVar.param("SCALEMAX", "6E2");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "He (para)");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": He (para)").toStdString().c_str());
   metaVar.end("Data_HeatFlux_ComponentParallelToMagField");
   
   return(_SUCCESS);
}

int WriteCef::writeHeatFluxPerpMeta()
{
   char tmp[50];
   sprintf(tmp, "%s Electron Heat Flux (perp)", str_C);
   
   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_HeatFlux_ComponentPerpendicularToMagField");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");
   metaVar.paramQ("PROPERTY", "Heat_Flux");
   metaVar.paramQ("CATDESC", tmp);
   metaVar.paramQ("UNITS", "microW m^-2");
   metaVar.paramQ("SI_CONVERSION", "1.0E-6>W m^-2");
   metaVar.paramQ("TENSOR_ORDER", "1");
   metaVar.paramQ("COORDINATE_SYSTEM", str_CS);
   metaVar.paramQ("FRAME", str_F);
   metaVar.param("REPRESENTATION_1", "\"x\", \"y\", \"z\"");
   metaVar.param("SIZES", "3");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("LABEL_1", "\"Hx\", \"Hy\", \"Hz\"");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("SIGNIFICANT_DIGITS", "8");
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.paramQ("FIELDNAM", "GSE Heat Flux (perp)");
   metaVar.param("SCALEMIN", "-6E2");
   metaVar.param("SCALEMAX", "6E2");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "He (perp)");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": He (perp)").toStdString().c_str());
   metaVar.end("Data_HeatFlux_ComponentPerpendicularToMagField");  
   
   return(_SUCCESS);
}

int WriteCef::writeEnergyFluxMeta()
{
   char tmp[50];
   sprintf(tmp, "%s Electron Energy Flux", str_C);
   
   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_EnergyFlux"); 
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");
   metaVar.paramQ("PROPERTY", "Particle_Energy_Flux");
   metaVar.paramQ("CATDESC", tmp);
   metaVar.paramQ("UNITS", "microW m^-2");           // incorrect
   metaVar.paramQ("SI_CONVERSION", "1.0E-6>W m^-2"); // incorrect
   metaVar.paramQ("TENSOR_ORDER", "1");
   metaVar.paramQ("COORDINATE_SYSTEM", str_CS);
   metaVar.paramQ("FRAME", str_F);
   metaVar.param("REPRESENTATION_1", "\"x\", \"y\", \"z\"");
   metaVar.param("SIZES", "3");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("LABEL_1", "\"Qx\", \"Qy\", \"Qz\"");
   metaVar.param("SIGNIFICANT_DIGITS", "8");
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.paramQ("FIELDNAM", "GSE Energy Flux");
   metaVar.param("SCALEMIN", "-6E2");
   metaVar.param("SCALEMAX", "6E2");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "Qe");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": Qe").toStdString().c_str());
   metaVar.end("Data_EnergyFlux"); 
   
   return(_SUCCESS);
}

int WriteCef::writeEnergyFluxParaMeta()
{
   char tmp[50];
   sprintf(tmp, "%s Electron Energy Flux (para)", str_C);
   
   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_EnergyFlux_ComponentParallelToMagField");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");
   metaVar.paramQ("PROPERTY", "Particle_Energy_Flux"); 
   metaVar.paramQ("CATDESC", tmp);
   metaVar.paramQ("UNITS", "microW m^-2");           // incorrect
   metaVar.paramQ("SI_CONVERSION", "1.0E-6>W m^-2"); // incorrect
   metaVar.paramQ("TENSOR_ORDER", "1");
   metaVar.paramQ("COORDINATE_SYSTEM", str_CS);
   metaVar.paramQ("FRAME", str_F);
   metaVar.param("REPRESENTATION_1", "\"x\", \"y\", \"z\"");
   metaVar.param("SIZES", "3");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("LABEL_1", "\"Qx\", \"Qy\", \"Qz\"");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("SIGNIFICANT_DIGITS", "8");
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.paramQ("FIELDNAM", "GSE Energy Flux (para)");
   metaVar.param("SCALEMIN", "-6E2");
   metaVar.param("SCALEMAX", "6E2");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "Qe (para)");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": Qe (para)").toStdString().c_str());
   metaVar.end("Data_EnergyFlux_ComponentParallelToMagField"); 
   
   return(_SUCCESS);
}

int WriteCef::writeEnergyFluxPerpMeta()
{
   char tmp[50];
   sprintf(tmp, "%s Electron Energy Flux (perp)", str_C);
   
   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_EnergyFlux_ComponentPerpendicularToMagField"); 
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");
   metaVar.paramQ("PROPERTY", "Particle_Energy_Flux");
   metaVar.paramQ("CATDESC", tmp);
   metaVar.paramQ("UNITS", "microW m^-2");           // incorrect
   metaVar.paramQ("SI_CONVERSION", "1.0E-6>W m^-2"); // incorrect
   metaVar.paramQ("TENSOR_ORDER", "1");
   metaVar.paramQ("COORDINATE_SYSTEM", str_CS);
   metaVar.paramQ("FRAME", str_F);
   metaVar.param("REPRESENTATION_1", "\"x\", \"y\", \"z\"");
   metaVar.param("SIZES", "3");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("LABEL_1", "\"Qx\", \"Qy\", \"Qz\"");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("SIGNIFICANT_DIGITS", "8");
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.paramQ("FIELDNAM", "GSE Energy Flux (perp)");
   metaVar.param("SCALEMIN", "-6E2");
   metaVar.param("SCALEMAX", "6E2");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "Qe (perp)");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": Qe (perp)").toStdString().c_str());
   metaVar.end("Data_EnergyFlux_ComponentPerpendicularToMagField");   
   
   return(_SUCCESS);
}

int WriteCef::writePressureMeta()
{
   char tmp[50];
   sprintf(tmp, "%s Electron Pressure Tensor", str_C);
   
   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_Pressure");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");  
   metaVar.paramQ("PROPERTY", "Pressure_Tensor");
   metaVar.paramQ("CATDESC", tmp);
   metaVar.paramQ("UNITS", "nPa");
   metaVar.paramQ("SI_CONVERSION", "1.0E-9>Pa");
   metaVar.param("TENSOR_ORDER", "2");
   metaVar.paramQ("COORDINATE_SYSTEM", str_CS);
   metaVar.paramQ("FRAME", str_F);
   metaVar.param("REPRESENTATION_1", "\"x\", \"y\", \"z\"");
   metaVar.param("SIZES", "3,3");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("LABEL_1", "\"x\", \"y\", \"z\"");
   metaVar.param("LABEL_2", "\"x\", \"y\", \"z\"");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("SIGNIFICANT_DIGITS", "8");
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");    
   metaVar.paramQ("FIELDNAM", "GSE Pressure");
   metaVar.param("SCALEMIN", "-6E2");
   metaVar.param("SCALEMAX", "6E2");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "Pe");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": Pe").toStdString().c_str());
   metaVar.end("Data_Pressure");     
   
   return(_SUCCESS);
}

int WriteCef::writeMagFieldMeta()
{  
   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_MagField");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Magnetic Field");
   metaVar.paramQ("PROPERTY", "Vector");
   metaVar.paramQ("CATDESC", "GSE Magnetic Field Vector");
   metaVar.paramQ("UNITS", "nT");
   metaVar.paramQ("SI_CONVERSION", "1.0E-9>T");
   metaVar.paramQ("TENSOR_ORDER", "1");
   metaVar.paramQ("COORDINATE_SYSTEM", "GSE>Geocentric Solar Ecliptic");
   metaVar.paramQ("FRAME", "vector>gse_xyz");
   metaVar.param("REPRESENTATION_1", "\"x\", \"y\", \"z\"");
   metaVar.param("SIZES", "3");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("LABEL_1", "\"Vx\", \"Vy\", \"Vz\"");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("SIGNIFICANT_DIGITS", "8");
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.param("SCALEMIN", "-1E2");
   metaVar.param("SCALEMAX", "1E2");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "B");
   metaVar.paramQ("FIELDNAM", "Magnetic Field");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": B").toStdString().c_str());
   metaVar.end("Data_MagField");
   
   return(_SUCCESS);
} 

int WriteCef::writePotentialMeta()
{   
   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_Potential");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Instrument");
   metaVar.paramQ("PROPERTY", "Probe_Potential");
   metaVar.paramQ("CATDESC", "Spin-averaged Spacecraft potential");
   metaVar.paramQ("UNITS", "eV");
   metaVar.paramQ("SI_CONVERSION", "1>eV");
   metaVar.param("SIZES", "1");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("SIGNIFICANT_DIGITS", "8");
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.param("SCALEMIN", "0");
   metaVar.param("SCALEMAX", "80");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "Sc pot");
   metaVar.paramQ("FIELDNAM", "Spacecraft Potential (spin-averaged)");
   metaVar.end("Data_Potential");

   return(_SUCCESS);
}

int WriteCef::writeCurrentMeta()
{
   char tmp[50];
   sprintf(tmp, "%s Electron Current", str_C);

   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_Current");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");
   metaVar.paramQ("PROPERTY", "Current");
   metaVar.paramQ("CATDESC", tmp);
   metaVar.paramQ("UNITS", "A/m^2");
   metaVar.paramQ("SI_CONVERSION", "1.0>A m^-2");
   metaVar.paramQ("TENSOR_ORDER", "1");
   metaVar.paramQ("COORDINATE_SYSTEM", str_CS);
   metaVar.paramQ("FRAME", str_F);
   metaVar.param("REPRESENTATION_1", "\"x\", \"y\", \"z\"");
   metaVar.param("SIZES", "3");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("LABEL_1", "\"Jx\", \"Jy\", \"Jz\"");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("SIGNIFICANT_DIGITS", "8");   
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "Je");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": Je").toStdString().c_str());
   metaVar.end("Data_Current");
	
   return(_SUCCESS);
}

int WriteCef::writeCurrentParaMeta()
{
   char tmp[50];
   sprintf(tmp, "%s Electron Current (para)", str_C);

   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_Current_ComponentParallelToMagField");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");
   metaVar.paramQ("PROPERTY", "Current");
   metaVar.paramQ("CATDESC", tmp);
   metaVar.paramQ("UNITS", "A/m^2");
   metaVar.paramQ("SI_CONVERSION", "1.0>A m^-2");
   metaVar.paramQ("TENSOR_ORDER", "1");
   metaVar.paramQ("COORDINATE_SYSTEM", str_CS);
   metaVar.paramQ("FRAME", str_F);
   metaVar.param("REPRESENTATION_1", "\"x\", \"y\", \"z\"");
   metaVar.param("SIZES", "3");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("LABEL_1", "\"Jx\", \"Jy\", \"Jz\"");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("SIGNIFICANT_DIGITS", "8");   
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "Je (para)");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": Je (para)").toStdString().c_str());
   metaVar.end("Data_Current_ComponentParallelToMagField");
   
   return(_SUCCESS);
}

int WriteCef::writeCurrentPerpMeta()
{
   char tmp[50];
   sprintf(tmp, "%s Electron Current (perp)", str_C);

   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_Current_ComponentPerpendicularToMagField");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");
   metaVar.paramQ("PROPERTY", "Current");
   metaVar.paramQ("CATDESC", tmp);
   metaVar.paramQ("UNITS", "A/m^2");
   metaVar.paramQ("SI_CONVERSION", "1.0>A m^-2");
   metaVar.paramQ("TENSOR_ORDER", "1");
   metaVar.paramQ("COORDINATE_SYSTEM", str_CS);
   metaVar.paramQ("FRAME", str_F);
   metaVar.param("REPRESENTATION_1", "\"x\", \"y\", \"z\"");
   metaVar.param("SIZES", "3");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("LABEL_1", "\"Jx\", \"Jy\", \"Jz\"");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("SIGNIFICANT_DIGITS", "8");   
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "Je (perp)");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": Je (perp)").toStdString().c_str());
   metaVar.end("Data_Current_ComponentPerpendicularToMagField");
   
   return(_SUCCESS);
}

int WriteCef::writeCurrentMagMeta()
{
   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_Current_Magnitude");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");
   metaVar.paramQ("PROPERTY", "Current");
   metaVar.paramQ("CATDESC", "Electron Current");
   metaVar.paramQ("UNITS", "A/m^2");
   metaVar.paramQ("SI_CONVERSION", "1.0>A m^-2");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("SIGNIFICANT_DIGITS", "8");
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "Je");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": Je").toStdString().c_str());
   metaVar.end("Data_Current_Magnitude");

   return(_SUCCESS);
}

int WriteCef::writeCurrentParaMagMeta()
{
   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_Current_Para_Magnitude");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");
   metaVar.paramQ("PROPERTY", "Current");
   metaVar.paramQ("CATDESC", "Electron Current (para)");
   metaVar.paramQ("UNITS", "A/m^2");
   metaVar.paramQ("SI_CONVERSION", "1.0>A m^-2");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("SIGNIFICANT_DIGITS", "8");
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "Je (para)");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": Je (para)").toStdString().c_str());
   metaVar.end("Data_Current_Para_Magnitude");
   
   return(_SUCCESS);
}

int WriteCef::writeCurrentPerpMagMeta()
{
   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_Current_Perp_Magnitude");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");
   metaVar.paramQ("PROPERTY", "Current");
   metaVar.paramQ("CATDESC", "Electron Current (perp)");
   metaVar.paramQ("UNITS", "A/m^2");
   metaVar.paramQ("SI_CONVERSION", "1.0>A m^-2");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("SIGNIFICANT_DIGITS", "8");
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "Je (perp)");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": Je (perp)").toStdString().c_str());
   metaVar.end("Data_Current_Perp_Magnitude");
   
   return(_SUCCESS);
}

int WriteCef::writeVelocityMagMeta()
{
   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_Velocity_Magnitude");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");
   metaVar.paramQ("PROPERTY", "Velocity");
   metaVar.paramQ("CATDESC", "Electron Velocity");
   metaVar.paramQ("UNITS", "km s^-1");
   metaVar.paramQ("SI_CONVERSION", "1.0E3>m s^-1");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("SIGNIFICANT_DIGITS", "8");
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "Ve");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": Ve").toStdString().c_str());
   metaVar.end("Data_Velocity_Magnitude");
   
   return(_SUCCESS);
}

int WriteCef::writeVelocityParaMagMeta()
{
   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_Velocity_Para_Magnitude");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");
   metaVar.paramQ("PROPERTY", "Velocity");
   metaVar.paramQ("CATDESC", "Electron Velocity (para)");
   metaVar.paramQ("UNITS", "km s^-1");
   metaVar.paramQ("SI_CONVERSION", "1.0E3>m s^-1");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("SIGNIFICANT_DIGITS", "8");
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "Ve (para)");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": Ve (para)").toStdString().c_str());
   metaVar.end("Data_Velocity_Para_Magnitude");
   
   return(_SUCCESS);
}

int WriteCef::writeVelocityPerpMagMeta()
{
   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_Velocity_Perp_Magnitude");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");
   metaVar.paramQ("PROPERTY", "Velocity");
   metaVar.paramQ("CATDESC", "Electron Velocity (perp)");
   metaVar.paramQ("UNITS", "km s^-1");
   metaVar.paramQ("SI_CONVERSION", "1.0E3>m s^-1");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("SIGNIFICANT_DIGITS", "8");
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "Ve (perp)");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": Ve (perp)").toStdString().c_str());
   metaVar.end("Data_Velocity_Perp_Magnitude");
   
   return(_SUCCESS);
}

int WriteCef::writeCharacteristicEnergyMeta()
{
   WriteHeaderMetaVariable metaVar(&out);
   metaVar.start("Data_CharacteristicEnergy");
   metaVar.paramQ("PARAMETER_TYPE", "Data");
   metaVar.paramQ("ENTITY", "Electron");
   metaVar.paramQ("PROPERTY", "Number_Density");
   metaVar.paramQ("CATDESC", "Electron Characteristic Energy");
   metaVar.paramQ("UNITS", "eV");
   metaVar.paramQ("SI_CONVERSION", "1.0E6>(particles)m^-3");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("DEPEND_0", "time_tags");
   metaVar.param("VALUE_TYPE", "FLOAT");
   metaVar.param("SIGNIFICANT_DIGITS", "8");
   metaVar.param("FILLVAL", "-3.4E38");
   metaVar.param("QUALITY", "0");
   metaVar.param("SCALEMIN", "1E0");
   metaVar.param("SCALEMAX", "1E2");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("LABLAXIS", "Char. energy");
   metaVar.paramQ("FIELDNAM", (char *)(fieldNameComp + ": Ne").toStdString().c_str());
   metaVar.end("Data_CharacteristicEnergy");

   return(_SUCCESS);
}

int WriteCef::writeMetaStatusVariables()				   
{
   WriteHeaderMetaVariable metaVar(&out);
   
   metaVar.start("Status_Quality_HEEA");
   metaVar.param("VALUE_TYPE", "INT");
   metaVar.paramQ("PARAMETER_TYPE", "Support_Data");
   metaVar.paramQ("CATDESC", "HEEA Status: Quality");
   metaVar.paramQ("UNITS", "Unitless");
   metaVar.paramQ("SI_CONVERSION", "1>unitless");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("SIGNIFICANT_DIGITS", "3");
   metaVar.param("FILLVAL", "-1");
   metaVar.param("QUALITY", "0");
   metaVar.param("SCALEMIN", "0");
   metaVar.param("SCALEMAX", "255");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("DEPEND_0", "time_tags");
   metaVar.paramQ("LABLAXIS", "HEEA Status: Quality");
   metaVar.paramQ("FIELDNAM", "HEEA Status: Quality");
   metaVar.end("Status_Quality_HEEA");
   
   metaVar.start("Status_CountStats_HEEA");
   metaVar.param("VALUE_TYPE", "INT");
   metaVar.paramQ("PARAMETER_TYPE", "Support_Data");
   metaVar.paramQ("CATDESC", "HEEA Status: Count Statistics");
   metaVar.paramQ("UNITS", "Unitless");
   metaVar.paramQ("SI_CONVERSION", "1>unitless");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("SIGNIFICANT_DIGITS", "3");
   metaVar.param("FILLVAL", "-1");
   metaVar.param("QUALITY", "0");
   metaVar.param("SCALEMIN", "0");
   metaVar.param("SCALEMAX", "255");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("DEPEND_0", "time_tags");
   metaVar.paramQ("LABLAXIS", "HEEA Status: Count Statistics");
   metaVar.paramQ("FIELDNAM", "HEEA Status: Count Statistics");   
   metaVar.end("Status_CountStats_HEEA");
   
   metaVar.start("Status_DatastreamErrors_HEEA");
   metaVar.param("VALUE_TYPE", "INT");
   metaVar.paramQ("PARAMETER_TYPE", "Support_Data");
   metaVar.paramQ("CATDESC", "HEEA Status: Datastream Errors");
   metaVar.paramQ("UNITS", "Unitless");
   metaVar.paramQ("SI_CONVERSION", "1>unitless");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("SIGNIFICANT_DIGITS", "3");
   metaVar.param("FILLVAL", "-1");
   metaVar.param("QUALITY", "0");
   metaVar.param("SCALEMIN", "0");
   metaVar.param("SCALEMAX", "255");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("DEPEND_0", "time_tags");
   metaVar.paramQ("LABLAXIS", "HEEA Status: Datastream Errors");
   metaVar.paramQ("FIELDNAM", "HEEA Status: Datastream Errors");
   metaVar.end("Status_DatastreamErrors_HEEA");

   metaVar.start("Status_Quality_LEEA");
   metaVar.param("VALUE_TYPE", "INT");
   metaVar.paramQ("PARAMETER_TYPE", "Support_Data");
   metaVar.paramQ("CATDESC", "LEEA Status: Quality");
   metaVar.paramQ("UNITS", "Unitless");
   metaVar.paramQ("SI_CONVERSION", "1>unitless");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("SIGNIFICANT_DIGITS", "3");
   metaVar.param("FILLVAL", "-1");
   metaVar.param("QUALITY", "0");
   metaVar.param("SCALEMIN", "0");
   metaVar.param("SCALEMAX", "255");
   metaVar.param("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("DEPEND_0", "time_tags");
   metaVar.paramQ("LABLAXIS", "LEEA Status: Quality");
   metaVar.paramQ("FIELDNAM", "LEEA Status: Quality");
   metaVar.end("Status_Quality_LEEA");
   
   metaVar.start("Status_CountStats_LEEA");
   metaVar.param("VALUE_TYPE", "INT");
   metaVar.paramQ("PARAMETER_TYPE", "Support_Data");
   metaVar.paramQ("CATDESC", "LEEA Status: Count Statistics");
   metaVar.paramQ("UNITS", "Unitless");
   metaVar.paramQ("SI_CONVERSION", "1>unitless");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("SIGNIFICANT_DIGITS", "3");
   metaVar.param("FILLVAL", "-1");
   metaVar.param("QUALITY", "0");
   metaVar.param("SCALEMIN", "0");
   metaVar.param("SCALEMAX", "255");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("DEPEND_0", "time_tags");
   metaVar.paramQ("LABLAXIS", "LEEA Status: Count Statistics");
   metaVar.paramQ("FIELDNAM", "LEEA Status: Count Statistics");   
   metaVar.end("Status_CountStats_LEEA");
   
   metaVar.start("Status_DatastreamErrors_LEEA");
   metaVar.param("VALUE_TYPE", "INT");
   metaVar.paramQ("PARAMETER_TYPE", "Support_Data");
   metaVar.paramQ("CATDESC", "LEEA Status: Datastream Errors");
   metaVar.paramQ("UNITS", "Unitless");
   metaVar.paramQ("SI_CONVERSION", "1>unitless");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("SIGNIFICANT_DIGITS", "3");
   metaVar.param("FILLVAL", "-1");
   metaVar.param("QUALITY", "0");
   metaVar.param("SCALEMIN", "0");
   metaVar.param("SCALEMAX", "255");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("DEPEND_0", "time_tags");
   metaVar.paramQ("LABLAXIS", "LEEA Status: Datastream Errors");
   metaVar.paramQ("FIELDNAM", "LEEA Status: Datastream Errors");
   metaVar.end("Status_DatastreamErrors_LEEA");

   metaVar.start("Status_Eclipse");
   metaVar.param("VALUE_TYPE", "INT");
   metaVar.paramQ("PARAMETER_TYPE", "Support_Data");
   metaVar.paramQ("CATDESC", "Status: Eclipse");
   metaVar.paramQ("UNITS", "Unitless");
   metaVar.paramQ("SI_CONVERSION", "1>unitless"); 
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("SIGNIFICANT_DIGITS", "3");
   metaVar.param("FILLVAL", "-1");
   metaVar.param("QUALITY", "0");
   metaVar.param("SCALEMIN", "0");
   metaVar.param("SCALEMAX", "255");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("DEPEND_0", "time_tags");
   metaVar.paramQ("LABLAXIS", "Status: Eclipse");
   metaVar.paramQ("FIELDNAM", "Status: Eclipse");
   metaVar.end("Status_Eclipse");

   metaVar.start("Status_InterferenceFromEFW");
   metaVar.param("VALUE_TYPE", "INT");
   metaVar.paramQ("PARAMETER_TYPE", "Support_Data");
   metaVar.paramQ("CATDESC", "Status: Interference from EFW");
   metaVar.paramQ("UNITS", "Unitless");
   metaVar.paramQ("SI_CONVERSION", "1>unitless");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("SIGNIFICANT_DIGITS", "3");
   metaVar.param("FILLVAL", "-1");
   metaVar.param("QUALITY", "0");
   metaVar.param("SCALEMIN", "0");
   metaVar.param("SCALEMAX", "255");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("DEPEND_0", "time_tags");
   metaVar.paramQ("LABLAXIS", "Status: Interference from EFW");
   metaVar.paramQ("FIELDNAM", "Status: Interference from EFW");
   metaVar.end("Status_InterferenceFromEFW");

   metaVar.start("Status_InterferenceFromWHI");
   metaVar.param("VALUE_TYPE", "INT");
   metaVar.paramQ("PARAMETER_TYPE", "Support_Data");
   metaVar.paramQ("CATDESC", "Status: Interference from WHI");
   metaVar.paramQ("UNITS", "Unitless");
   metaVar.paramQ("SI_CONVERSION", "1>unitless");
   metaVar.paramQ("FRAME", "scalar>na");
   metaVar.param("SIZES", "1");
   metaVar.param("SIGNIFICANT_DIGITS", "3");
   metaVar.param("FILLVAL", "-1");
   metaVar.param("QUALITY", "0");
   metaVar.param("SCALEMIN", "0");
   metaVar.param("SCALEMAX", "255");
   metaVar.paramQ("SCALETYP", "Linear");
   metaVar.paramQ("DISPLAYTYPE", "Time_Series");
   metaVar.paramQ("DEPEND_0", "time_tags"); 
   metaVar.paramQ("LABLAXIS", "Status: Interference from WHI");
   metaVar.paramQ("FIELDNAM", "Status: Interference from WHI");
   metaVar.end("Status_InterferenceFromWHI");

      
   return(_SUCCESS);
}

int WriteCef::writeMetaMomentsSetup(QString id1, QString id2, int sensorCombination)
{

   QString qSt, qStSc, qStD;
   if (id1.isEmpty() && !id2.isEmpty())
   {
      qSt = id2.mid(0, 2) + " ";
      QStringList qStSplit = id2.split("_");
      qStD = qStSplit.at(3);
   }
   else if (!id1.isEmpty() && id2.isEmpty())
   {
      qSt = id1.mid(0, 2) + " ";
      QStringList qStSplit = id1.split("_");
      qStD = qStSplit.at(3);
   }
   else if (!id1.isEmpty() && !id2.isEmpty())
   {
      qSt = id1.mid(0, 2) + " ";
      QStringList qStSplit1 = id1.split("_");
      QStringList qStSplit2 = id2.split("_");
      qStD = qStSplit1.at(3) + " + " + qStSplit2.at(3);
   }

   qStSc = sensorCombinationToStr(sensorCombination);

   qSt = qSt + "PEACE " + qStSc + " [" + qStD + "] Moments";   
   fieldNameComp = qStSc + " [" + qStD + "]";   

   if (id1.contains("PITCH_FULL"))
   {
      qSt = "PEACE 2-sec Moments (from PITCH_FULL)";
      fieldNameComp = "2-sec Moments (from PITCH_FULL)";
   }
   else if (id1.contains("PITCH_3D"))
   {
      qSt = "PEACE 2-sec Moments (from PITCH_3D)";
      fieldNameComp = "2-sec Moments (from PITCH_3D)";
   }

   out << "START_META = DATASET_TITLE" << endl;
   out << "   ENTRY = \"" << (qSt.toStdString()).c_str() << "\"" << endl;
   out << "END_META = DATASET_TITLE" << endl;
   out << "!" << endl;
   return(_SUCCESS);                                                       
}  



