/*******************************************************************************
 **
 **  $Id: abstractreader.h 158 2008-11-29 04:53:38Z AndrewLahiff $
 **
 ******************************************************************************
 **
 ** Copyright (C) 2007-2008 Andrew Lahiff.
 **
 ** This file is part of ReadFile.
 **
 ** ReadFile is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** ReadFile is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with ReadFile.  If not, see <http://www.gnu.org/licenses/>.
 **
 *******************************************************************************/

#ifndef ABSTRACTREADER_H_
#define ABSTRACTREADER_H_

#include "readfile_defs.h"

#include <QString>

class AbstractReader
{
   public:
      virtual ~AbstractReader() {};

      virtual int open(QString) = 0;                       // open a file
      virtual int readHeader() = 0;                        // read the header
      virtual int readRecord(bool) = 0;                    // read a data record
      virtual void close() = 0;                            // close the file

      virtual QString getGlobalAttr(QString) = 0;          // get a global attribute
      virtual QString getVariableAttr(int, QString) = 0;   // get a variable attribute

      virtual int getDataValue(int, short int*) = 0;       // get data (short int)
      virtual int getDataValue(int, int*)  = 0;            // get data (int)
      virtual int getDataValue(int, float*) = 0;           // get data (float)
      virtual int getDataValue(int, double*) = 0;          // get data (double)
      virtual int getDataValue(int, QString*) = 0;         // get data (QString)

      virtual int getNumVariables() = 0;                   // get the number of variables
      virtual int getVariableDimension(int) = 0;           // get the number of dimensions
      virtual int getVariableSize(int, int) = 0;           // get the size of the specified dimension
      virtual DataType getVariableDataType(int) = 0;       // get the variable data type

      virtual QString getFileBeginDateTime() = 0;          // get file begin date/time
      virtual QString getFileEndDateTime() = 0;            // get file end date/time

   private:

};

#endif // ABSTRACTREADER_H_
