/*******************************************************************************
 **
 ** $Id: readfile.cpp 158 2008-11-29 04:53:38Z AndrewLahiff $
 **
 ****************************************************************************** 
 ** 
 ** Copyright (C) 2007-2008 Andrew Lahiff.
 ** 
 ** This file is part of ReadFile.
 **
 ** ReadFile is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 ** 
 ** ReadFile is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 ** 
 ** You should have received a copy of the GNU General Public License
 ** along with ReadFile.  If not, see <http://www.gnu.org/licenses/>.
 ** 
 *******************************************************************************/

#include "readfile.h"

//=============================================================================
//
// ReadFile class
//
//=============================================================================

//-----------------------------------------------------------------------------
//
// constructor
//
//-----------------------------------------------------------------------------

ReadFile::ReadFile()
{

}

//-----------------------------------------------------------------------------
//
// set filename and open file
//
//-----------------------------------------------------------------------------

int ReadFile::open(QString userFileName)
{
   if (userFileName.endsWith(".cef", Qt::CaseInsensitive)) thisFileType = CEF;
   else if (userFileName.endsWith(".cdf", Qt::CaseInsensitive)) thisFileType = CDF;
   else return(_READ_FAILED);

   QFile fileTest;
   if (!fileTest.exists(userFileName)) return(_READ_FAILED);

   if (thisFileType == CEF)
   {
      fileCef = new ReadCef();
      return(fileCef->open(userFileName));
   }
   else
   {
      fileCdf = new ReadCdf();
      return(fileCdf->open(userFileName));
   }
}

//-----------------------------------------------------------------------------
//
// read CDF header
//
//-----------------------------------------------------------------------------

int ReadFile::readHeader()
{
   if (thisFileType == CEF) return(fileCef->readHeader());
   else return(fileCdf->readHeader());
}

//-----------------------------------------------------------------------------
//
// return number of variables
//
//-----------------------------------------------------------------------------

int ReadFile::getNumVariables()
{
   if (thisFileType == CEF) return(fileCef->getNumVariables());
   else return(fileCdf->getNumVariables());
}

//-----------------------------------------------------------------------------
//
// return number of dimensions of variable
//
//-----------------------------------------------------------------------------

int ReadFile::getVariableDimension(int index)
{
   if (thisFileType == CEF) return(fileCef->getVariableDimension(index));
   else return(fileCdf->getVariableDimension(index));
}

//-----------------------------------------------------------------------------
//
// return size of a dimension of a variable
//
//-----------------------------------------------------------------------------

int ReadFile::getVariableSize(int index, int inIndex)
{
   if (thisFileType == CEF) return(fileCef->getVariableSize(index, inIndex));
   else return(fileCdf->getVariableSize(index, inIndex));
}

//-----------------------------------------------------------------------------
//
// return variable data type
//
//-----------------------------------------------------------------------------

DataType ReadFile::getVariableDataType(int index)
{
   if (thisFileType == CEF) return fileCef->getVariableDataType(index);
   else return fileCdf->getVariableDataType(index);
}

//-----------------------------------------------------------------------------
//
// get begin date/time
//
//-----------------------------------------------------------------------------

QString ReadFile::getFileBeginDateTime()
{
   if (thisFileType == CEF) return(fileCef->getFileBeginDateTime());
   else return(fileCdf->getFileBeginDateTime());
}

//-----------------------------------------------------------------------------
//
// get end date/time
//
//-----------------------------------------------------------------------------

QString ReadFile::getFileEndDateTime()
{
   if (thisFileType == CEF) return(fileCef->getFileEndDateTime());
   else return(fileCdf->getFileEndDateTime());
}

//-----------------------------------------------------------------------------
//
// read a data record
//
//-----------------------------------------------------------------------------

int ReadFile::readRecord(bool dU)
{
   if (thisFileType == CEF) return(fileCef->readRecord(dU));
   else return(fileCdf->readRecord(dU));
}

//-----------------------------------------------------------------------------
//
// get a data value from the current record (short int version)
//
//-----------------------------------------------------------------------------
   
int ReadFile::getDataValue(int index, short int *ptr)
{
   if (thisFileType == CEF) return(fileCef->getDataValue(index, ptr));
   else return(fileCdf->getDataValue(index, ptr));
}

//-----------------------------------------------------------------------------
//
// get a data value from the current record (integer version)
//
//-----------------------------------------------------------------------------

int ReadFile::getDataValue(int index, int *ptr)
{
   if (thisFileType == CEF) return(fileCef->getDataValue(index, ptr));
   else return(fileCdf->getDataValue(index, ptr));
}

//-----------------------------------------------------------------------------
//    
// get a data value from the current record (float version)
//    
//-----------------------------------------------------------------------------

int ReadFile::getDataValue(int index, float *ptr)
{     
   if (thisFileType == CEF) return(fileCef->getDataValue(index, ptr));
   else return(fileCdf->getDataValue(index, ptr));
}

//-----------------------------------------------------------------------------
//    
// get a data value from the current record (double version)
//    
//-----------------------------------------------------------------------------

int ReadFile::getDataValue(int index, double *ptr)
{     
   if (thisFileType == CEF) return(fileCef->getDataValue(index, ptr));
   else return(fileCdf->getDataValue(index, ptr));
}

//-----------------------------------------------------------------------------
//    
// get a data value from the current record (string version)
//    
//-----------------------------------------------------------------------------

int ReadFile::getDataValue(int index, QString *ptr)
{
   if (thisFileType == CEF) return(fileCef->getDataValue(index, ptr));
   else return(fileCdf->getDataValue(index, ptr));
}

//-----------------------------------------------------------------------------
//    
// get a data value from the current record (EPOCH)
//    
//-----------------------------------------------------------------------------

int ReadFile::getDataEpoch(int index, QString *ptr)
{
   if (thisFileType == CEF) return(fileCef->getDataValue(index, ptr));
   else return(fileCdf->getEpochValue(index, ptr));
}

//-----------------------------------------------------------------------------
//
// close CDF file
//
//-----------------------------------------------------------------------------

void ReadFile::close()
{
   if (thisFileType == CEF)
   {
      fileCef->close();
      delete fileCef;
   }
   else
   {
      fileCdf->close();
      delete fileCdf;
   }

   return;
}

//-----------------------------------------------------------------------------
//
// get a global attribute
//
//-----------------------------------------------------------------------------

QString ReadFile::getGlobalAttr(QString what)
{
   if (thisFileType == CEF) return(fileCef->getGlobalAttr(what));
   else return(fileCdf->getGlobalAttr(what));
}

//-----------------------------------------------------------------------------
//
// get a variable attribute
//
//-----------------------------------------------------------------------------

QString ReadFile::getVariableAttr(int index, QString what)
{
   if (thisFileType == CEF) return(fileCef->getVariableAttr(index, what));
   else return(fileCdf->getVariableAttr(index, what));
}

//-----------------------------------------------------------------------------
//
// get lower or upper deltas
//
//-----------------------------------------------------------------------------

int ReadFile::getDataValueDelta(int index, int which, float *ptr)
{
   if (thisFileType == CEF) return(fileCef->getDataValueDelta(index, which, ptr));
   else return(fileCdf->getDataValueDelta(index, which, ptr));
}

